/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.algorithm;

import java.util.Iterator;
import java.util.List;

/**
 * A special node that isn't active on its own, but that sends all received
 * messages to all neighbours except the sender. It can also have a special
 * graphical representation and it is not used for the node counts in the
 * statistics. Repeaters can e.g. be used to model a bus structure.
 * @author Jrg Rdenauer
 */
public class Repeater extends NodeAdapter {
  
  /** Type name of the class */
  public static final String TYPE_NAME = "Repeater";

  /**
   * Forwards the received message to all neighbours except the sender
   * @see fdda.algorithm.Node#messageReceived(fdda.algorithm.Node, fdda.algorithm.Message)
   */
  protected final void messageReceived(Node sender, Message msg) 
  throws WrongStateException {
    List neighbours = getNeighbours();
    for (Iterator it = neighbours.iterator(); it.hasNext(); ) {
      Node neighbour = (Node) it.next();
      if ((neighbour != sender) && canSendMessageTo(neighbour)) {
        Message message = createMessage(neighbour);
        message.copyContents(msg);
        message.send();
      }
    }
  }

  /* (non-Javadoc)
   * @see fdda.algorithm.Node#getType()
   */
  public final String getType() {
    return TYPE_NAME;
  }

}
