/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.core;

import fdda.core.demonstration.Demonstration;
import fdda.core.modelling.ModelManager;
import fdda.core.projects.ProjectManager;

import fdda.utils.ComponentNotInitializedDisaster;

/**
 * Component class for the core functions.
 * @author Jrg Rdenauer
 */
public abstract class Core {

  /** The concrete implementation */
  private static Core core = null;

  /**
   * Sets the concrete implementation
   * @param aCore the core object
   */
  public static final void setCoreImplementation(Core aCore) {
    core = aCore;
  }

  /**
   * Returns the model manager of the core component
   * @return the model manager
   */
  public static final ModelManager getModelManager() {
    if (core == null) {
      throw new ComponentNotInitializedDisaster("Core");
    }
    return core.internalGetModelManager();
  }
  
  /**
   * Returns the demonstration manager implementation
   * 
   * @return the demonstration manager
   */
  public static final Demonstration getDemonstration() {
    if (core == null) {
      throw new ComponentNotInitializedDisaster("Core");
    }
    return core.internalGetDemonstration();
  }
  
  /**
   * Returns the project manager implementation
   * 
   * @return the project manager
   */
  public static final ProjectManager getProjectManager() {
    if (core == null) {
      throw new ComponentNotInitializedDisaster("Core");
    }
    return core.internalGetProjectManager();
  }

  /**
   * Returns the model manager
   * @return the model manager
   */
  protected abstract ModelManager internalGetModelManager();
  
  /**
   * Returns the demonstration manager
   * @return the demonstration manager
   */
  protected abstract Demonstration internalGetDemonstration();
  
  /**
   * Returns the project manager
   * @return the project manager
   */
  protected abstract ProjectManager internalGetProjectManager();
}