/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.core.demonstration;

/**
 * Contains statistics about a demonstration - either currently running or 
 * already run and stopped.
 * @author Jrg Rdenauer
 */
public interface Statistics {

  /**
   * Returns the number of sent messages
   * @return # sent messages
   */
  int getSentMessages();
  /**
   * Returns the number of received messages
   * @return # received messages
   */
  int getReceivedMessages();
  
  /**
   * Returns the number of started nodes
   * @return # started nodes
   */
  int getStartedNodes();
  /**
   * Returns the number of nodes that were active at one point of time
   * @return # active nodes
   */
  int getActiveNodes();
  
  /**
   * Returns the number of nodes that are currently active
   * @return # currently active nodes
   */
  int getCurrentlyActiveNodes();
  /**
   * Returns the number of nodes that are currently paused
   * @return # paused nodes
   */
  int getCurrentlyPausedNodes();
  /**
   * Returns the number of messages that are currently active
   * @return # active messages
   */
  int getCurrentlyActiveMessages();
  
  /**
   * Returns the number of nodes that were used (either started, or active, or 
   * received a message)
   * @return # used nodes
   */
  int getUsedNodes();
  /**
   * Returns the number of nodes in the project
   * @return # present nodes
   */
  int getPresentNodes();
  /**
   * Returns the number of connections that were used
   * @return # connections
   */
  int getUsedConnections();
  /**
   * Returns the number of connections in the project
   * @return # present connections
   */
  int getPresentConnections();
  
  /**
   * Returns the number of node failures (either not working at start or broken 
   * down during demo)
   * @return # node failures
   */
  int getNodeFailures();
  /**
   * Returns the number of connection failures (either not working at start or 
   * broken down during demo)
   * @return # connection failures
   */
  int getConnectionFailures();
  /**
   * Returns the number of nodes that recovered during the demo
   * @return # node recoveries
   */
  int getNodeRecoveries();
  /**
   * Returns the number of connections that recovered during the demo
   * @return # connection recoveries
   */
  int getConnectionRecoveries();
}
