/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.core.impl;

import fdda.core.Core;
import fdda.core.demonstration.Demonstration;
import fdda.core.demonstration.Demonstrator;
import fdda.algorithm.WrongStateException;

import fdda.core.projects.Project;
import fdda.dialog.GraphicsControl;

import fdda.utils.Notification;

import java.util.HashMap;

/**
 * Implementation of the demonstration manager. 
 * @author Jrg Rdenauer
 */
public class DemonstrationImpl implements Demonstration {
  
  /** the demonstrators */
  private HashMap demonstrators = null;
  
  /**
   * Constructor
   */
  public DemonstrationImpl() {
    demonstrators = new HashMap();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.demonstration.Demonstration#start(fdda.core.projects.Project)
   */
  public synchronized void start(Project project, 
                                 GraphicsControl graphicsControl) 
  throws WrongStateException {
    DemonstratorImpl demonstrator = null;
    if (demonstrators.containsKey(project)) {
      demonstrator = (DemonstratorImpl) demonstrators.get(project);
      if (!demonstrator.isStopped()) {
        throw new WrongStateException("Already started!");
      }
    }
    else {
      demonstrator = new DemonstratorImpl(project, graphicsControl);
      demonstrators.put(project, demonstrator);
    }
    Notification.indicateOccupationState();
    demonstrator.reset();
    Core.getProjectManager().setDemoStateForActiveProject();
    demonstrator.start();
    Notification.indicateNormalState();
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.demonstration.Demonstration#pause(fdda.core.projects.Project)
   */
  public synchronized void pause(Project project) throws WrongStateException {
    if (!demonstrators.containsKey(project) || 
        (!((Demonstrator) demonstrators.get(project)).isRunning() &&
         !((Demonstrator) demonstrators.get(project)).isPaused())) {
      throw new WrongStateException("Project isn't in demo mode!");
    }
    DemonstratorImpl demo = (DemonstratorImpl) demonstrators.get(project);
    if (demo.isPaused()) {
      demo.endPause();
    }
    else {
      demo.pause();
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.demonstration.Demonstration#stop(fdda.core.projects.Project)
   */
  public synchronized void stop(Project project) throws WrongStateException {
    if (!demonstrators.containsKey(project) || 
        (!((Demonstrator) demonstrators.get(project)).isRunning() &&
         !((Demonstrator) demonstrators.get(project)).isPaused())) {
      throw new WrongStateException("Project isn't in demo mode!");      
    }
    ((DemonstratorImpl) demonstrators.get(project)).stop();
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.demonstration.Demonstration#wasDemonstrated(fdda.core.projects.Project)
   */
  public synchronized boolean wasDemonstrated(Project project) {
    return (demonstrators.containsKey(project));
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.demonstration.Demonstration#getDemonstrator(fdda.core.projects.Project)
   */
  public synchronized Demonstrator getDemonstrator(Project project) 
  throws WrongStateException {
    if (!demonstrators.containsKey(project)) {
      throw new WrongStateException("Project wasn't demonstrated yet!");
    }
    return (Demonstrator) demonstrators.get(project);
  }
  
}
