/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.core.impl;

import fdda.core.modelling.ModelManager;

import fdda.core.projects.Project;

import fdda.graphics.Line;
import fdda.graphics.Node;

/**
 * Implementation of the model manager.
 * @author Jrg Rdenauer
 */
public class ModellingManagerImpl implements ModelManager {
  
  /** the next ID */
  private int nextID;
  
  /**
   * Constructor
   */
  public ModellingManagerImpl() {
    nextID = 1;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.modelling.ModelManager#getNextID()
   */
  public int getNextID() {
    int result = nextID;
    nextID++;
    return result;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.modelling.ModelManager#addNode(fdda.core.projects.Project, fdda.graphics.Node)
   */
  public void addNode(Project project, Node graphicalRepresentation) {
    fdda.model.Node modelNode = new fdda.model.impl.Node(getNextID(), project, 
      graphicalRepresentation);
    modelNode.initialize();
    graphicalRepresentation.setModelNode(modelNode);
    project.addNode(modelNode);
    project.setChanged(true);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.modelling.ModelManager#removeNode(fdda.core.projects.Project, fdda.model.Node)
   */
  public void removeNode(Project project, fdda.model.Node node) {
    if (node.getConnections().isEmpty()) {
      project.removeNode(node);
      project.setChanged(true);
    }
    else {
      throw new IllegalArgumentException("Node still has connections!");
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.modelling.ModelManager#addConnection(fdda.core.projects.Project, fdda.model.Node, fdda.model.Node, fdda.graphics.Line, boolean)
   */
  public void addConnection(Project project, 
                             fdda.model.Node startNode, fdda.model.Node endNode,
                             Line graphicalRepresentation, boolean directed) {
    fdda.model.Connection connection = 
      new fdda.model.impl.Connection(getNextID(), directed, 
        startNode, endNode, graphicalRepresentation);
    graphicalRepresentation.setConnection(connection);
    startNode.addConnection(connection);
    endNode.addConnection(connection);
    project.addConnection(connection);
    project.setChanged(true);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.modelling.ModelManager#removeConnection(fdda.core.projects.Project, fdda.model.Connection)
   */
  public void removeConnection(Project project, 
                               fdda.model.Connection connection) {
    connection.getStartNode().removeConnection(connection);
    connection.getEndNode().removeConnection(connection);
    project.removeConnection(connection);
    project.setChanged(true);
  }

}
