/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.core.impl;

import fdda.algorithm.Algorithm;
import fdda.graphics.GraphicalSystem;

import fdda.model.Node;
import fdda.model.Connection;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
// import java.util.Date;

/**
 * Implementation of the project interface.
 * @author Jrg Rdenauer
 */
class Project implements fdda.core.projects.Project {
  
  /** the name of the project */
  private String name = "";
  
  /** the corresponding graphical system */
  private GraphicalSystem graphicalSystem = null;
  
  /** the nodes in the project */
  private HashMap nodes = new HashMap(30);
  /** the connections in the project */
  private HashSet connections = new HashSet(90);
  
  /** the algorithm demonstrated by the project */
  private Algorithm algorithm;

  /** whether the project was changed since the last save operation */
  private boolean changed;

  /** the storage location of the project (if known) */
  private String location = null;

  /** the state of the project */
  private int projectState;
  
  /** whether project-specific options are used */
  // private boolean optionsUsed;

  /**
   * Constructor
   */
  public Project() {
    setLocation(null);
    //this.options = new HashMap();
    //this.optionsUsed = true;
    this.setState(STATE_MOD);
    // this.updateTimeStamp();
    setAlgorithm(Algorithm.getDefaultAlgorithm());
    setChanged(false);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#setGraphicalSystem(fdda.graphics.GraphicalSystem)
   */
  public void setGraphicalSystem(GraphicalSystem system) {
    this.graphicalSystem = system;
    system.setProject(this);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#getGraphicalSystem()
   */
  public GraphicalSystem getGraphicalSystem() {
    return graphicalSystem;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#getName()
   */
  public String getName() {
    return name;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#getLocation()
   */
  public String getLocation() {
    return location;
  }

  /**
   * Sets the name of the project
   * @param name the name
   */
  public void setName(String name) {
    this.name = name;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#setLocation(java.lang.String)
   */
  public void setLocation(String location) {
    this.location = location;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#isLocationKnown()
   */
  public boolean isLocationKnown() {
    return location != null;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#getState()
   */
  public int getState() {
    return projectState;
  }
  
  /**
   * Sets the current state of the project (MOD or DEMONSTRATION)
   * @param projectState the state
   */
  public void setState(int projectState){
    this.projectState = projectState;
  }
  
  /*
  public void updateTimeStamp() {
    Date curTime = new Date();
    this.setOption("project.TimeStamp", Long.toString(curTime.getTime()),
      false, true);
  }
  */
  
  /**
   * Returns a project-specific option.
   * @param name the name of the option
   * @return the value of the option
   */
  /*
  public String getOption(String name) {
    Object option = options.get(name);
    if (option == null) {
      return null;
    }
    ProjectOption projectOption = (ProjectOption) option;
    if (optionsUsed || projectOption.dominant) {
      return projectOption.value;
    }
    else {
      return null;
    }
  }
  */

  /**
   * Sets a project-specific option. The option is persistent, but not dominant.
   * @param name the name of the option
   * @param value the value of the option
   */
  /*
  public void setOption(String name, String value) {
    options.put(name, new ProjectOption(name, value, true, false));
  }
  */

  /**
   * Sets a project-specific option. The option is not dominant.
   * @param name the name of the option
   * @param value the value of the option
   * @param persistent whether the option shall be persistent
   */
  /*
  public void setOption(String name, String value, boolean persistent) {
    options.put(name, new ProjectOption(name, value, persistent, false));
  }
  */

  /**
   * Sets a project-specific option.
   *  
   * @param name the name of the option
   * @param value the value of the option
   * @param persistent whether the option shall be persistent
   * @param dominant whether the option shall be dominant (used even if 
   * default options are used normally)
   */
  /*
  public void setOption(String name, String value, boolean persistent,
                        boolean dominant) {
    options.put(name, new ProjectOption(name, value, persistent, dominant));
  }
  */

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#areOptionsUsed()
   */
  /*  
  public boolean areOptionsUsed() {
    return optionsUsed;
  }
  */

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#setOptionsUsed(boolean)
   */
  /*
  public void setOptionsUsed(boolean used) {
    this.optionsUsed = used;
  }
  */
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#addNode(fdda.model.Node)
   */
  public void addNode(Node node) {
    nodes.put(new Integer(node.getNr()), node);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#removeNode(fdda.model.Node)
   */
  public void removeNode(Node node) {
    nodes.remove(new Integer(node.getNr()));    
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#nodes()
   */
  public Iterator nodes() {
    return nodes.values().iterator();
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#addConnection(fdda.model.Connection)
   */
  public void addConnection(Connection connection) {
    connections.add(connection);    
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#removeConnection(fdda.model.Connection)
   */
  public void removeConnection(Connection connection) {
    connections.remove(connection);    
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#connections()
   */
  public Iterator connections() {
    return connections.iterator();
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#clear()
   */
  public void clear() {
    nodes.clear();
    connections.clear();
    changed = true;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#wasChanged()
   */  
  public boolean wasChanged() {
    return changed;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#setChanged(boolean)
   */
  public void setChanged(boolean changed) {
    this.changed = changed;
    if (!changed && graphicalSystem != null) graphicalSystem.setChanged(false);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#getAlgorithm()
   */  
  public Algorithm getAlgorithm() {
    return algorithm;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.Project#setAlgorithm(fdda.algorithm.Algorithm)
   */
  public void setAlgorithm(Algorithm algorithm) {
    this.algorithm = algorithm;
    // set all nodes initially to the default type
    String defaultType = algorithm.getDefaultType();
    for (Iterator it = nodes(); it.hasNext(); ) {
      Node node = (Node) it.next();
      if (!(node.isRepeater())) {
        node.getProperty(Node.NODE_TYPE_PROPERTY_NAME).setValueAt(
          0, defaultType);
      }
    }
  }
  
  /** the project-specific options */
  // private HashMap options;

  /**
   * Stores a project-speicfic option. Options can be persistent (stored 
   * together with the project). They can be dominant (used even if 
   * project-specific options are not used normally).
   */
  //private class ProjectOption {

    /** name of the option */
    //private String name;

    /** value of the option */
    //private String value;

    /** whether the option is persistent */
    //private boolean persistent;
  
    /** whether the option is dominant */
    //private boolean dominant;
    
    /**
     * Constructor
     * 
     * @param name the name
     * @param value the value
     * @param persistent shall the option be persistent?
     * @param dominant shall the option be dominant?
     */
    /*
    public ProjectOption(String name, String value, boolean persistent,
                         boolean dominant) {
      this.name = name;
      this.value = value;
      this.persistent = persistent;
      this.dominant = dominant;
    }
    */
  //}
  
}