/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.core.modelling;

import fdda.model.*;
import fdda.core.projects.Project;

/**
 * Interface for the class that manages the model.
 * @author Jrg Rdenauer
 */
public interface ModelManager {

  /**
   * Returns the next ID for a node or a connection. IDs must be globally 
   * unique.
   * @return the next ID
   */
  int getNextID();  

  /**
   * Adds a node to a project.
   * 
   * @param project the project
   * @param graphicalRepresentation the graphical representation of the node
   */
  void addNode(Project project, fdda.graphics.Node graphicalRepresentation);

  /**
   * Removes a node from the project. The node mustn't be connected to any other
   * nodes.
   * 
   * @param project the project
   * @param node the node
   * @throws IllegalArgumentException if there are still connections at this 
   * node
   */
  void removeNode(Project project, Node node);

  /**
   * Adds a connection to a project
   * 
   * @param project the project
   * @param startNode the first node
   * @param endNode the second node
   * @param graphicalRepresentation the graphical representation of the 
   * connection (i.e. a line)
   * @param directed whether the connection shall be directed
   */
  void addConnection(Project project, Node startNode, Node endNode,
                     fdda.graphics.Line graphicalRepresentation,
                     boolean directed);

  /**
   * Removes a connection from a project
   * 
   * @param project the project
   * @param connection the connection
   */
  void removeConnection(Project project, Connection connection);
}