/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog;

import fdda.algorithm.Algorithms;
import fdda.core.Core;

import fdda.utils.Notification;

import java.util.Set;
import java.util.Iterator;

/**
 * Controls the algorithm dialog
 * @author Jrg Rdenauer
 */
public class AlgorithmDialogControl implements DialogControl {
  
  /** the data */
  private AlgorithmData data;
  /** the dialog */
  private Dialog dialog;
  /** the old algorithm, to detect changes */
  private String oldAlgorithm;
  
  /**
   * Constructor
   * @param dialog the dialog
   */
  public AlgorithmDialogControl(Dialog dialog) {
    this.dialog = dialog;
    oldAlgorithm = 
      Core.getProjectManager().getActiveProject().getAlgorithm().getName();
    this.data = new AlgorithmData();
    data.setCurrentAlgorithm(oldAlgorithm);
    Set knownAlgorithms = Algorithms.getInstance().getAvailableAlgorithms();
    String[] names = new String[knownAlgorithms.size()];
    String[] descriptions = new String[names.length];
    int i = 0;
    for (Iterator it = knownAlgorithms.iterator(); it.hasNext(); ++i) {
      names[i] = (String) it.next();
      descriptions[i] = 
        Algorithms.getInstance().getAlgorithm(names[i]).getDescription();
    }
    data.setAlgorithmDescriptions(descriptions);
    data.setKnownAlgorithms(names);
    dialog.updateObject(data);
    
  }

  /**
   * Always throws exception, there are no commands
   * @see fdda.dialog.DialogControl#executeCommand(int)
   */
  public void executeCommand(int command) throws NoSuchCommandException {
    throw new NoSuchCommandException();
  }

  /* (non-Javadoc)
   * @see fdda.dialog.DialogControl#canClose(boolean)
   */
  public boolean canClose(boolean closedByOK) {
    if (closedByOK) {
      String newAlgorithm = data.getCurrentAlgorithm();
      if (!(newAlgorithm.equals(oldAlgorithm))) {
        int retCode = Notification.message(Notification.M_CONFIRM, 
          Notification.S_MOD, "Algorithmus", "Hiermit ndern alle Knoten ihren"
          + " Typ. Sind Sie sicher?", new String[] {"Ja", "Nein"});
        if (retCode == 0) {
          Core.getProjectManager().getActiveProject().setAlgorithm(
            Algorithms.getInstance().getAlgorithm(newAlgorithm));
        }
        else {
          return false;
        }
      }
    }
    return true;
  }

  /**
   * Returns null, there can be no errors
   * @see fdda.dialog.DialogControl#getErrors()
   */
  public String getErrors() {
    return null;
  }

}
