/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog;

/**
 * Generic interface for dialogs.
 * @author Jrg Rdenauer
 */
public interface Dialog {
  
  /**
   * Updates the dialog with the given data (changes displayed values)
   * @param data the data
   */
  void updateObject(Object data);
  
  /**
   * Returns the currently selected or entered data
   * @return the data
   */
  Object getData();
  
  /**
   * Disables the execution of a command (e.g. disables a button)
   * @param command the command ID
   * @throws NoSuchCommandException if no such command is supported by the 
   * dialog
   */
  void disableCommand(int command) throws NoSuchCommandException;
  
  /**
   * Enables the execution of a command (e.g. enables a button)
   * @param command the command ID
   * @throws NoSuchCommandException if no such command is supported by the 
   * dialog
   */  
  void enableCommand(int command) throws NoSuchCommandException;
}
