/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog;

/**
 * Generic interface for classes that control dialogs. 
 * @author Jrg Rdenauer
 */
public interface DialogControl {

  /**
   * Executes a command (e.g. the user pressed a button).
   * @param command the command ID
   * @throws NoSuchCommandException if no such command is supported by this 
   * dialog control
   */
  void executeCommand(int command) throws NoSuchCommandException;
  
  /**
   * Returns whether the dialog can be closed. If parameter is true, checks 
   * should be performed whether the current data is valid. If the data is valid
   * and the dialog can be closed, processing may occur before the dialog really
   * closes. To avoid awkward situations, consider processing in an extra 
   * thread.
   * @param closedByOK whether the user closed the dialog with a confirming
   * choice
   * @return whether the dialog can be closed
   */
  boolean canClose(boolean closedByOK);
  
  /**
   * If the dialog can't be closed because data is not valid, this method 
   * returns an error message for the user.
   * @return error message
   */
  String getErrors();

}
