/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog;

import fdda.core.demonstration.Statistics;
import fdda.utils.ComponentNotInitializedDisaster;

/**
 * Factory and interface of the dialog component. Also provides the graphics
 * control component. For different types of dialogs or graphics, different
 * factories may be used.
 * @author Jrg Rdenauer
 */
public abstract class DialogFactory {
    
    /** the instance (singleton) */
    private static DialogFactory theFactory = null;
    
    /**
     * Sets the factory and initializes the component
     * @param factory the factory
     */
    public static final void setFactory(DialogFactory factory) {
        theFactory = factory;
    }
    
    /**
     * Returns the factory.
     * @return the factory
     */
    public static final DialogFactory getInstance() {
      if (theFactory == null) {
        throw new ComponentNotInitializedDisaster("DialogFactory");
      }
      return theFactory;
    }
    
    /**
     * Creates an instance of a graphics control object.
     * @return graphics control
     */
    public abstract GraphicsControl createGraphicsControl();
    
    /**
     * Returns a short description of the dialog component, e.g. 'swing'.
     * @return description
     */
    public abstract String getDialogComponent();
    
    /**
     * Returns a short description of the used graphics component, e.g. 
     * 'java2d'.
     * @return description
     */
    public abstract String getGraphicsComponent();
    
    /**
     * Displays the options dialog.
     */
    public abstract void showOptionsDialog();
    
    /**
     * Displays an instance of the statistics dialog.
     * @param statistics the statistics (data for the dialog)
     */
    public abstract void showStatisticsDialog(Statistics statistics);
    
    /**
     * Displays the algorithm selection dialog.
     */
    public abstract void showAlgorithmDialog();
    
    /**
     * Displays the about dialog.
     */
    public abstract void showAboutDialog();

}
