/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog;

import java.awt.Graphics;
import java.awt.image.BufferedImage;

/**
 * Interface of the component that manages all graphical operations while
 * modelling the system.
 * @author Jrg Rdenauer
 */
public interface GraphicsControl {

  /** operation: pointer */
  int POINTER = 1;

  /** operation: add a node */
  int NODE = 2;

  /** operation: add a line */
  int LINE = 3;

  /** operation: delete */
  int DELETE = 4;

  /** mode: editing allowed */
  int S_UNLOCKED = 17;

  /** mode: editing not allowed */
  int S_LOCKED = 19;

  /** operation: add a directed line */
  int DIRECTED_LINE = 5;

  /**
   * Zooms in. Needs not to be supported.
   */
  void zoomIn();

  /**
   * Zooms out. Needs not to be supported.
   */
  void zoomOut();

  /**
   * Selects all movable components.
   */
  void selectMovables();

  /**
   * Paints the graphics, using a specified Graphics object
   * @param g the graphics object
   */
  void paint(Graphics g);
  
  /**
   * Paints the graphics on an image.
   * @return the new image
   */
  BufferedImage paintOnImage();
  
  /**
   * Deletes selected components
   */
  void deleteSelection();

  /**
   * Changes the mode
   * @param mode the new mode (one of the constants)
   */
  void setEditingMode(int mode);

  /**
   * Returns the current mode
   * @return the current mode
   */
  int getEditingMode();

  /**
   * Selects an action for the user
   * @param action the new action (one of the constants)
   */
  void setSelectedAction(int action);

  /**
   * Returns the currently selected action
   * @return the action
   */
  int getSelectedAction();

  /**
   * Sets the graphical system that is managed
   * @param system the system
   */
  void setGraphicalSystem(fdda.graphics.GraphicalSystem system);
  
  /**
   * Returns the object that manages the graphical messages.
   * @return the message control object
   */
  MessageControl getMessageControl();
  
}