/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog;

import fdda.graphics.Message;

/**
 * Interface for the class that manages the graphical representations of the
 * messages. This may e.g. be of special importance regarding clipping.
 * @author Jrg Rdenauer
 */
public interface MessageControl {
  
  /**
   * Indicates that a new cycle of the demo has begun.
   */
  void startCycle();
  
  /**
   * Moves the given messages forward.
   * @param messages the messages
   */
  void moveMessages(java.util.Set messages);
  
  /**
   * Adds a message to the graphics
   * @param message the message
   */
  void addMessage(Message message);
  
  /**
   * Removes a message from the graphics
   * @param message the message
   */
  void removeMessage(Message message);
  
  /**
   * Removes all messages from the graphics
   */
  void removeAllMessages();

  /**
   * Indicates that a cycle of the demo has finished.
   */
  void cycleDone();
}
