/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog;

import java.util.Set;

/**
 * Class that hold the data for the options dialog.
 * @author Jrg Rdenauer
 */
public class Options {

  /** the current system reader plug-in */
  private String systemReader;
  /** the current system writer plug-in */
  private String systemWriter;
  /** the current algorithm reader plug-in*/
  private String algReader;
  /** the currently selected start action (constant) */
  private int startAction;
  /** the currently selected grid width */
  private int gridWidth;
  /** whether the application shall ask the user before it is closed */
  private boolean askAtClosing;
  /** all possible system reader plug-ins */
  private Set possibleReaders;
  /** all possible system writer plug-ins */
  private Set possibleWriters;
  /** all possible algorithm reader plug-ins */
  private Set possibleAlgReaders;
  /** speed of the messages */
  private String messageSpeed;
  /** minimum demonstration cycle time */
  private String minCycleTime;
  /** the browser command */
  private String browserCommand;
  
  
  /**
   * Returns the grid width
   * @return the grid width
   */
  public int getGridWidth() {
    return gridWidth;
  }

  /**
   * Returns the start action
   * @return the start action
   */
  public int getStartAction() {
    return startAction;
  }

  /**
   * Returns the system reader plug-in name
   * @return the system reader
   */
  public String getSystemReader() {
    return systemReader;
  }

  /**
   * Returns the algorithm reader plug-in name
   * @return the algorithm reader
   */
  public String getAlgorithmReader() {
    return algReader;
  }

  /**
   * Returns the system writer plug-in name
   * @return the system writer
   */
  public String getSystemWriter() {
    return systemWriter;
  }

  /**
   * Sets the grid width
   * @param i the grid width
   */
  public void setGridWidth(int i) {
    gridWidth = i;
  }

  /**
   * Sets the start action
   * @param i the start action
   */
  public void setStartAction(int i) {
    startAction = i;
  }

  /**
   * Sets the system reader plug-in name
   * @param string the system reader
   */
  public void setSystemReader(String string) {
    systemReader = string;
  }

  /**
   * Sets the algorithm reader plug-in name
   * @param string the algorithm reader
   */
  public void setAlgorithmReader(String string) {
    algReader = string;
  }

  /**
   * Sets the system writer plug-in name
   * @param string the system writer
   */
  public void setSystemWriter(String string) {
    systemWriter = string;
  }

  /**
   * Returns whether the application shall ask before close
   * @return whether the application shall ask
   */
  public boolean shallAskAtClosing() {
    return askAtClosing;
  }

  /**
   * Sets whether the application shall ask before close
   * @param b whether the application shall ask
   */
  public void setAskAtClosing(boolean b) {
    askAtClosing = b;
  }

  /**
   * Returns the possible system reader choices
   * @return the system readers
   */
  public Set getPossibleReaders() {
    return possibleReaders;
  }

  /**
   * Returns the possible algorithm reader choices
   * @return the algorithm readers
   */
  public Set getPossibleAlgorithmReaders() {
    return possibleAlgReaders;
  }

  /**
   * Sets the possible system writer choices
   * @return the possible system writers
   */
  public Set getPossibleWriters() {
    return possibleWriters;
  }

  /**
   * Sets the possible system reader choices
   * @param set the possible system readers
   */
  public void setPossibleReaders(Set set) {
    possibleReaders = set;
  }

  /**
   * Sets the possible algorithm reader choices
   * @param set the possible algorithm readers
   */
  public void setPossibleAlgorithmReaders(Set set) {
    possibleAlgReaders = set;
  }

  /**
   * Sets the possible system writer choices
   * @param set the possible system writers
   */
  public void setPossibleWriters(Set set) {
    possibleWriters = set;
  }

  /**
   * Returns the message speed
   * @return the message speed
   */
  public String getMessageSpeed() {
    return messageSpeed;
  }

  /**
   * Returns the minimum cycle time
   * @return the minimum cycle time
   */
  public String getMinCycleTime() {
    return minCycleTime;
  }

  /**
   * Sets the message speed
   * @param d the message speed
   */
  public void setMessageSpeed(String d) {
    messageSpeed = d;
  }

  /**
   * Sets the minimum cycle time
   * @param l the minimum cycle time
   */
  public void setMinCycleTime(String l) {
    minCycleTime = l;
  }

  /**
   * Returns the browser command
   * @return the browser command
   */
  public String getBrowserCommand() {
    return browserCommand;
  }

  /**
   * Sets the browser command
   * @param string the browser command
   */
  public void setBrowserCommand(String string) {
    browserCommand = string;
  }

}
