/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog;

import fdda.core.demonstration.Statistics;

/**
 * Controls the statistics dialog.
 * @author Jrg Rdenauer
 */
public class StatisticsDialogControl implements DialogControl {
  
  /** Command: refresh the statistics. Currently without action */
  public static final int COMMAND_REFRESH = 37;
  
  /** the statistics */
  private Statistics statistics;
  /** the dialog */
  private Dialog dialog;

  /**
   * Constructor
   * @param dialog the dialog
   * @param statistics the statistics
   */
  public StatisticsDialogControl(Dialog dialog, Statistics statistics) {
    this.statistics = statistics;
    this.dialog = dialog;
    dialog.updateObject(statistics);
  }

  /* (non-Javadoc)
   * @see fdda.dialog.DialogControl#executeCommand(int)
   */
  public void executeCommand(int command) throws NoSuchCommandException {
    if (command != COMMAND_REFRESH) {
      throw new NoSuchCommandException();
    }
    dialog.updateObject(statistics);
  }

  /**
   * Always returns true.
   * @see fdda.dialog.DialogControl#canClose(boolean)
   */
  public boolean canClose(boolean closedByOK) {
    return true; // can always close
  }

  /**
   * Returns null (no errors ever).
   * @see fdda.dialog.DialogControl#getErrors()
   */
  public String getErrors() {
    return null;
  }

}
