/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.java2d;

import java.util.prefs.Preferences;

import fdda.graphics.java2d.Painter;

/**
 * Abstract superclass of all manipulators.
 * @author Jrg Rdenauer
 */
public abstract class AbstractManipulator implements Manipulator {

  /** the graphics holder */
  protected GraphicHolder graphicHolder;

  /** the painter used for redrawing the components */
  protected Painter painter;

  /**
   * Constructor
   * @param graphicHolder the object holding the graphics
   */
  public AbstractManipulator(GraphicHolder graphicHolder) {
    this.setGraphicHolder(graphicHolder);
    this.painter = new Painter(graphicHolder);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#getGraphicHolder()
   */
  public GraphicHolder getGraphicHolder() {
    return graphicHolder;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#setGraphicHolder(fdda.dialog.java2d.GraphicHolder)
   */
  public void setGraphicHolder(GraphicHolder graphicHolder) {
    this.graphicHolder = graphicHolder;
  }

  /**
   * Changes a coordinate so that it lies on the grid
   * @param u the old coordinate
   * @param isX whether the coordinate is an x coordinate (else it's y)
   * @return the new coordinate on the grid
   */
  protected final double moveToGrid(double u, boolean isX) {
    long calc = Math.round(u);
    int gridWidth = 
      Preferences.userNodeForPackage(getClass()).getInt("GridXWidth", 10);
    long GRID_X_WIDTH = gridWidth;
    long GRID_Y_WIDTH = gridWidth;
    long GRID_W = isX ? GRID_X_WIDTH : GRID_Y_WIDTH;
    long diff = calc % GRID_W; // modulo operation
    // Only change if grid is visible
    if (Preferences.userRoot().node("fdda/graphics").
        getBoolean("showGrid", false)) {
      calc -= diff;
      // move to _nearest_ grid
      if (diff > GRID_W / 2) {
        calc += GRID_W;
      }
      assert calc % GRID_W == 0;
      return ((double) calc);
    }
    else {
      return u;
    }
  }

}