/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.java2d;

import fdda.graphics.java2d.GraphicalSystem;
import fdda.graphics.java2d.Painter;

import java.awt.Rectangle;

/**
 * Defines the interface of the object holding the graphics of a project.
 * @author Jrg Rdenauer
 */
public interface GraphicHolder {

  /**
   * Returns the graphical system
   * @return the graphical system
   */
  GraphicalSystem getGraphicalSystem();

  /**
   * Repaints a part of the screen (that should contain the graphics). Closely
   * resembles repaint of Component
   * @param clippingRect a rectangle defining the clipping area
   * @param painter the painter instance used for drawing
   * @see java.awt.Component#repaint()
   */
  void redraw(Rectangle clippingRect, Painter painter);

  /**
   * Returns the width of a string in the current graphics context.
   * @param s the string
   * @return the width of the string
   */
  double stringWidth(String s);

  /**
   * Returns the height of a (any) string in the current graphics context.
   * @return the string height
   * @see java.awt.FontMetrics#getHeight()
   */
  double stringHeight();
  
}