/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.java2d;

import fdda.core.demonstration.Statistics;
import fdda.dialog.AlgorithmDialogControl;
import fdda.dialog.DialogFactory;
import fdda.dialog.GraphicsControl;
import fdda.dialog.OptionsDialogControl;
import fdda.dialog.StatisticsDialogControl;

import fdda.dialog.swing.AlgorithmDialog;
import fdda.dialog.swing.OptionsDialog;
import fdda.dialog.swing.StatisticsDialog;
import fdda.dialog.swing.AboutDialog;
import fdda.dialog.swing.WindowManager;

/**
 * Factory for the java2d and swing components.
 * @author Jrg Rdenauer
 */
public class Java2DFactory extends DialogFactory {

 /*
  *  (non-Javadoc)
  * @see fdda.dialog.DialogFactory#createGraphicsControl()
  */    
 public GraphicsControl createGraphicsControl() {
    return new Java2DControl(); 
  }
  
  /**
   * Returns "swing".
   * @see fdda.dialog.DialogFactory#getDialogComponent()
   */
  public String getDialogComponent() {
    return "swing";
  }
  
  /**
   * Returns "java2d".
   * @see fdda.dialog.DialogFactory#getGraphicsComponent()
   */
  public String getGraphicsComponent() {
    return "java2d";
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.dialog.DialogFactory#showOptionsDialog()
   */
  public void showOptionsDialog() {
    OptionsDialog dialog = 
      new OptionsDialog(WindowManager.getInstance().getMainFrame());
    OptionsDialogControl control = new OptionsDialogControl(dialog);
    dialog.setDialogControl(control);
    dialog.setModal(true);
    dialog.pack();
    dialog.setVisible(true);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.dialog.DialogFactory#showStatisticsDialog(fdda.core.demonstration.Statistics)
   */
  public void showStatisticsDialog(Statistics statistics) {
    StatisticsDialog dialog = 
      new StatisticsDialog(WindowManager.getInstance().getMainFrame());
    StatisticsDialogControl control = 
      new StatisticsDialogControl(dialog, statistics);
    dialog.setStatisticsDialogControl(control);
    dialog.setModal(false);
    dialog.pack();
    dialog.setVisible(true);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.dialog.DialogFactory#showAlgorithmDialog()
   */
  public void showAlgorithmDialog() {
    AlgorithmDialog dialog = 
      new AlgorithmDialog(WindowManager.getInstance().getMainFrame());
    AlgorithmDialogControl control = new AlgorithmDialogControl(dialog);
    dialog.setDialogControl(control);
    dialog.setModal(true);
    dialog.pack();
    dialog.setVisible(true);
  }    
  
  /*
   *  (non-Javadoc)
   * @see fdda.dialog.DialogFactory#showAboutDialog()
   */
  public void showAboutDialog() {
    AboutDialog dialog = 
      new AboutDialog(WindowManager.getInstance().getMainFrame());
    dialog.setModal(true);
    dialog.setVisible(true);
  }
}
