/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.java2d;

import java.awt.Shape;
import java.awt.geom.Area;
import java.util.Collection;
import java.util.Iterator;
import java.util.Set;

import fdda.dialog.MessageControl;
import fdda.graphics.GraphicalSystem;
import fdda.graphics.Message;
import fdda.graphics.java2d.Painter;

/**
 * Message control implementation for the java2d component. One instance for
 * each graphical system. An actual repaint is made only at the end of each 
 * cycle; the necessary clipping areas are collected while in the cycle.
 * @author Jrg Rdenauer
 */
public class Java2DMessageControl implements MessageControl {
  
  /** the graphical system */
  private GraphicalSystem system;
  /** the painter that draws the system */
  private Painter painter;
  
  /*
   *  (non-Javadoc)
   * @see fdda.dialog.MessageControl#startCycle()
   */
  public synchronized void startCycle() {
    painter.setClippingShape(new Area());
  }

  /**
   * Constructor
   * @param graphicHolder the graphics holder for the project
   */
  public Java2DMessageControl(GraphicHolder graphicHolder) {
    this.system = graphicHolder.getGraphicalSystem();
    this.painter = new Painter(graphicHolder);
  }

  /* (non-Javadoc)
   * @see fdda.dialog.MessageControl#moveMessages(java.util.Set)
   */
  public synchronized void moveMessages(Set messages) {
    Area clippingShape = new Area();
    for (Iterator it = messages.iterator(); it.hasNext(); ) {
      Message message = 
        ((fdda.algorithm.Message) it.next()).getGraphicalRepresentation();
      clippingShape.add(new Area(message.getShape()));
      message.moveForward();
      clippingShape.add(new Area(message.getShape()));
    }
    painter.addClippingShape(clippingShape);
  }


  /* (non-Javadoc)
   * @see fdda.dialog.MessageControl#addMessage(fdda.graphics.Message)
   */
  public synchronized void addMessage(Message message) {
    system.addMessage(message);
  }

  /* (non-Javadoc)
   * @see fdda.dialog.MessageControl#removeMessage(fdda.graphics.Message)
   */
  public synchronized void removeMessage(Message message) {
    Shape clippingShape = message.getShape();
    system.removeMessage(message);
    painter.addClippingShape(clippingShape);
  }

  /* (non-Javadoc)
   * @see fdda.dialog.MessageControl#removeAllMessages()
   */
  public synchronized void removeAllMessages() {
    Collection messages = system.getAllMessages();
    Area clippingShape = new Area();
    for (Iterator it = messages.iterator(); it.hasNext(); ) {
      clippingShape.add(new Area(((Message) it.next()).getShape()));
    }
    system.clearMessages();
    painter.addClippingShape(clippingShape);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.dialog.MessageControl#cycleDone()
   */
  public synchronized void cycleDone() {
    painter.repaintSystem();
    painter.updateScreen();   
  }    

}
