/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.java2d;

import fdda.graphics.Component;
import fdda.graphics.Node;
import fdda.graphics.ComponentFactory;

import fdda.graphics.Selection;
import fdda.graphics.SelectionModifier;

import java.awt.geom.Area;

/**
 * Adds nodes. Only finish() is important.
 * @author Jrg Rdenauer
 */
public class NodeCreator extends AbstractManipulator 
    implements SelectionModifier {

  /**
   * Constructor. 
   * @param graphicHolder the graphic holder
   */
  public NodeCreator(GraphicHolder graphicHolder) {
    super(graphicHolder);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#start(fdda.graphics.Component)
   */
  public void start(Component component) throws NoSuitableComponentException {
    start(component.getX(), component.getY());
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#start(double, double)
   */
  public void start(double x, double y) throws NoSuitableComponentException {
    // nichts
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#execute(double, double)
   */
  public void execute(double x, double y) throws NoSuitableComponentException {
    // nichts
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#finish(double, double)
   */
  public void finish(double x, double y) throws NoSuitableComponentException {
    x = this.moveToGrid(x, true);
    y = this.moveToGrid(y, false);
    Node node = ComponentFactory.getInstance().createNode(x, y, true);
    node.setGraphicalSystem(graphicHolder.getGraphicalSystem());
    graphicHolder.getGraphicalSystem().addNode(node);
    graphicHolder.getGraphicalSystem().setChanged(true);
    fdda.core.Core.getModelManager().addNode(
      graphicHolder.getGraphicalSystem().getProject(), node);
    Selection selection =
      ComponentFactory.getInstance().getComponentSelection();
    Area clippingArea = new Area(selection.getShape());
    selection.replaceWith(node, this);
    clippingArea.add(new Area(node.getShape()));
    painter.setClippingShape(clippingArea);
    painter.repaintSystem();
    painter.updateScreen();
  }

}