/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.java2d;

import fdda.graphics.Component;
import fdda.graphics.ComponentFactory;
import fdda.graphics.Selection;
import fdda.graphics.SelectionModifier;

import fdda.graphics.java2d.Point;
import fdda.graphics.java2d.Line;


import java.awt.Shape;
import java.awt.geom.Area;

/** 
 * Creates a point on a line. Starting component must be a line.
 * @author Jrg Rdenauer
 */
public class PointCreator extends AbstractManipulator 
    implements SelectionModifier {

  /** The line */
  private Line line;

  /**
   * Constructor
   * @param graphicHolder the graphic holder
   */
  public PointCreator(GraphicHolder graphicHolder) {
    super(graphicHolder);
    this.line = null;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#start(fdda.graphics.Component)
   */
  public void start(Component component) throws NoSuitableComponentException {
    if (component == null || !(component instanceof Line)) {
      throw new NoSuitableComponentException("Keine Linie uebergeben!");
    }
    this.line = (Line) component;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#start(double, double)
   */
  public void start(double x, double y) throws NoSuitableComponentException {
    Component component = graphicHolder.getGraphicalSystem().getElementAt(x, y);
    if (component == null || !(component instanceof Line)) {
      throw new NoSuitableComponentException("Keine Linie an den Koordinaten x="
        + x + ", y=" + y + "!");
    }
    this.line = (Line) component;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#execute(double, double)
   */
  public void execute(double x, double y) {
    // nothing
  }

  /**
   * Adds the point at the specified place. If the line doesn't go through that
   * place, the point is added anyway and the line will be changed.
   * @see fdda.dialog.java2d.Manipulator#finish(double, double)
   */
  public void finish(double x, double y) throws NoSuitableComponentException {
    if (line == null) {
      throw new NoSuitableComponentException("Noch keine Linie festgelegt!");
    }
    Shape oldLineShape = line.getShape();
    x = moveToGrid(x, true);
    y = moveToGrid(y, false);
    Point newPoint = line.addPoint(x, y);
    graphicHolder.getGraphicalSystem().setChanged(true);
    // change selection: new point is selected
    Selection theSelection =
        ComponentFactory.getInstance().getComponentSelection();
    Shape selectionShape = theSelection.getShape();
    theSelection.replaceWith(newPoint, this);

    Area clippingArea = new Area(line.getShape());
    clippingArea.add(new Area(oldLineShape));
    clippingArea.add(new Area(selectionShape));
    painter.setClippingShape(clippingArea);
    painter.repaintSystem();
    painter.updateScreen();
  }

}