/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.HeadlessException;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.ResourceBundle;

import javax.swing.*;

import fdda.Version;
import fdda.utils.Localization;

/**
 * The 'about' dialog
 * @author Jrg Rdenauer
 */
public class AboutDialog extends JDialog {
  
  private static String titleS = null;
  private static String authorS = null;
  private static String licenceS = null;
  private static String versionS = null;
  private static String thanksS = null;
  private static String closeS = null;
  private static String closeM = null;
  
  /**
   * Constructor
   * @param owner the owner
   * @throws HeadlessException
   */
  public AboutDialog(JFrame owner) throws HeadlessException {
    super(owner);
    if (closeS == null) loadLocalizedStrings();
    this.getContentPane().setLayout(new BorderLayout(10, 4));
    JLabel filler1 = new JLabel("");
    filler1.setPreferredSize(new Dimension(6, 6));
    this.getContentPane().add(filler1, BorderLayout.NORTH);
    JLabel filler2 = new JLabel("");
    filler2.setPreferredSize(new Dimension(3, 6));
    this.getContentPane().add(filler2, BorderLayout.WEST);
    JLabel filler3 = new JLabel("");
    filler3.setPreferredSize(new Dimension(3, 6));
    this.getContentPane().add(filler3, BorderLayout.EAST);
    JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
    JButton closeButton = new JButton(closeS);
    closeButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        AboutDialog.this.dispose();
      }
    });
    closeButton.setMnemonic(closeM.charAt(0));
    buttonPanel.add(closeButton);
    this.getContentPane().add(buttonPanel, BorderLayout.SOUTH);
    this.getRootPane().setDefaultButton(closeButton);
    JPanel infoPanel = new JPanel(new BorderLayout(10, 10));
    infoPanel.setBorder(BorderFactory.createEtchedBorder());
    JTextArea titleArea = new JTextArea("FDDA - Framework for the " +
      "Demonstration of Distributed Algorithms");
    titleArea.setEditable(false);
    titleArea.setFont(Font.decode("Arial-BOLD-14"));
    titleArea.setBackground(this.getContentPane().getBackground());
    titleArea.setLineWrap(true);
    titleArea.setWrapStyleWord(true);    
    infoPanel.add(titleArea, BorderLayout.NORTH);
    JLabel imageLabel = new JLabel(new ImageIcon(getClass().getResource(
      "/icon.gif")));
    infoPanel.add(imageLabel, BorderLayout.WEST);
    JTextArea mainArea = new JTextArea();
    mainArea.setEditable(false);
    mainArea.setFont(Font.decode("Arial-PLAIN-12"));
    mainArea.setBackground(this.getContentPane().getBackground());
    mainArea.setLineWrap(true);
    mainArea.setWrapStyleWord(true);    
    String info = Localization.formatMessage(versionS, new Object[] {
      new Integer(Version.MAJOR_NR), new Integer(Version.MINOR_NR), 
      Version.formatDate(Localization.getLocale())});
    info += "\n" + thanksS;
    mainArea.setText(info);
    infoPanel.add(mainArea, BorderLayout.CENTER);
    JTextArea licenceArea = new JTextArea();
    licenceArea.setEditable(false);
    licenceArea.setBackground(this.getContentPane().getBackground());
    licenceArea.setFont(Font.decode("Arial-PLAIN-10"));
    licenceArea.setText(authorS + " " + licenceS);
    licenceArea.setLineWrap(true);
    licenceArea.setWrapStyleWord(true);    
    infoPanel.add(licenceArea, BorderLayout.SOUTH);
    this.getContentPane().add(infoPanel, BorderLayout.CENTER);
    this.setTitle(titleS);
    this.setLocation(Toolkit.getDefaultToolkit().getScreenSize().width / 2
      - this.getSize().width / 2, 
      Toolkit.getDefaultToolkit().getScreenSize().height / 2
      - this.getSize().height / 2);
    this.pack();
    this.setSize(new Dimension(340, 184));
  }
  
  private static void loadLocalizedStrings() {
    ResourceBundle bundle = 
      Localization.getResourceBundleForPackage(AboutDialog.class);
    titleS = Localization.getLocalizedString(bundle, 
      "aboutTitleS", "About FDDA");
    authorS = Localization.getLocalizedString(bundle,
      "authorS", "Copyright  Jrg Rdenauer 2004.");
    licenceS = Localization.getLocalizedString(bundle,
      "licenceS", "This is free software.");
    versionS = Localization.getLocalizedString(bundle,
      "versionS", "Version {0}.{1}, compilation time {2}.");
    thanksS = Localization.getLocalizedString(bundle,
      "thanksS", "Many thanks to the team of BERLin.");
    closeS = Localization.getLocalizedString(bundle, 
      "closeS", "Close");
    closeM = Localization.getLocalizedString(bundle,
      "closeM", "C");
  }
}
