/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import fdda.dialog.AlgorithmData;
import fdda.dialog.Dialog;
import fdda.dialog.DialogControl;
import fdda.dialog.NoSuchCommandException;
import fdda.utils.Localization;

/**
 * The dialog where the user can select the alogrithm for a project.
 * @author Jrg Rdenauer
 */
public class AlgorithmDialog extends JDialog implements Dialog {
  
  private static String okS = null;
  private static String okM = null;
  private static String cancelS = null;
  private static String cancelM = null;
  private static String titleS = null;
  
  private JList algorithmList;
  private JTextArea descriptionField;
  private JButton okButton;
  private JButton cancelButton;
  
  /** the dialog control */
  private DialogControl control;
  /** the algorithm data */
  private AlgorithmData theData;

  /**
   * Constructor.
   * @param owner the owner
   */
  public AlgorithmDialog(Frame owner) throws HeadlessException  {
    super(owner);
    if (okS == null) loadLocalizedStrings();
    this.getContentPane().setLayout(new BorderLayout(0, 15));
    JPanel upperPanel = new JPanel(new BorderLayout(0, 10));
    JPanel lowerPanel = new JPanel(new BorderLayout(0, 0));
    
    JLabel fillLabel1 = new JLabel("");
    fillLabel1.setPreferredSize(new Dimension(10, 10));
    this.getContentPane().add(fillLabel1, BorderLayout.EAST);
    JLabel fillLabel2 = new JLabel("");
    fillLabel2.setPreferredSize(new Dimension(10, 10));
    this.getContentPane().add(fillLabel2, BorderLayout.WEST);
    JLabel fillLabel3 = new JLabel("");
    fillLabel3.setPreferredSize(new Dimension(10, 1));
    this.getContentPane().add(fillLabel3, BorderLayout.NORTH);
    
    algorithmList = new JList();
    algorithmList.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        theData.setCurrentAlgorithm(
          (String)algorithmList.getSelectedValue());
        descriptionField.setText(
          theData.getAlgorithmDescriptions()[algorithmList.getSelectedIndex()]);
      }
    });
    algorithmList.setPreferredSize(new Dimension(150, 90));
    upperPanel.add(algorithmList, BorderLayout.CENTER);
    
    JPanel textPanel = new JPanel(new BorderLayout());
    descriptionField = new JTextArea("");
    descriptionField.setEditable(false);
    descriptionField.setFocusable(false);
    descriptionField.setPreferredSize(new Dimension(150, 45));
    descriptionField.setLineWrap(true);
    descriptionField.setWrapStyleWord(true);
    descriptionField.setFont(Font.decode("Dialog-PLAIN-11"));
    
    textPanel.add(descriptionField, BorderLayout.CENTER);
    JLabel fillLabel4 = new JLabel("");
    fillLabel4.setPreferredSize(new Dimension(10, 10));
    JLabel fillLabel5 = new JLabel("");
    fillLabel5.setPreferredSize(new Dimension(10, 10));
    textPanel.add(fillLabel4, BorderLayout.EAST);
    textPanel.add(fillLabel5, BorderLayout.WEST);
    
    JPanel buttonsPanel = new JPanel(new FlowLayout(FlowLayout.CENTER, 50, 20));
    okButton = new JButton(okS);
    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (AlgorithmDialog.this.control.canClose(true)) {
          AlgorithmDialog.this.dispose();
        }
      }
    });
    okButton.setMnemonic(okM.charAt(0));
    buttonsPanel.add(okButton);
    cancelButton = new JButton(cancelS);
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (AlgorithmDialog.this.control.canClose(false)) {
          AlgorithmDialog.this.dispose();
        }
      }
    });
    cancelButton.setMnemonic(cancelM.charAt(0));
    buttonsPanel.add(cancelButton);
    okButton.setPreferredSize(cancelButton.getPreferredSize());
    this.getRootPane().setDefaultButton(okButton);
    lowerPanel.add(buttonsPanel, BorderLayout.SOUTH);
    lowerPanel.add(textPanel, BorderLayout.NORTH);
    this.getContentPane().add(upperPanel, BorderLayout.CENTER);
    this.getContentPane().add(lowerPanel, BorderLayout.SOUTH);
    
    this.setTitle(titleS);
    this.pack();
    this.setLocation(Toolkit.getDefaultToolkit().getScreenSize().width / 2
      - this.getSize().width / 2, 
      Toolkit.getDefaultToolkit().getScreenSize().height / 2
      - this.getSize().height / 2);    
  }
  
  /**
   * Sets the dialog control
   * @param control the control
   */
  public void setDialogControl(DialogControl control) {
    this.control = control;
  }
  
  /* (non-Javadoc)
   * @see fdda.dialog.Dialog#updateObject(java.lang.Object)
   */
  public void updateObject(Object data) {
    theData = (AlgorithmData) data;
    algorithmList.setListData(theData.getKnownAlgorithms());
    algorithmList.setSelectedValue(theData.getCurrentAlgorithm(), true);
    descriptionField.setText(
      theData.getAlgorithmDescriptions()[algorithmList.getSelectedIndex()]);
  }

  /* (non-Javadoc)
   * @see fdda.dialog.Dialog#getData()
   */
  public Object getData() {
    return theData;
  }

  /* (non-Javadoc)
   * @see fdda.dialog.Dialog#disableCommand(int)
   */
  public void disableCommand(int command) throws NoSuchCommandException {
    throw new NoSuchCommandException();
  }

  /* (non-Javadoc)
   * @see fdda.dialog.Dialog#enableCommand(int)
   */
  public void enableCommand(int command) throws NoSuchCommandException {
    throw new NoSuchCommandException();
  }
  
  /**
   * Loads the localized strings.
   */
  private static void loadLocalizedStrings() {
    ResourceBundle bundle = 
      Localization.getResourceBundleForPackage(AlgorithmDialog.class);
    okS = Localization.getLocalizedString(bundle, "okS", "Ok");
    okM = Localization.getLocalizedString(bundle, "okM", "O");
    cancelS = Localization.getLocalizedString(bundle, "cancelS", "Cancel");
    cancelM = Localization.getLocalizedString(bundle, "cancelM", "C");
    titleS = Localization.getLocalizedString(bundle, "algTitleS", 
      "Choose Algorithm");
  }

}
