/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing;

import fdda.dialog.GraphicsControl;

import fdda.graphics.ComponentFactory;
import fdda.graphics.Selection;
import fdda.graphics.SelectionEvent;
import fdda.graphics.SelectionListener;

import fdda.graphics.Node;
import fdda.graphics.Line;
import fdda.utils.Localization;

import fdda.core.Core;
import fdda.core.projects.*;

import fdda.dialog.swing.utils.RollOverButton;
import fdda.dialog.swing.utils.RollOverToggleButton;
import fdda.dialog.swing.utils.ToggleButtonGroup;

import java.awt.Dimension;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Iterator;
import java.util.ResourceBundle;

import javax.swing.JToolBar;

/**
 * The button bar shown for quick access to functions that are used often.
 * @author Jrg Rdenauer
 */
public class ButtonBar extends JToolBar
  implements SelectionListener, ProjectListener {

  private RollOverButton robNewProject;
  private RollOverButton robOpenProject;
  private RollOverButton robSaveProject;

  private RollOverToggleButton rtbPointerTool;
  private RollOverToggleButton rtbNodeTool;
  private RollOverToggleButton rtbLineTool;
  private RollOverToggleButton rtbDirLineTool;
  private RollOverToggleButton rtbDeleteTool;
  private ToggleButtonGroup tbgToolsGroup;
  
  private RollOverToggleButton rtbStart;
  private RollOverToggleButton rtbPause;
  private RollOverButton robStop;


  private RollOverToggleButton rtbProperties;

  private RollOverButton robZoomin;
  private RollOverButton robZoomout;

  private RollOverButton robTile;
  private RollOverButton robCascade;

  private RollOverButton robHelp;

  /** the selection */
  private Selection theSelection;

  /** iterator over the selection */
  private Iterator currentSelection;

  /** the main frame */
  private MainFrame frame;

  /**
   * Constructor. Builds the bar.
   * @param frame the main frame
   */
  public ButtonBar(final MainFrame frame) {
    super();
    this.frame = frame;

    // listen to selection changes
    theSelection = ComponentFactory.getInstance().getComponentSelection();
    theSelection.add(this);

    // listen to project changes
    Core.getProjectManager().addProjectListener(this);

    this.setPreferredSize(new Dimension(30, 30));
    setFloatable(false);
    setBorderPainted(false);
    
    // load tooltip texts
    ResourceBundle bundle = 
      Localization.getResourceBundleForPackage(getClass());
    String newProjectT = Localization.getLocalizedString(bundle, 
      "newProjectT", "Creates a new project");
    String openProjectT = Localization.getLocalizedString(bundle, 
      "openProjectT", "Opens a project");
    String saveProjectT = Localization.getLocalizedString(bundle, 
      "saveProjectT", "Saves the project");
    String pointerT = Localization.getLocalizedString(bundle, 
      "pointerT", "Select / Move");
    String nodeT = Localization.getLocalizedString(bundle, 
      "nodeT", "Add nodes");
    String lineT = Localization.getLocalizedString(bundle, 
      "lineT", "Add connections");
    String dirLineT = Localization.getLocalizedString(bundle, 
      "dirLineT", "Add directed connections");
    String deleteT = Localization.getLocalizedString(bundle, 
      "deleteT", "Delete parts");
    String propertiesT = Localization.getLocalizedString(bundle, 
      "popertiesT", "Show property editor");
    String zoomInT = Localization.getLocalizedString(bundle, 
      "zoomInT", "Zoom in");
    String zoomOutT = Localization.getLocalizedString(bundle, 
      "zoomOutT", "Zoom out");
    String startT = Localization.getLocalizedString(bundle, 
      "startT", "Start the demo");
    String pauseT = Localization.getLocalizedString(bundle, 
      "pauseT", "Pause the demo");
    String stopT = Localization.getLocalizedString(bundle, 
      "stopT", "Stop the demo");
    String tileT = Localization.getLocalizedString(bundle, 
      "tileT", "Tile windows");
    String cascadeT = Localization.getLocalizedString(bundle, 
      "cascadeT", "Cascade windows");
    String helpT = Localization.getLocalizedString(bundle, 
      "helpT", "Show help pages");

    // create buttons
    
    // project buttons
    robNewProject = new RollOverButton(new javax.swing.ImageIcon(
      getClass().getResource("/kneu.gif")), newProjectT);
    robOpenProject = new RollOverButton(new javax.swing.ImageIcon(
      getClass().getResource("/koeffnen.gif")), openProjectT);
    robSaveProject = new RollOverButton(new javax.swing.ImageIcon(
      getClass().getResource("/kspeichern.gif")), saveProjectT);
    // tools buttons
    rtbPointerTool = new RollOverToggleButton(new javax.swing.ImageIcon(
      getClass().getResource("/kpointer.gif")), pointerT);
    rtbNodeTool = new RollOverToggleButton(new javax.swing.ImageIcon(
      getClass().getResource("/knode.gif")), nodeT);
    rtbLineTool = new RollOverToggleButton(new javax.swing.ImageIcon(
      getClass().getResource("/kline.gif")), lineT);
    rtbDirLineTool = new RollOverToggleButton(new javax.swing.ImageIcon(
      getClass().getResource("/kdirline.gif")), dirLineT);
    rtbDeleteTool = new RollOverToggleButton(new javax.swing.ImageIcon(
      getClass().getResource("/kdelete.gif")), deleteT);
    // view buttons
    rtbProperties = new RollOverToggleButton(new javax.swing.ImageIcon(
      getClass().getResource("/keigenschaften.gif")), propertiesT);
    robZoomin = new RollOverButton(new javax.swing.ImageIcon(
      getClass().getResource("/kzoomplus.gif")), zoomInT);
    robZoomout = new RollOverButton(new javax.swing.ImageIcon(
      getClass().getResource("/kzoomminus.gif")), zoomOutT);
    // demo buttons
    rtbStart = new RollOverToggleButton(new javax.swing.ImageIcon(
      getClass().getResource("/kstart.gif")), startT);
    rtbPause = new RollOverToggleButton(new javax.swing.ImageIcon(
      getClass().getResource("/kpause.gif")), pauseT);
    robStop = new RollOverButton(new javax.swing.ImageIcon(
      getClass().getResource("/kstop.gif")), stopT);
    // window buttons
    robTile = new RollOverButton(new javax.swing.ImageIcon(
      getClass().getResource("/ktile.gif")), tileT);
    robCascade = new RollOverButton(new javax.swing.ImageIcon(
      getClass().getResource("/kcascade.gif")), cascadeT);
    robHelp = new RollOverButton(new javax.swing.ImageIcon(
      getClass().getResource("/khilfe.gif")), helpT);

    add(robNewProject);
    add(robOpenProject);
    add(robSaveProject);
    addSeparator();

    add(rtbPointerTool);
    add(rtbNodeTool);
    add(rtbLineTool);
    add(rtbDirLineTool);
    add(rtbDeleteTool);
    addSeparator();

    add(rtbProperties);
    addSeparator();

    add(robZoomin);
    add(robZoomout);
    addSeparator();

    add(rtbStart);
    add(rtbPause);
    add(robStop);
    addSeparator();

    add(robTile);
    add(robCascade);

    addSeparator();
    add(robHelp);

    // deaactivate some buttons 
    robOpenProject.setEnabled(true);
    robSaveProject.setEnabled(false);
    robTile.setEnabled(false);
    robCascade.setEnabled(false);
    rtbPause.setEnabled(false);
    robStop.setEnabled(false);

    tbgToolsGroup = new ToggleButtonGroup();
    tbgToolsGroup.addButton(rtbPointerTool);
    tbgToolsGroup.addButton(rtbNodeTool);
    tbgToolsGroup.addButton(rtbLineTool);
    tbgToolsGroup.addButton(rtbDirLineTool);
    tbgToolsGroup.addButton(rtbDeleteTool);
    tbgToolsGroup.setDefaultButton(rtbPointerTool);
    tbgToolsGroup.selectButton(rtbPointerTool);

    ActionListener alrNewProject = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        frame.newProject();
      }
    };
    robNewProject.addActionListener(alrNewProject);

    ActionListener alrOpenProject = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        frame.openProject(null);
      }
    };
    robOpenProject.addActionListener(alrOpenProject);

    ActionListener alrSaveProject = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        frame.saveProject(false);
      }
    };
    robSaveProject.addActionListener(alrSaveProject);

    ActionListener alrPointerTool = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        boolean shiftPressed = 
          ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
        frame.useTool(GraphicsControl.POINTER, shiftPressed);
      }
    };
    rtbPointerTool.addActionListener(alrPointerTool);

    ActionListener alrNodeTool = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        boolean shiftPressed = 
          ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
        frame.useTool(GraphicsControl.NODE, shiftPressed);
      }
    };
    rtbNodeTool.addActionListener(alrNodeTool);

    ActionListener alrLineTool = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        boolean shiftPressed = 
          ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
        frame.useTool(GraphicsControl.LINE, shiftPressed);
      }
    };
    rtbLineTool.addActionListener(alrLineTool);
    
    ActionListener alrDirLineTool = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        boolean shiftPressed = 
          ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
        frame.useTool(GraphicsControl.DIRECTED_LINE, shiftPressed);
      }
    };
    rtbDirLineTool.addActionListener(alrDirLineTool);

    ActionListener alrDeleteTool = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        boolean shiftPressed = 
          ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
        frame.useTool(GraphicsControl.DELETE, shiftPressed);
      }
    };
    rtbDeleteTool.addActionListener(alrDeleteTool);

    ActionListener alrProperties = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        frame.showObjectInspector(rtbProperties.isSelected());
      }
    };
    rtbProperties.addActionListener(alrProperties);

    ActionListener alrZoomin = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        frame.zoomIn();
      }
    };
    robZoomin.addActionListener(alrZoomin);

    ActionListener alrZoomout = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        frame.zoomOut();
      }
    };
    robZoomout.addActionListener(alrZoomout);
    
    ActionListener alrStart = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        frame.startDemonstration();
      }
    };
    rtbStart.addActionListener(alrStart);
    
    ActionListener alrPause = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        frame.pauseDemonstration();
      }
    };
    rtbPause.addActionListener(alrPause);
    
    ActionListener alrStop = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        frame.stopDemonstration();
      }
    };
    robStop.addActionListener(alrStop);

    ActionListener alrTile = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        frame.tile();
      }
    };
    robTile.addActionListener(alrTile);

    ActionListener alrCascade = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        frame.cascade();
      }
    };
    robCascade.addActionListener(alrCascade);

    ActionListener alrHelp = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        frame.help(0);
      }
    };
    robHelp.addActionListener(alrHelp);

    this.setVisible(true);
  }
  
  /**
   * Returns the button group for the tools buttons
   * @return the tools buttons group
   */
  public ToggleButtonGroup getToolsButtonGroup() {
    return tbgToolsGroup;
  }

  /**
   * Informs the button bar about the type of a system frame
   * @param code the type
   */
  public void setSystemFrameButtons(int code) {
    robZoomin.setEnabled(code != SystemFrame.SF_NONE);
    robZoomout.setEnabled(code != SystemFrame.SF_NONE);
  }

  /**
   * Sets the status of the properties button
   */
  public void setOInspectorButton(boolean status) {
    rtbProperties.setSelected(status);
  }
  
  /**
   * Informs the button bar that another tool was selected
   * @param tool the new tool
   * @param sticky whether the selection is sticky
   */
  public void toolSelected(int tool, boolean sticky) {
    switch(tool) {
      case GraphicsControl.POINTER: 
        tbgToolsGroup.selectButton(rtbPointerTool); 
        break;
      case GraphicsControl.NODE: 
        tbgToolsGroup.selectButton(rtbNodeTool, sticky); 
        break;
      case GraphicsControl.LINE: 
        tbgToolsGroup.selectButton(rtbLineTool, sticky); 
        break;
      case GraphicsControl.DIRECTED_LINE:
        tbgToolsGroup.selectButton(rtbDirLineTool, sticky); 
        break;
      case GraphicsControl.DELETE:
        tbgToolsGroup.selectButton(rtbDeleteTool, sticky);
        break;
      default: tbgToolsGroup.revertToDefaultButton();
    }
  }
  
  /**
   * Informs the button bar that the demo was paused or continued
   * @param pause true, iff it was paused
   */
  public void demoPaused(boolean pause) {
    rtbPause.setSelected(pause);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.SelectionListener#selectionChanged(fdda.graphics.SelectionEvent)
   */
  public void selectionChanged(SelectionEvent e) {
    // modelling state?
    //Project project = Core.getProjectManager().getActiveProject();
    //boolean model = project != null && project.getState() == Project.STATE_MOD;
    // get selection
    currentSelection = theSelection.getSelectionIterator();
    // only one component selected?
    if (theSelection.getSelectedComponent() != null ) {
      if (theSelection.getSelectedComponent() instanceof Line) {
        // a line
        //robDelete.setEnabled(model);
      }
      else if (theSelection.getSelectedComponent() instanceof Node) {
        // a node
        //robDelete.setEnabled(model);
      }
      else {
        //robDelete.setEnabled(false);
      }
    }
    else if (currentSelection.hasNext() == false) {
      // no element or more than one element
      //robDelete.setEnabled(model);
    }
    else {
      // more than one element
      //robDelete.setEnabled(model);
   }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectListener#projectOpened(fdda.core.projects.ProjectEvent)
   */
  public void projectOpened(ProjectEvent e) {
    robSaveProject.setEnabled(true);
    robZoomin.setEnabled(true);
    robZoomout.setEnabled(true);
    robTile.setEnabled(true);
    robCascade.setEnabled(true);
    rtbStart.setEnabled(true);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectListener#projectClosed(fdda.core.projects.ProjectEvent)
   */
  public void projectClosed(ProjectEvent e) {
    if (Core.getProjectManager().getNrOfProjects() == 0) {
      // no project open now
      robSaveProject.setEnabled(false);
      robZoomout.setEnabled(false);
      robZoomin.setEnabled(false);
      robTile.setEnabled(false);
      robCascade.setEnabled(false);
      rtbStart.setEnabled(false);
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectListener#activeProjectChanged(fdda.core.projects.ProjectEvent)
   */
  public void activeProjectChanged(ProjectEvent e) {
    if (Core.getProjectManager().isInDemoState()) {
      if (((Project)e.getSource()).getState() == Project.STATE_DEMONSTRATION) {
        robStop.setEnabled(true);
        rtbPause.setEnabled(true);
        setModellingEnabled(false);
      }
      else {
        robStop.setEnabled(false);
        rtbPause.setEnabled(false);
        setModellingEnabled(true);
      }
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectListener#projectStateChanged(fdda.core.projects.ProjectEvent)
   */
  public void projectStateChanged(ProjectEvent e) {
    boolean model = (((Project) e.getSource()).getState() == Project.STATE_MOD);
    setModellingEnabled(model);
    rtbStart.setSelected(!model);
    rtbStart.setEnabled(model);
    robStop.setEnabled(!model);
    rtbPause.setEnabled(!model);
    if (model) rtbPause.setSelected(false);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectListener#projectNameChanged(fdda.core.projects.ProjectEvent)
   */
  public void projectNameChanged(ProjectEvent e) {
  }

  /**
   * Activates or deactivates buttons pertaining to the modelling
   * @param model whether modelling is allowed
   */
  private void setModellingEnabled(boolean model) {
    if (model) {
      rtbPointerTool.setEnabled(true);
      rtbNodeTool.setEnabled(true);
      rtbLineTool.setEnabled(true);
      rtbDirLineTool.setEnabled(true);
      rtbDeleteTool.setEnabled(true);
      //selectionChanged(new SelectionEvent(ComponentSelection.getInstance()));
    }
    else {
      rtbPointerTool.setEnabled(true);
      rtbNodeTool.setEnabled(false);
      rtbLineTool.setEnabled(false);
      rtbDirLineTool.setEnabled(false);
      rtbDeleteTool.setEnabled(false);
    }
  }

}
