/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing;

//import fdda.Version;

import fdda.graphics.*;
import fdda.core.Core;
import fdda.core.projects.*;
import fdda.dialog.GraphicsControl;
import fdda.dialog.swing.utils.ToggleButtonGroup;

import fdda.utils.Localization;
import fdda.utils.Logging;
//import fdda.utils.Notification;
import fdda.utils.ObjectCache;

import fdda.plugins.ProjectInformation;
import fdda.plugins.SystemReader;
import fdda.plugins.SystemWriter;


import java.awt.Event;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import java.util.HashSet;
import java.util.Iterator;
import java.util.ResourceBundle;

import java.util.prefs.Preferences;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

/**
 * Contains the menu bar of fdda.
 * @author Jrg Rdenauer
 */
public class MenuBar extends JMenuBar
  implements SelectionListener, ProjectListener {
    
  /** how many 'last projects' entries? */
  public static final int LAST_PROJECTS_COUNT = 4;

  private static String projectMS, editMS, viewMS, windowMS, demoMS, helpMS;
  private static String projectMM, editMM, viewMM, windowMM, demoMM, helpMM;
  
  private static String newProjectMS, openProjectMS, saveProjectMS;
  private static String saveProjectAsMS, closeProjectMS, quitMS;
  private static String newProjectMM, openProjectMM, saveProjectMM;
  private static String saveProjectAsMM, closeProjectMM, quitMM;
  
  private static String deleteMM, pointerMM, nodeMM, lineMM, dirLineMM;
  private static String erazorMM, selectAllMM, optionsMM;
  private static String deleteMS, pointerMS, nodeMS, lineMS, dirLineMS;
  private static String erazorMS, selectAllMS, optionsMS;
  
  private static String zoomInMS, zoomOutMS, propertiesMS, gridMS;
  private static String zoomInMM, zoomOutMM, propertiesMM, gridMM;
  
  private static String algorithmMS, startMS, pauseMS, stopMS, statsMS;
  private static String algorithmMM, startMM, pauseMM, stopMM, statsMM;
  
  private static String tileMS, cascadeMS, jpegMS, tileMM, cascadeMM, jpegMM;
  
  private static String indexMS, aboutMS, indexMM, aboutMM;
  private static String tipsMS, tipsMM;
  
  private static boolean localized = false;

  /** the main frame */
  private MainFrame frame;

  private JMenu       mnuProject;
  private JMenu       mnuModify;
  private JMenu       mnuView;
  private JMenu       mnuWindow;
  private JMenu       mnuDemo;
  private JMenu       mnuHelp;

  private JMenuItem   muiNewProject;
  private JMenuItem   muiOpenProject;
  private JMenuItem   muiSaveProject;
  private JMenuItem   muiSaveProjectAs;
  private JMenuItem   muiCloseProject;
  private JMenuItem   muiQuit;

  private JMenuItem   muiDelete;
  private JMenuItem muiPointer;
  private JMenuItem muiNode;
  private JMenuItem muiLine;
  private JMenuItem muiDirLine;
  private JMenuItem muiErazor;
  private ToggleButtonGroup toolsMenuGroup;
  private JMenuItem muiSelectAll;
  private JMenuItem   muiPreferences;
  
  private JMenuItem   muiZoomin;
  private JMenuItem   muiZoomout;
  private JMenuItem   muiProperties;
  private JMenuItem   muiGrid;

  private JMenuItem muiAlgorithm;
  private JMenuItem   muiStart;
  private JMenuItem muiPause;
  private JMenuItem   muiStop;
  private JMenuItem muiStatistics;

  private JMenuItem   muiTile;
  private JMenuItem   muiCascade;
  private JMenuItem   muiJPEG;

  private JMenuItem   muiHelp;
  private JMenuItem   muiTips;
  private JMenuItem   muiAbout;

  /** the selection */
  private Selection theSelection;

  /** Iterator of the selection */
  private Iterator currentSelection;

  /**
   * Konstruktor der Klasse MenuBar.
   * @param frmMainFrame Zeiger auf die aufrufende MainFrame-Klasse
   */
  public MenuBar (final MainFrame mainFrame) {
    super();
    frame = mainFrame;
    
    if (!localized) loadLocalizedStrings();

    // register with selection to react to changes
    theSelection = ComponentFactory.getInstance().getComponentSelection();
    // bei der Selection anmelden
    theSelection.add(this);

    // also listen to project changes
    Core.getProjectManager().addProjectListener(this);

    // create menues
    mnuProject   = new JMenu(projectMS);
    mnuModify    = new JMenu(editMS);
    mnuView      = new JMenu(viewMS);
    mnuWindow    = new JMenu(windowMS);
    mnuDemo      = new JMenu(demoMS);
    mnuHelp      = new JMenu(helpMS);

    mnuProject.setMnemonic(projectMM.charAt(0));
    mnuModify.setMnemonic(editMM.charAt(0));
    mnuView.setMnemonic(viewMM.charAt(0));
    mnuWindow.setMnemonic(windowMM.charAt(0));
    mnuDemo.setMnemonic(demoMM.charAt(0));
    mnuHelp.setMnemonic(helpMM.charAt(0));

    // menu project
    muiNewProject = mnuProject.add(new JMenuItem(newProjectMS, 
      new javax.swing.ImageIcon(getClass().getResource("/kneu.gif"))));
    muiNewProject.setMnemonic(newProjectMM.charAt(0));
    muiNewProject.setAccelerator(
      KeyStroke.getKeyStroke(KeyEvent.VK_N, Event.CTRL_MASK));
    muiOpenProject = mnuProject.add(new JMenuItem(openProjectMS, 
      new javax.swing.ImageIcon(getClass().getResource("/koeffnen.gif"))));
    muiOpenProject.setMnemonic(openProjectMM.charAt(0));
    muiOpenProject.setAccelerator(
      KeyStroke.getKeyStroke(KeyEvent.VK_O, Event.CTRL_MASK));
    muiSaveProject = mnuProject.add(new JMenuItem(saveProjectMS, 
      new javax.swing.ImageIcon(getClass().getResource("/kspeichern.gif"))));
    muiSaveProject.setMnemonic(saveProjectMM.charAt(0));
    muiSaveProject.setAccelerator(
      KeyStroke.getKeyStroke(KeyEvent.VK_S, Event.CTRL_MASK));
    muiSaveProjectAs = mnuProject.add(
      new JMenuItem(saveProjectAsMS, saveProjectAsMM.charAt(0)));
    muiCloseProject = 
      mnuProject.add(new JMenuItem(closeProjectMS, closeProjectMM.charAt(0)));
    mnuProject.addSeparator();
    muiQuit = 
      mnuProject.add(new JMenuItem(quitMS, quitMM.charAt(0)));
    muiQuit.setAccelerator(
      KeyStroke.getKeyStroke(KeyEvent.VK_Q, Event.CTRL_MASK));

    // menu edit
    muiSelectAll = 
      mnuModify.add(new JMenuItem(selectAllMS, selectAllMM.charAt(0)));
    muiDelete = 
      mnuModify.add(new JMenuItem(deleteMS, deleteMM.charAt(0)));
    muiDelete.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 
      Event.CTRL_MASK));
    mnuModify.addSeparator();
    muiPointer = mnuModify.add(new JRadioButtonMenuItem(pointerMS, 
      new javax.swing.ImageIcon(getClass().getResource("/kpointer.gif"))));
    muiPointer.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0));
    muiPointer.setMnemonic(pointerMM.charAt(0));
    muiNode = mnuModify.add(new JRadioButtonMenuItem(nodeMS, 
      new javax.swing.ImageIcon(getClass().getResource("/knode.gif"))));
    muiNode.setMnemonic(nodeMM.charAt(0));
    muiNode.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F6, 0));
    muiLine = mnuModify.add(new JRadioButtonMenuItem(lineMS, 
      new javax.swing.ImageIcon(getClass().getResource("/kline.gif"))));
    muiLine.setMnemonic(lineMM.charAt(0));
    muiLine.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F7, 0));
    muiDirLine = mnuModify.add(new JRadioButtonMenuItem(dirLineMS, 
      new javax.swing.ImageIcon(getClass().getResource("/kdirline.gif"))));
    muiDirLine.setMnemonic(dirLineMM.charAt(0));
    muiDirLine.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F8, 0));
    muiErazor = mnuModify.add(new JRadioButtonMenuItem(erazorMS, 
      new javax.swing.ImageIcon(getClass().getResource("/kdelete.gif"))));
    muiErazor.setMnemonic(erazorMM.charAt(0));
    muiErazor.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F9, 0));
    
    toolsMenuGroup = new ToggleButtonGroup();
    toolsMenuGroup.addButton(muiPointer);
    toolsMenuGroup.addButton(muiNode);
    toolsMenuGroup.addButton(muiLine);
    toolsMenuGroup.addButton(muiDirLine);
    toolsMenuGroup.addButton(muiErazor);
    toolsMenuGroup.setDefaultButton(muiPointer);
    toolsMenuGroup.selectButton(muiPointer);
    
    mnuModify.addSeparator();
    muiPreferences = 
      mnuModify.add(new JMenuItem(optionsMS, optionsMM.charAt(0)));

    // menu view
    muiZoomin = mnuView.add(new JMenuItem(zoomInMS, 
      new javax.swing.ImageIcon(getClass().getResource("/kzoomplus.gif"))));
    muiZoomin.setMnemonic(zoomInMM.charAt(0));
    muiZoomin.setAccelerator(
      KeyStroke.getKeyStroke(KeyEvent.VK_ADD, Event.CTRL_MASK));
    muiZoomout = mnuView.add(new JMenuItem(zoomOutMS, 
      new javax.swing.ImageIcon(getClass().getResource("/kzoomminus.gif"))));
    muiZoomout.setMnemonic(zoomOutMM.charAt(0));
    muiZoomout.setAccelerator(
      KeyStroke.getKeyStroke(KeyEvent.VK_SUBTRACT, Event.CTRL_MASK));
    mnuView.addSeparator();
    muiProperties = mnuView.add(
      new JCheckBoxMenuItem(propertiesMS, false));
    muiProperties.setMnemonic(propertiesMM.charAt(0));
    mnuView.addSeparator();
    muiGrid = mnuView.add(
      new JCheckBoxMenuItem(gridMS, true));
    muiGrid.setMnemonic(gridMM.charAt(0));

    // menu demo
    muiAlgorithm = 
      mnuDemo.add(new JMenuItem(algorithmMS, algorithmMM.charAt(0)));
    mnuDemo.addSeparator();
    muiStart = mnuDemo.add(new JMenuItem(startMS, new javax.swing.ImageIcon(
      getClass().getResource("/kstart.gif"))));
    muiStart.setMnemonic(startMM.charAt(0));
    muiPause = mnuDemo.add(new JCheckBoxMenuItem(pauseMS, 
      new javax.swing.ImageIcon(getClass().getResource("/kpause.gif"))));
    muiPause.setMnemonic(pauseMM.charAt(0));
    muiStop = mnuDemo.add(new JMenuItem(stopMS, new javax.swing.ImageIcon(
      getClass().getResource("/kstop.gif"))));
    muiStop.setMnemonic(stopMM.charAt(0));
    mnuDemo.addSeparator();
    muiStatistics = mnuDemo.add(new JMenuItem(statsMS, statsMM.charAt(0)));
    
    // menu window
    muiTile = mnuWindow.add(new JMenuItem(tileMS, new javax.swing.ImageIcon(
      getClass().getResource("/ktile.gif"))));
    muiTile.setMnemonic(tileMM.charAt(0));
    muiTile.setAccelerator(
      KeyStroke.getKeyStroke(KeyEvent.VK_T, Event.CTRL_MASK));
    muiCascade = mnuWindow.add(new JMenuItem(cascadeMS, 
      new javax.swing.ImageIcon(getClass().getResource("/kcascade.gif"))));
    muiCascade.setMnemonic(cascadeMM.charAt(0));
    muiCascade.setAccelerator(
      KeyStroke.getKeyStroke(KeyEvent.VK_K, Event.CTRL_MASK));
    muiJPEG = mnuWindow.add(new JMenuItem(jpegMS, jpegMM.charAt(0)));
    muiJPEG.setAccelerator(
      KeyStroke.getKeyStroke(KeyEvent.VK_J, Event.CTRL_MASK));
    // tell the window manager about the menu. 3 Entries are fixed.
    WindowManager.getInstance().setWindowMenu(mnuWindow, 3);

    // menu help
    muiHelp = mnuHelp.add(new JMenuItem(indexMS, indexMM.charAt(0)));
    muiHelp.setAccelerator(
      KeyStroke.getKeyStroke(KeyEvent.VK_H, Event.CTRL_MASK));
    muiTips = mnuHelp.add(new JMenuItem(tipsMS, tipsMM.charAt(0)));
    mnuHelp.addSeparator();
    muiAbout = mnuHelp.add(new JMenuItem(aboutMS, aboutMM.charAt(0)));

    add(mnuProject);
    add(mnuModify);
    add(mnuView);
    add(mnuDemo);
    add(mnuWindow);
    add(mnuHelp);

    // deactivate some entries
    muiOpenProject.setEnabled(true);
    muiSaveProject.setEnabled(false);
    muiSaveProjectAs.setEnabled(false);
    muiCloseProject.setEnabled(false);
    muiSelectAll.setEnabled(false);
    muiDelete.setEnabled(false);
    muiStart.setEnabled(false);
    muiPause.setEnabled(false);
    muiStop.setEnabled(false);
    muiStatistics.setEnabled(false);
    muiTile.setEnabled(false);
    muiCascade.setEnabled(false);
    muiJPEG.setEnabled(false);

    ActionListener alrNewProject = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        mainFrame.newProject();
      }
    };
    muiNewProject.addActionListener(alrNewProject);

    ActionListener alrOpenProject = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        mainFrame.openProject(null);
      }
    };
    muiOpenProject.addActionListener(alrOpenProject);

    ActionListener alrSaveProject = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        mainFrame.saveProject(false);
      }
    };
    muiSaveProject.addActionListener(alrSaveProject);

    ActionListener alrSaveProjectAs = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        mainFrame.saveProject(true);
      }
    };
    muiSaveProjectAs.addActionListener(alrSaveProjectAs);

    ActionListener alrCloseProject = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        mainFrame.closeProject();
      }
    };
    muiCloseProject.addActionListener(alrCloseProject);

    ActionListener alrJPEG = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        mainFrame.saveJPEG();
      }
    };
    muiJPEG.addActionListener(alrJPEG);

    ActionListener alrQuit = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        mainFrame.quit();
      }
    };
    muiQuit.addActionListener(alrQuit);

    ActionListener alrPreferences = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        mainFrame.showOptionsDialog();
      }
    };
    muiPreferences.addActionListener(alrPreferences);


    ActionListener alrSelectAll = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        mainFrame.selectAll();
      }
    };
    muiSelectAll.addActionListener(alrSelectAll);
    
    ActionListener alrPointer = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        boolean shiftPressed = ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
        mainFrame.useTool(GraphicsControl.POINTER, shiftPressed);
      }
    };
    muiPointer.addActionListener(alrPointer);
    ActionListener alrNode = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        boolean shiftPressed = ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
        mainFrame.useTool(GraphicsControl.NODE, shiftPressed);
      }
    };
    muiNode.addActionListener(alrNode);
    ActionListener alrLine = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        boolean shiftPressed = ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
        mainFrame.useTool(GraphicsControl.LINE, shiftPressed);
      }
    };
    muiLine.addActionListener(alrLine);
    ActionListener alrDirLine = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        boolean shiftPressed = ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
        mainFrame.useTool(GraphicsControl.DIRECTED_LINE, shiftPressed);
      }
    };
    muiDirLine.addActionListener(alrDirLine);
    ActionListener alrErazor = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        boolean shiftPressed = ((e.getModifiers() & ActionEvent.SHIFT_MASK) != 0);
        mainFrame.useTool(GraphicsControl.DELETE, shiftPressed);
      }
    };
    muiErazor.addActionListener(alrErazor);
    
    ActionListener alrAlgorithm = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        mainFrame.chooseAlgorithm();
      }
    };
    muiAlgorithm.addActionListener(alrAlgorithm);
    
    ActionListener alrStart = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        mainFrame.startDemonstration();
      }
    };
    muiStart.addActionListener(alrStart);
    ActionListener alrStop = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        mainFrame.stopDemonstration();
      }
    };
    muiStop.addActionListener(alrStop);
    ActionListener alrPause = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        mainFrame.pauseDemonstration();
      }
    };
    muiPause.addActionListener(alrPause);
    
    ActionListener alrStatistics = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        mainFrame.showStatistics();
      }
    };
    muiStatistics.addActionListener(alrStatistics);

    ActionListener alrProperties = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        mainFrame.showObjectInspector(muiProperties.isSelected());
      }
    };
    muiProperties.addActionListener(alrProperties);

    ActionListener alrGrid = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        mainFrame.showGrid(muiGrid.isSelected());
      }
    };
    muiGrid.addActionListener(alrGrid);

    ActionListener alrZoomin = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        mainFrame.zoomIn();
      }
    };
    muiZoomin.addActionListener(alrZoomin);

    ActionListener alrZoomout = new ActionListener()  {
      public void actionPerformed(ActionEvent e) {
        mainFrame.zoomOut();
      }
    };
    muiZoomout.addActionListener(alrZoomout);

    ActionListener alrTile = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        mainFrame.tile();
      }
    };
    muiTile.addActionListener(alrTile);

    ActionListener alrCascade = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        mainFrame.cascade();
      }
    };
    muiCascade.addActionListener(alrCascade);

    ActionListener alrDelete = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        mainFrame.deleteSelection();
      }
    };
    muiDelete.addActionListener(alrDelete);

    ActionListener alrHelp = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        mainFrame.help(0);
      }
    };
    muiHelp.addActionListener(alrHelp);
    
    ActionListener alrTips = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        mainFrame.help(1);
      }
    };
    muiTips.addActionListener(alrTips);

    ActionListener alrAbout = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        mainFrame.showInfo();
      }
    };
    muiAbout.addActionListener(alrAbout);

    rebuildLastFilesMenu();
  }
  
  /**
   * Informs the menu that another tool was selected
   * @param tool the tool
   * @param sticky whether the selection is 'sticky'
   */
  public void toolSelected(int tool, boolean sticky) {
    switch(tool) {
      case GraphicsControl.POINTER: 
        toolsMenuGroup.selectButton(muiPointer); 
        break;
      case GraphicsControl.NODE: 
        toolsMenuGroup.selectButton(muiNode, sticky); 
        break;
      case GraphicsControl.LINE: 
        toolsMenuGroup.selectButton(muiLine, sticky); 
        break;
      case GraphicsControl.DIRECTED_LINE:
        toolsMenuGroup.selectButton(muiDirLine, sticky);
        break;
      case GraphicsControl.DELETE: 
        toolsMenuGroup.selectButton(muiErazor, sticky);
        break;
      default: toolsMenuGroup.revertToDefaultButton();
    }
  }
  
  /**
   * Returns the tools button group
   * @return the tools button group
   */
  public ToggleButtonGroup getToolsMenuGroup() {
    return toolsMenuGroup;
  }

  /**
   * Informs the menu about the type of the window
   * @param code
   */
  public void setSystemFrameMEntrys(int code) {
    muiSelectAll.setEnabled(code != SystemFrame.SF_NONE);
    muiZoomin.setEnabled(code != SystemFrame.SF_NONE);
    muiZoomout.setEnabled(code != SystemFrame.SF_NONE);
    muiJPEG.setEnabled(code != SystemFrame.SF_NONE);
  }

  /**
   * Informs the window that the property editor was shown or hidden
   * @param status true iff it was shown
   */
  public void setOInspectorMEntry(boolean status) {
    muiProperties.setSelected(status);
  }

  /**
   * Selects the grid entry
   * @param status true iff selected
   */
  public void setGridMEntry(boolean status) {
    muiGrid.setSelected(status);
  }
  
  /**
   * Informs the menu that the demo was paused or not paused.
   * @param pause true iff paused
   */
  public void demoPaused(boolean pause) {
    muiPause.setSelected(pause);
  }

  /**
   * Rebuilds the menu for the last used files.
   */
  public void rebuildLastFilesMenu() {
    // remove old entries
    int i = 0;
    boolean found = false;
    while (!found) {
      JMenuItem item = mnuProject.getItem(i);
      if (item == muiQuit) {
        found = true;
      }
      else {
        i++;
      }
    }
    // i points to the 'quit'-entry
    while (i != mnuProject.getItemCount() - 1) {
      mnuProject.remove(i + 1);
    }

    // get last used projects
    ProjectInformation[] loadedInfos = null;
    ProjectInformation[] savedInfos = null;
    String className = Preferences.userRoot().node("fdda/plugins").get(
      "currentProjectWriter", "fdda.plugins.xmlwriter.ProjectWriter");
    if (className != null && !className.equals("")) {
      try {
        SystemWriter projectWriter =
          (SystemWriter) ObjectCache.getInstance().getObject(className);
        savedInfos = projectWriter.getLastSavedProjectsInformation();
      }
      catch (ClassNotFoundException e) {
        Logging.exceptionCaught(e);
      }
      catch (InstantiationException e) {
        Logging.exceptionCaught(e);
      }
      catch (IllegalAccessException e) {
        Logging.exceptionCaught(e);
      }
      catch (ClassCastException e) {
        Logging.exceptionCaught(e);
      }
    }
    className = Preferences.userRoot().node("fdda/plugins").get(
      "currentProjectReader", "fdda.plugins.xmlreader.ProjectReader");
    if (className != null && !className.equals("")) {
      try {
        SystemReader projectReader =
          (SystemReader) ObjectCache.getInstance().getObject(className);
        loadedInfos = projectReader.getLastLoadedProjectsInformation();
      }
      catch (ClassNotFoundException e) {
        Logging.exceptionCaught(e);
      }
      catch (InstantiationException e) {
        Logging.exceptionCaught(e);
      }
      catch (IllegalAccessException e) {
        Logging.exceptionCaught(e);
      }
      catch (ClassCastException e) {
        Logging.exceptionCaught(e);
      }
    }
    if (savedInfos.length > 0 || loadedInfos.length > 0) {
      mnuProject.addSeparator();
    }
    // merging of loaded and saved Projects
    HashSet allInfos = new HashSet();
    int saved = 0; 
    int loaded = 0;
    int count = 0;
    while (saved < savedInfos.length && loaded < loadedInfos.length &&
           count < LAST_PROJECTS_COUNT) {
      ProjectInformation info = 
        (savedInfos[saved].getProcessingTime().compareTo(
         loadedInfos[loaded].getProcessingTime()) < 0) ? 
        loadedInfos[loaded++] : savedInfos[saved++];
      if (!allInfos.contains(info.getLocation())) {
        allInfos.add(info.getLocation());
        addLastProjectItem(info, ++count);
      }
    }
    ProjectInformation[] remainingInfos = 
      saved < savedInfos.length ? savedInfos : loadedInfos;
    int pointer = saved < savedInfos.length ? saved : loaded;
    while (pointer < remainingInfos.length && count < LAST_PROJECTS_COUNT) {
      ProjectInformation info = remainingInfos[pointer++];
      if (!allInfos.contains(info.getLocation())) {
        allInfos.add(info.getLocation());
        addLastProjectItem(info, ++count);
      }
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.SelectionListener#selectionChanged(fdda.graphics.SelectionEvent)
   */
  public void selectionChanged(SelectionEvent e) {
    // modelling allowed?
    Project project = Core.getProjectManager().getActiveProject();
    boolean model = project != null && project.getState() == Project.STATE_MOD;
    currentSelection = theSelection.getSelectionIterator();
    if (theSelection.getSelectedComponent() != null) {
      // only one component selected
      muiDelete.setEnabled(model);
    }
    else if (!currentSelection.hasNext()) {
      // no component selected
      muiDelete.setEnabled(false);
    }
    else {
      // more than one component selected
      muiDelete.setEnabled(model);
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectListener#projectOpened(fdda.core.projects.ProjectEvent)
   */
  public void projectOpened(ProjectEvent e) {
    muiSaveProject.setEnabled(true);
    muiSaveProjectAs.setEnabled(true);
    muiCloseProject.setEnabled(true);
    muiSelectAll.setEnabled(true);
    muiZoomin.setEnabled(true);
    muiZoomout.setEnabled(true);
    muiTile.setEnabled(true);
    muiCascade.setEnabled(true);
    muiStart.setEnabled(true);
    muiStatistics.setEnabled(false);
    muiAlgorithm.setEnabled(true);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectListener#activeProjectChanged(fdda.core.projects.ProjectEvent)
   */
  public void activeProjectChanged(ProjectEvent e){
    if (Core.getProjectManager().isInDemoState()) {
      if (((Project) e.getSource()).getState() == Project.STATE_DEMONSTRATION) {
        muiPause.setEnabled(true);
        muiStop.setEnabled(true);
        this.setModellingEnabled(false);
      }
      else {
        muiStop.setEnabled(false);
        muiPause.setEnabled(false);
        this.setModellingEnabled(true);
      }
    }
    muiStatistics.setEnabled(Core.getDemonstration().wasDemonstrated(
      (Project) e.getSource()));
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectListener#projectClosed(fdda.core.projects.ProjectEvent)
   */
  public void projectClosed(ProjectEvent e) {
    if (Core.getProjectManager().getNrOfProjects() == 0) {
      // no project open now
      muiSaveProject.setEnabled(false);
      muiSaveProjectAs.setEnabled(false);
      muiCloseProject.setEnabled(false);
      muiSelectAll.setEnabled(false);
      muiZoomin.setEnabled(false);
      muiZoomout.setEnabled(false);
      muiTile.setEnabled(false);
      muiCascade.setEnabled(false);
      muiStart.setEnabled(false);
      muiStatistics.setEnabled(false);
      muiAlgorithm.setEnabled(false);
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectListener#projectStateChanged(fdda.core.projects.ProjectEvent)
   */
  public void projectStateChanged(ProjectEvent e) {
    boolean model = (((Project) e.getSource()).getState() == Project.STATE_MOD);
    setModellingEnabled(model);
    muiStart.setEnabled(model);
    muiStop.setEnabled(!model);
    muiPause.setEnabled(!model);
    if (model) muiPause.setSelected(false);
    muiStatistics.setEnabled(Core.getDemonstration().wasDemonstrated(
      Core.getProjectManager().getActiveProject()));
 }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectListener#projectNameChanged(fdda.core.projects.ProjectEvent)
   */
  public void projectNameChanged(ProjectEvent e) {
  }

  /**
   * Adds a menu item for a last used project
   * @param info information about the project
   * @param count the nr of the menu entry
   */
  private void addLastProjectItem(ProjectInformation info, int count) {
    String location = info.getLocation();
    String name = info.getName();
    JMenuItem muiLastProject = new JMenuItem(count + " " + name);
    muiLastProject.setMnemonic((count + "").charAt(0));
    mnuProject.add(muiLastProject);
    muiLastProject.addActionListener(new LastProjectListener(
      location, frame));
  }

  /**
   * (De-)Activates menu entries according to whether modelling is allowed
   * @model true, iff modelling is allowed
   */
  private void setModellingEnabled(boolean model) {
    if (model) {
      muiPointer.setEnabled(true);
      muiNode.setEnabled(true);
      muiLine.setEnabled(true);
      muiDirLine.setEnabled(true);
      muiErazor.setEnabled(true);
      muiDelete.setEnabled(true);
      muiAlgorithm.setEnabled(true);
     // selectionChanged(new SelectionEvent(ComponentSelection.getInstance()));
    }
    else {
      frame.useTool(GraphicsControl.POINTER, false);
      muiDelete.setEnabled(false);
      muiPointer.setEnabled(true);
      muiNode.setEnabled(false);
      muiLine.setEnabled(false);
      muiDirLine.setEnabled(false);
      muiErazor.setEnabled(false);
      muiAlgorithm.setEnabled(false);
    }
  }

  /**
   * Loads the localized Strings
   */
  private void loadLocalizedStrings() {
    ResourceBundle bundle = 
      Localization.getResourceBundleForPackage(MenuBar.class);
     
    projectMS = Localization.getLocalizedString(bundle,
    "projectMS", "Project"); 
    editMS = Localization.getLocalizedString(bundle,
    "editMS", "Edit"); 
    viewMS = Localization.getLocalizedString(bundle,
    "viewMS", "View"); 
    windowMS = Localization.getLocalizedString(bundle,
    "windowMS", "Window"); 
    demoMS = Localization.getLocalizedString(bundle,
    "demoMS", "Demonstration"); 
    helpMS = Localization.getLocalizedString(bundle,
    "helpMS", "Help");
    projectMM = Localization.getLocalizedString(bundle,
    "projectMM", "P"); 
    editMM = Localization.getLocalizedString(bundle,
    "editMM", "E"); 
    viewMM = Localization.getLocalizedString(bundle,
    "viewMM", "V"); 
    windowMM = Localization.getLocalizedString(bundle,
    "windowMM", "W"); 
    demoMM = Localization.getLocalizedString(bundle,
    "demoMM", "D"); 
    helpMM = Localization.getLocalizedString(bundle,
    "helpMM", "H");;
    newProjectMS = Localization.getLocalizedString(bundle,
    "newProjectMS", "New"); 
    openProjectMS = Localization.getLocalizedString(bundle,
    "openProjectMS", "Open..."); 
    saveProjectMS = Localization.getLocalizedString(bundle,
    "saveProjectMS", "Save");
    saveProjectAsMS = Localization.getLocalizedString(bundle,
    "saveProjectAsMS", "Save as..."); 
    closeProjectMS = Localization.getLocalizedString(bundle,
    "closeProjectMS", "Close"); 
    quitMS = Localization.getLocalizedString(bundle,
    "quitMS", "Quit");
    newProjectMM = Localization.getLocalizedString(bundle,
    "newProjectMM", "N"); 
    openProjectMM = Localization.getLocalizedString(bundle,
    "openProjectMM", "O"); 
    saveProjectMM = Localization.getLocalizedString(bundle,
    "saveProjectMM", "S");
    saveProjectAsMM = Localization.getLocalizedString(bundle,
    "saveProjectAsMM", "A"); 
    closeProjectMM = Localization.getLocalizedString(bundle,
    "closeProjectMM", "C"); 
    quitMM = Localization.getLocalizedString(bundle,
    "quitMM", "Q");
    deleteMM = Localization.getLocalizedString(bundle,
    "deleteMM", "T"); 
    pointerMM = Localization.getLocalizedString(bundle,
    "pointerMM", "S"); 
    nodeMM = Localization.getLocalizedString(bundle,
    "nodeMM", "N"); 
    lineMM = Localization.getLocalizedString(bundle,
    "lineMM", "L"); 
    dirLineMM = Localization.getLocalizedString(bundle,
    "dirLineMM", "D");
    erazorMM = Localization.getLocalizedString(bundle,
    "erazorMM", "E"); 
    selectAllMM = Localization.getLocalizedString(bundle,
    "selectAllMM", "S"); 
    optionsMM = Localization.getLocalizedString(bundle,
    "optionsMM", "O");
    deleteMS = Localization.getLocalizedString(bundle,
    "deleteMS", "Delete"); 
    pointerMS = Localization.getLocalizedString(bundle,
    "pointerMS", "Select"); 
    nodeMS = Localization.getLocalizedString(bundle,
    "nodeMS", "Node"); 
    lineMS = Localization.getLocalizedString(bundle,
    "lineMS", "Line"); 
    dirLineMS = Localization.getLocalizedString(bundle,
    "dirLineMS", "Directed line");
    erazorMS = Localization.getLocalizedString(bundle,
    "erazorMS", "Erazor"); 
    selectAllMS = Localization.getLocalizedString(bundle,
    "selectAllMS", "Select all"); 
    optionsMS = Localization.getLocalizedString(bundle,
    "optionsMS", "Options...");
    zoomInMS = Localization.getLocalizedString(bundle,
    "zoomInMS", "Zoom in"); 
    zoomOutMS = Localization.getLocalizedString(bundle,
    "zoomOutMS", "Zoom out"); 
    propertiesMS = Localization.getLocalizedString(bundle,
    "propertiesMS", "Property editor"); 
    gridMS = Localization.getLocalizedString(bundle,
    "gridMS", "Show grid");
    zoomInMM = Localization.getLocalizedString(bundle,
    "zoomInMM", "Z"); 
    zoomOutMM = Localization.getLocalizedString(bundle,
    "zoomOutMM", "O"); 
    propertiesMM = Localization.getLocalizedString(bundle,
    "popertiesMM", "P"); 
    gridMM = Localization.getLocalizedString(bundle,
    "gridMM", "G");
    algorithmMS = Localization.getLocalizedString(bundle,
    "algorithmMS", "Algorithm..."); 
    startMS = Localization.getLocalizedString(bundle,
    "startMS", "Start"); 
    pauseMS = Localization.getLocalizedString(bundle,
    "pauseMS", "Pause"); 
    stopMS = Localization.getLocalizedString(bundle,
    "stopMS", "Stop"); 
    statsMS = Localization.getLocalizedString(bundle,
    "statsMS", "Statistics...");
    algorithmMM = Localization.getLocalizedString(bundle,
    "algorithmMM", "A"); 
    startMM = Localization.getLocalizedString(bundle,
    "startMM", "S"); 
    pauseMM = Localization.getLocalizedString(bundle,
    "pauseMM", "P"); 
    stopMM = Localization.getLocalizedString(bundle,
    "stopMM", "S"); 
    statsMM = Localization.getLocalizedString(bundle,
    "statsMM", "T");
    tileMS = Localization.getLocalizedString(bundle,
    "tileMS", "Tile"); 
    cascadeMS = Localization.getLocalizedString(bundle,
    "cascadeMS", "Cascade"); 
    jpegMS = Localization.getLocalizedString(bundle,
    "jpegMS", "Export as JPEG..."); 
    tileMM = Localization.getLocalizedString(bundle,
    "tileMM", "T"); 
    cascadeMM = Localization.getLocalizedString(bundle,
    "cascaseMM", "C"); 
    jpegMM = Localization.getLocalizedString(bundle,
    "jpegMM", "J");
    indexMS = Localization.getLocalizedString(bundle,
    "indexMS", "Contents"); 
    aboutMS = Localization.getLocalizedString(bundle,
    "aboutMS", "About..."); 
    indexMM = Localization.getLocalizedString(bundle,
    "indexMM", "C"); 
    aboutMM = Localization.getLocalizedString(bundle,
    "aboutMM", "A");
    tipsMS = Localization.getLocalizedString(bundle,
    "tipsMS", "Tips");
    tipsMM = Localization.getLocalizedString(bundle,
    "tipsMM", "T");
    
    localized = true;
  }

  /**
   * Action Listener for the 'last projects' entries
   */
  class LastProjectListener implements ActionListener {
    
    /** the location */    
    private String location = null;
    /** the main frame */
    private MainFrame frame = null;

    /**
     * Constructor
     * @param location location of the project
     * @param frame the main frame
     */
    public LastProjectListener(String location, MainFrame frame) {
      this.location = location;
      this.frame = frame;
    }
    /*
     *  (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
      frame.openProject(location);
    }
  };

}
