/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing;

import fdda.core.Core;
import fdda.dialog.Options;
import fdda.dialog.OptionsCommands;
import fdda.dialog.Dialog;
import fdda.dialog.DialogControl;
import fdda.dialog.NoSuchCommandException;
import fdda.utils.Localization;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.Iterator;
import java.util.ResourceBundle;

import javax.swing.*;

/**
 * The options dialog.
 * @author Jrg Rdenauer
 */
public class OptionsDialog extends JDialog implements Dialog {
  
  private static String titleS = null;
  private static String actionTitleS = null;
  private static String noProjectS = null;
  private static String lastProjectS = null;
  private static String newProjectS = null;
  private static String diverseS = null;
  private static String quitQuestionS = null;
  private static String gridWidthS = null;
  private static String demoTitleS = null;
  private static String minCycleTimeS = null;
  private static String msgSpeedS = null;
  private static String pluginTitleS = null;
  private static String configureS = null;
  private static String configure1M = null;
  private static String configure2M = null;
  private static String configure3M = null;
  private static String projectLoaderS = null;
  private static String projectSaverS = null;
  private static String algLoaderS = null;
  private static String lastProjectM = null;
  private static String noProjectM = null;
  private static String newProjectM = null;
  private static String quitQuestionM = null;
  private static String okS = null;
  private static String okM = null;
  private static String cancelS = null;
  private static String cancelM = null;
  private static String errorsTitleS = null;
  private static String browserS = null;
  private static String browserT = null;

  /** the options */
  private Options options;
  /** the dialog control */
  private DialogControl control;
  
  private JComboBox systemWriterBox;
  private JComboBox systemReaderBox;
  private JComboBox algReaderBox;
  private JSpinner gridWidthBox;
  private SpinnerNumberModel gridWidthModel;
  private JCheckBox askAtEndBox;
  private ButtonGroup actionGroup;
  private JRadioButton noActionButton;
  private JRadioButton newActionButton;
  private JRadioButton openActionButton;
  
  private JTextField cycleTimeBox;
  private JTextField messageSpeedBox;
  private JTextField browserField;
  
  private JButton configureWriterButton;
  private JButton configureReaderButton;
  private JButton configureAlgReaderButton;
  private JButton okButton;
  private JButton cancelButton;
  
  /**
   * Constructor
   * @param owner the owner
   * @throws HeadlessException
   */
  public OptionsDialog(Frame owner) 
  throws HeadlessException {
    super(owner);
    if (titleS == null) loadLocalizedStrings();
    JPanel optionsPanel = new JPanel(new BorderLayout());
    
    JPanel upperPanel = new JPanel(new BorderLayout(7, 7));
    JPanel startActionPanel = new JPanel(new GridLayout(0, 1, 7, 7));
    startActionPanel.setBorder(BorderFactory.createTitledBorder(
      BorderFactory.createEtchedBorder(), actionTitleS));
    actionGroup = new ButtonGroup();
    noActionButton = new JRadioButton(noProjectS);
    noActionButton.setMnemonic(noProjectM.charAt(0));
    actionGroup.add(noActionButton);
    startActionPanel.add(noActionButton);
    newActionButton = new JRadioButton(newProjectS);
    newActionButton.setMnemonic(newProjectM.charAt(0));
    actionGroup.add(newActionButton);
    startActionPanel.add(newActionButton);
    openActionButton = new JRadioButton(lastProjectS);
    openActionButton.setMnemonic(lastProjectM.charAt(0));
    actionGroup.add(openActionButton);
    startActionPanel.add(openActionButton);
    upperPanel.add(startActionPanel, BorderLayout.WEST);
    
    JPanel gridPanel = new JPanel(new BorderLayout(3, 7));
    gridPanel.setBorder(BorderFactory.createTitledBorder(
      BorderFactory.createEtchedBorder(), diverseS));
    JLabel askAtEndLabel = new JLabel(quitQuestionS);
    askAtEndBox = new JCheckBox();
    askAtEndLabel.setLabelFor(askAtEndBox);
    askAtEndBox.setMnemonic(quitQuestionM.charAt(0));
    JLabel gridLabel = new JLabel(gridWidthS);
    gridWidthModel = new SpinnerNumberModel(10, 1, 30, 1);
    gridWidthBox = new JSpinner(gridWidthModel);
    gridWidthBox.setPreferredSize(new Dimension(
      gridWidthBox.getPreferredSize().width + 4, 
      gridWidthBox.getPreferredSize().height));
    JLabel browserLabel = new JLabel(browserS);
    browserField = new JTextField();
    browserLabel.setLabelFor(browserField);
    browserField.setToolTipText(browserT);
    JButton browserButton = new JButton("...");
    browserButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        File file = new File(System.getProperty("system.home"));
        JFileChooser chooser = new JFileChooser(file);
        chooser.setAcceptAllFileFilterUsed(true);
        int retCode = chooser.showOpenDialog(OptionsDialog.this);
        if (retCode == JFileChooser.APPROVE_OPTION) {
          File selectedFile = chooser.getSelectedFile();
          browserField.setText(selectedFile.getAbsolutePath());
        }
      }
    });
    JPanel askAtEndPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 7, 7));
    askAtEndPanel.add(askAtEndBox);
    askAtEndPanel.add(askAtEndLabel);
    JPanel gridAlignPanel1 = new JPanel(new BorderLayout(7, 7));
    gridAlignPanel1.add(askAtEndPanel, BorderLayout.NORTH);
    JPanel gridWidthPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 7, 7));
    gridWidthPanel.add(gridLabel);
    gridWidthPanel.add(gridWidthBox);
    gridAlignPanel1.add(gridWidthPanel, BorderLayout.SOUTH);
    JPanel browserPanel = new JPanel(new BorderLayout(7, 7));
    browserPanel.add(browserLabel, BorderLayout.WEST);
    browserPanel.add(browserField, BorderLayout.CENTER);
    browserPanel.add(browserButton, BorderLayout.EAST);
    gridPanel.add(browserPanel, BorderLayout.SOUTH);
    gridPanel.add(gridAlignPanel1, BorderLayout.NORTH);
    upperPanel.add(gridPanel, BorderLayout.CENTER);
    optionsPanel.add(upperPanel, BorderLayout.NORTH);
    
    JPanel pluginsPanel = new JPanel(new BorderLayout(10, 5));
    pluginsPanel.setBorder(BorderFactory.createTitledBorder(
      BorderFactory.createEtchedBorder(), pluginTitleS));
    JPanel pluginLabelsPanel = new JPanel(new GridLayout(3, 0, 15, 7));
    JPanel pluginBoxesPanel = new JPanel(new GridLayout(3, 0, 15, 7));
    JPanel pluginButtonsPanel = new JPanel(new GridLayout(3, 0, 15, 7));
    JLabel loadLabel = new JLabel(projectLoaderS);
    pluginLabelsPanel.add(loadLabel);
    systemReaderBox = new JComboBox();
    systemReaderBox.setEditable(false);
    systemReaderBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (OptionsDialog.this.control != null) try {
          OptionsDialog.this.control.executeCommand(
            OptionsCommands.CHANGE_SYSTEM_READER);
        }
        catch (NoSuchCommandException ex) {
          assert false;
        }
      }
    });
    systemReaderBox.addItem("XML-Dateien");
    systemReaderBox.setPreferredSize(new Dimension(
      systemReaderBox.getPreferredSize().width + 20, 
      systemReaderBox.getPreferredSize().height));
    pluginBoxesPanel.add(systemReaderBox);
    configureReaderButton = new JButton(configureS);
    configureReaderButton.setMnemonic(configure1M.charAt(0));
    configureReaderButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try {
          OptionsDialog.this.control.executeCommand(
            OptionsCommands.SHOW_SYSTEM_READER_CONFIG);
        }
        catch (NoSuchCommandException ex) {
          assert false;
        }
      }
    });
    pluginButtonsPanel.add(configureReaderButton);

    JLabel writeLabel = new JLabel(projectSaverS);
    pluginLabelsPanel.add(writeLabel);
    systemWriterBox = new JComboBox();
    systemWriterBox.setEditable(false);
    systemWriterBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (OptionsDialog.this.control != null) try {
          OptionsDialog.this.control.executeCommand(
            OptionsCommands.CHANGE_SYSTEM_WRITER);
        }
        catch (NoSuchCommandException ex) {
          assert false;
        }
      }
    });
    pluginBoxesPanel.add(systemWriterBox);
    configureWriterButton = new JButton(configureS);
    configureWriterButton.setMnemonic(configure2M.charAt(0));
    configureWriterButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try {
          OptionsDialog.this.control.executeCommand(
            OptionsCommands.SHOW_SYSTEM_WRITER_CONFIG);
        }
        catch (NoSuchCommandException ex) {
          assert false;
        }
      }
    });
    pluginButtonsPanel.add(configureWriterButton);
    pluginsPanel.add(pluginLabelsPanel, BorderLayout.WEST);
    pluginsPanel.add(pluginBoxesPanel, BorderLayout.CENTER);
    pluginsPanel.add(pluginButtonsPanel, BorderLayout.EAST);
    
    JLabel loadAlgLabel = new JLabel(algLoaderS);
    pluginLabelsPanel.add(loadAlgLabel);
    algReaderBox = new JComboBox();
    algReaderBox.setEditable(false);
    algReaderBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        if (OptionsDialog.this.control != null) try {
          OptionsDialog.this.control.executeCommand(
            OptionsCommands.CHANGE_ALGORITHM_READER);
        }
        catch (NoSuchCommandException ex) {
          assert false;
        }
      }
    });
    algReaderBox.addItem("XML-Dateien");
    algReaderBox.setPreferredSize(new Dimension(
      algReaderBox.getPreferredSize().width + 20, 
      algReaderBox.getPreferredSize().height));
    pluginBoxesPanel.add(algReaderBox);
    configureAlgReaderButton = new JButton(configureS);
    configureAlgReaderButton.setMnemonic(configure3M.charAt(0));
    configureAlgReaderButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try {
          OptionsDialog.this.control.executeCommand(
            OptionsCommands.SHOW_ALG_READER_CONFIG);
        }
        catch (NoSuchCommandException ex) {
          assert false;
        }
      }
    });
    pluginButtonsPanel.add(configureAlgReaderButton);

    JPanel demoPanel = new JPanel(new BorderLayout());
    demoPanel.setBorder(BorderFactory.createTitledBorder(
      BorderFactory.createEtchedBorder(), demoTitleS));
    JPanel cycleTimePanel = new JPanel(new FlowLayout());
    cycleTimePanel.add(new JLabel(minCycleTimeS));
    cycleTimeBox = new JTextField();
    cycleTimeBox.setHorizontalAlignment(JTextField.RIGHT);
    cycleTimeBox.setPreferredSize(
      new Dimension(33, (int) cycleTimeBox.getPreferredSize().getHeight()));
    cycleTimePanel.add(cycleTimeBox);
    if (Core.getProjectManager().isInDemoState()) 
      cycleTimeBox.setEnabled(false);
    demoPanel.add(cycleTimePanel, BorderLayout.WEST);
    JPanel messageSpeedPanel = new JPanel(new FlowLayout());
    messageSpeedPanel.add(new JLabel(msgSpeedS));
    messageSpeedBox = new JTextField();
    messageSpeedBox.setPreferredSize(
      new Dimension(35, (int) cycleTimeBox.getPreferredSize().getHeight()));
    messageSpeedBox.setHorizontalAlignment(JTextField.RIGHT);
    messageSpeedPanel.add(messageSpeedBox);
    if (Core.getProjectManager().isInDemoState()) 
      messageSpeedBox.setEnabled(false);
    demoPanel.add(messageSpeedPanel, BorderLayout.EAST);
    
    JPanel southPanel = new JPanel(new BorderLayout());
    southPanel.add(pluginsPanel, BorderLayout.SOUTH);
    southPanel.add(demoPanel, BorderLayout.NORTH);
    
    optionsPanel.add(southPanel, BorderLayout.SOUTH);
    
    JLabel fillerLabel = new JLabel("");
    fillerLabel.setSize(10, 20);
    optionsPanel.add(fillerLabel, BorderLayout.CENTER);
    JPanel buttonsPanel2 = 
      new JPanel(new FlowLayout(FlowLayout.CENTER, 15, 10));
    JPanel buttonsPanel =  new JPanel(new GridLayout(0, 2, 15, 10));
    okButton = new JButton(okS);
    okButton.setMnemonic(okM.charAt(0));
    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (OptionsDialog.this.control.canClose(true)) {
          OptionsDialog.this.dispose();
        }
        else {
          String errors = OptionsDialog.this.control.getErrors();
          JOptionPane.showMessageDialog(OptionsDialog.this, errors, 
            errorsTitleS, JOptionPane.ERROR_MESSAGE);
        }
      }
    });
    buttonsPanel.add(okButton);
    cancelButton = new JButton(cancelS);
    cancelButton.setMnemonic(cancelM.charAt(0));
    cancelButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (OptionsDialog.this.control.canClose(false)) {
          OptionsDialog.this.dispose();
        }
      }
    });
    this.getRootPane().setDefaultButton(okButton);
    buttonsPanel.add(cancelButton);
    buttonsPanel2.add(buttonsPanel);
    this.getContentPane().setLayout(new BorderLayout());
    this.getContentPane().add(optionsPanel, BorderLayout.CENTER);
    this.getContentPane().add(buttonsPanel2, BorderLayout.SOUTH);
    this.setTitle(titleS);
    this.pack();
    this.setLocation(Toolkit.getDefaultToolkit().getScreenSize().width / 2
      - this.getSize().width / 2, 
      Toolkit.getDefaultToolkit().getScreenSize().height / 2
      - this.getSize().height / 2);
  }
  
  /**
   * Sets the dialog control
   * @param control the control
   */
  public void setDialogControl(DialogControl control) {
    this.control = control;
  }

  /* (non-Javadoc)
   * @see fdda.dialog.Dialog#disableCommand(int)
   */
  public void disableCommand(int command) throws NoSuchCommandException {
    if (command == OptionsCommands.SHOW_SYSTEM_READER_CONFIG) {
      configureReaderButton.setEnabled(false);
    }
    else if (command == OptionsCommands.SHOW_SYSTEM_WRITER_CONFIG) {
      configureWriterButton.setEnabled(false);
    }
    else if (command == OptionsCommands.CHANGE_SYSTEM_READER) {
      systemReaderBox.setEnabled(false);
    }
    else if (command == OptionsCommands.CHANGE_SYSTEM_WRITER) {
      systemWriterBox.setEnabled(false);
    }
    else if (command == OptionsCommands.CHANGE_ALGORITHM_READER) {
      algReaderBox.setEnabled(false);
    }
    else if (command == OptionsCommands.SHOW_ALG_READER_CONFIG) {
      configureAlgReaderButton.setEnabled(false);
    }
    else {
      throw new NoSuchCommandException();
    }
  }

  /* (non-Javadoc)
   * @see fdda.dialog.Dialog#enableCommand(int)
   */
  public void enableCommand(int command) throws NoSuchCommandException {
    if (command == OptionsCommands.SHOW_SYSTEM_READER_CONFIG) {
      configureReaderButton.setEnabled(true);
    }
    else if (command == OptionsCommands.SHOW_SYSTEM_WRITER_CONFIG) {
      configureWriterButton.setEnabled(true);
    }
    else if (command == OptionsCommands.CHANGE_SYSTEM_READER) {
      systemReaderBox.setEnabled(true);
    }
    else if (command == OptionsCommands.CHANGE_SYSTEM_WRITER) {
      systemWriterBox.setEnabled(true);
    }
    else if (command == OptionsCommands.CHANGE_ALGORITHM_READER) {
      algReaderBox.setEnabled(true);
    }
    else if (command == OptionsCommands.SHOW_ALG_READER_CONFIG) {
      configureAlgReaderButton.setEnabled(true);
    }
    else {
      throw new NoSuchCommandException();
    }
  }

  /* (non-Javadoc)
   * @see fdda.dialog.Dialog#getData()
   */
  public Object getData() {
    if (options != null) {
      options.setGridWidth(gridWidthModel.getNumber().intValue());
      if (noActionButton.isSelected()) {
        options.setStartAction(MainFrame.NOTHING);
      }
      else if (newActionButton.isSelected()) {
        options.setStartAction(MainFrame.NEW_PROJECT);
      }
      else if (openActionButton.isSelected()) {
        options.setStartAction(MainFrame.LAST_PROJECT);
      }
      options.setAskAtClosing(askAtEndBox.isSelected());
      options.setSystemReader(systemReaderBox.getSelectedItem().toString());
      options.setSystemWriter(systemWriterBox.getSelectedItem().toString());
      options.setAlgorithmReader(algReaderBox.getSelectedItem().toString());
      options.setMinCycleTime(cycleTimeBox.getText());
      options.setMessageSpeed(messageSpeedBox.getText());
      options.setBrowserCommand(browserField.getText());
    }
    return options;
  }

  /* (non-Javadoc)
   * @see fdda.dialog.Dialog#updateObject(java.lang.Object)
   */
  public void updateObject(Object data) {
    if (data != null) {
      options = (Options) data;
      askAtEndBox.setSelected(options.shallAskAtClosing());
      gridWidthModel.setValue(new Integer(options.getGridWidth()));
      systemReaderBox.removeAllItems();
      Iterator it = options.getPossibleReaders().iterator();
      while (it.hasNext()) {
        systemReaderBox.addItem(it.next());
      }
      systemReaderBox.setPreferredSize(new Dimension(
        systemReaderBox.getPreferredSize().width + 20, 
        systemReaderBox.getPreferredSize().height));
      systemReaderBox.setSelectedItem(options.getSystemReader());

      algReaderBox.removeAllItems();
      it = options.getPossibleAlgorithmReaders().iterator();
      while (it.hasNext()) {
        algReaderBox.addItem(it.next());
      }
      algReaderBox.setPreferredSize(new Dimension(
        algReaderBox.getPreferredSize().width + 20, 
        algReaderBox.getPreferredSize().height));
      algReaderBox.setSelectedItem(options.getAlgorithmReader());

      systemWriterBox.removeAllItems();
      it = options.getPossibleWriters().iterator();
      while (it.hasNext()) {
        systemWriterBox.addItem(it.next());
      }
      systemWriterBox.setPreferredSize(new Dimension(
        systemWriterBox.getPreferredSize().width + 20,
        systemWriterBox.getPreferredSize().height));
      systemWriterBox.setSelectedItem(options.getSystemWriter());
      if (options.getStartAction() == MainFrame.NOTHING) {
        noActionButton.setSelected(true);
      }
      else if (options.getStartAction() == MainFrame.NEW_PROJECT) {
        newActionButton.setSelected(true);
      }
      else if (options.getStartAction() == MainFrame.LAST_PROJECT) {
        openActionButton.setSelected(true);
      }
      cycleTimeBox.setText("" + options.getMinCycleTime());
      messageSpeedBox.setText("" + options.getMessageSpeed());
      browserField.setText(options.getBrowserCommand());
    }
  }
  
  /**
   * Loads the localized strings.
   */
  private static void loadLocalizedStrings() {
    ResourceBundle bundle = 
      Localization.getResourceBundleForPackage(OptionsDialog.class);
    titleS = Localization.getLocalizedString(bundle, 
      "optionsTitleS", "Settings");
    actionTitleS = Localization.getLocalizedString(bundle, 
      "actionTitleS", "When FDDA starts...");
    noProjectS = Localization.getLocalizedString(bundle, 
      "noProjectS", "open no project");
    lastProjectS = Localization.getLocalizedString(bundle, 
      "lastProjectS", "re-open the last project");
    newProjectS = Localization.getLocalizedString(bundle, 
      "newProjectS", "open a new project");
    diverseS = Localization.getLocalizedString(bundle, 
      "diverseS", "Miscellaneous");
    quitQuestionS = Localization.getLocalizedString(bundle, 
      "quitQuestionS", "Warn before closing FDDA? ");
    gridWidthS = Localization.getLocalizedString(bundle, 
      "gridWidthS", "Spacing of grid: ");
    demoTitleS = Localization.getLocalizedString(bundle, 
      "demoTitleS", "Demonstration");
    minCycleTimeS = Localization.getLocalizedString(bundle, 
      "minCycleTimeS", "Minimum cycle time (ms): ");
    msgSpeedS = Localization.getLocalizedString(bundle, 
      "msgSpeedS", "Message speed: ");
    pluginTitleS = Localization.getLocalizedString(bundle, 
      "pluginTitleS", "Plug-ins");
    configureS = Localization.getLocalizedString(bundle, 
      "configureS", "Configure...");
    configure1M = Localization.getLocalizedString(bundle, 
      "configure1M", "F");
    configure2M = Localization.getLocalizedString(bundle, 
      "configure2M", "G");
    configure3M = Localization.getLocalizedString(bundle, 
      "configure3M", "U");
    projectLoaderS = Localization.getLocalizedString(bundle, 
      "projectLoaderS", "Plug-in to load projects: ");
    projectSaverS = Localization.getLocalizedString(bundle, 
      "projectSaverS", "Plug-in to save projects: ");
    algLoaderS = Localization.getLocalizedString(bundle, 
      "algLoaderS", "Plug-in to load algorithms: ");
    lastProjectM = Localization.getLocalizedString(bundle, 
      "lastProjectM", "L");
    noProjectM = Localization.getLocalizedString(bundle, 
      "noProjectM", "N");
    newProjectM = Localization.getLocalizedString(bundle, 
      "newProjectM", "E");
    quitQuestionM = Localization.getLocalizedString(bundle, 
      "quitQuestionM", "W");
    okS = Localization.getLocalizedString(bundle, 
      "okS", "Ok");
    okM = Localization.getLocalizedString(bundle, 
      "okM", "O");
    cancelS = Localization.getLocalizedString(bundle, 
      "cancelS", "Cancel");
    cancelM = Localization.getLocalizedString(bundle, 
      "cancelM", "C");
    errorsTitleS = Localization.getLocalizedString(bundle, 
      "errorsTitleS", "Errors");
    browserS = Localization.getLocalizedString(bundle, 
      "browserS", "Web Browser: ");
    browserT = Localization.getLocalizedString(bundle,
      "browserT", "Insert %u for the URL parameter");
  }

}
