/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.HeadlessException;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.NumberFormat;
import java.util.ResourceBundle;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import fdda.dialog.Dialog;
import fdda.dialog.NoSuchCommandException;
import fdda.dialog.StatisticsDialogControl;
import fdda.utils.Localization;

import fdda.core.demonstration.Statistics;

/**
 * Non-modal dialog that shows statistics about the last demonstration of a
 * project. 
 * @author Jrg Rdenauer
 */
public class StatisticsDialog extends JDialog implements Dialog {
  
  private static String msgSentS = null;
  private static String msgSentNS = null;
  private static String msgReceivedS = null;
  private static String msgSentES = null;
  private static String activeNodesS = null;
  private static String pausedNodesS = null;
  private static String activeMsgS = null;
  private static String startedNodesS = null;
  private static String activeNodes2S = null;
  private static String usedNodesS = null;
  private static String usedMsgS = null;
  private static String failedNodesS = null;
  private static String failedConnsS = null;
  
  private static String closeS = null;
  private static String refreshS = null;
  private static String closeM = null;
  private static String refreshM = null;
  
  private static String titleS = null;
  
  /** the statistics */
  private Statistics stats;
  /** the dialog control */
  private StatisticsDialogControl control;
  
  private JButton okButton;
  private JButton refreshButton;
  
  private JLabel msgSentLabel, msgSentNLabel, msgReceivedLabel;
  private JLabel msgReceivedPercentLabel, msgSentELabel;
  private JLabel activeNodesLabel, pausedNodesLabel, activeMsgLabel;
  private JLabel startedNodesLabel, activeNodes2Label, activeNodesPercentLabel;
  private JLabel usedNodesLabel, usedNodesPercentLabel, usedMsgLabel;
  private JLabel failedNodesLabel, failedConnsLabel, usedMsgPercentLabel;
   

  /**
   * Constructor
   * @param owner owner of the dialog
   * @throws HeadlessException
   */  
  public StatisticsDialog(Frame owner) throws HeadlessException  {
    super(owner);
    if (msgSentS == null) {
      loadLocalizedStrings();
    }
    this.getContentPane().setLayout(new BorderLayout());
    JLabel leftLabel = new JLabel("");
    leftLabel.setPreferredSize(new Dimension(12, 12));
    this.getContentPane().add(leftLabel, BorderLayout.WEST);
    JLabel rightLabel = new JLabel("");
    rightLabel.setPreferredSize(new Dimension(12, 12));
    this.getContentPane().add(rightLabel, BorderLayout.EAST);
    JLabel topLabel = new JLabel("");
    topLabel.setPreferredSize(new Dimension(8, 8));
    this.getContentPane().add(topLabel, BorderLayout.NORTH);
    
    JPanel statisticsPanel = new JPanel(new BorderLayout(10, 10));
    JLabel leftLabel1 = new JLabel("");
    leftLabel1.setPreferredSize(new Dimension(8, 8));
    statisticsPanel.add(leftLabel1, BorderLayout.WEST);
    JLabel leftLabel2 = new JLabel("");
    leftLabel2.setPreferredSize(new Dimension(8, 8));
    statisticsPanel.add(leftLabel2, BorderLayout.EAST);
    JLabel leftLabel3 = new JLabel("");
    leftLabel3.setPreferredSize(new Dimension(8, 8));
    statisticsPanel.add(leftLabel3, BorderLayout.NORTH);
    JLabel leftLabel4 = new JLabel("");
    leftLabel4.setPreferredSize(new Dimension(8, 8));
    statisticsPanel.add(leftLabel4, BorderLayout.SOUTH);
    JPanel dataPanel = new JPanel(new BorderLayout(5, 5));
    statisticsPanel.add(dataPanel, BorderLayout.CENTER);
    statisticsPanel.setBorder(BorderFactory.createEtchedBorder());
    JPanel column1 = new JPanel(new GridLayout(16, 1, 5, 5));
    JPanel column2and3 = new JPanel(new BorderLayout(5, 5));
    JPanel column2 = new JPanel(new GridLayout(16, 1, 5, 5));
    JPanel column3 = new JPanel(new GridLayout(16, 1, 5, 5));
    dataPanel.add(column1, BorderLayout.CENTER);
    dataPanel.add(column2and3, BorderLayout.EAST);
    column2and3.add(column2, BorderLayout.WEST);
    column2and3.add(column3, BorderLayout.EAST);
    
    column1.add(new JLabel(msgSentS));
    msgSentLabel = new JLabel();
    msgSentLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    msgSentLabel.setOpaque(true);
    msgSentLabel.setBackground(Color.CYAN);
    column2.add(msgSentLabel);
    column3.add(new JLabel());
    
    column1.add(new JLabel(msgSentNS));
    msgSentNLabel = new JLabel();
    msgSentNLabel.setBackground(Color.CYAN);
    msgSentNLabel.setOpaque(true);
    msgSentNLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    column2.add(msgSentNLabel);
    column3.add(new JLabel());
    
    column1.add(new JLabel(msgSentES));
    msgSentELabel = new JLabel();
    msgSentELabel.setBackground(Color.CYAN);
    msgSentELabel.setHorizontalAlignment(SwingConstants.RIGHT);
    msgSentELabel.setOpaque(true);
    column2.add(msgSentELabel);
    column3.add(new JLabel());
    
    column1.add(new JLabel());
    column2.add(new JLabel());
    column3.add(new JLabel());
    
    column1.add(new JLabel(activeNodesS));
    activeNodesLabel = new JLabel();
    activeNodesLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    activeNodesLabel.setBackground(Color.YELLOW);
    activeNodesLabel.setOpaque(true);
    column2.add(activeNodesLabel);
    column3.add(new JLabel());

    column1.add(new JLabel(pausedNodesS));
    pausedNodesLabel = new JLabel();
    pausedNodesLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    pausedNodesLabel.setBackground(Color.YELLOW);
    pausedNodesLabel.setOpaque(true);
    column2.add(pausedNodesLabel);
    column3.add(new JLabel());

    column1.add(new JLabel(activeMsgS));
    activeMsgLabel = new JLabel();
    activeMsgLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    activeMsgLabel.setBackground(Color.YELLOW);
    activeMsgLabel.setOpaque(true);
    column2.add(activeMsgLabel);
    column3.add(new JLabel());

    column1.add(new JLabel());
    JLabel fillLabel1 = new JLabel();
    fillLabel1.setPreferredSize(new Dimension(35, 10));
    column2.add(fillLabel1);
    JLabel fillLabel2 = new JLabel();
    fillLabel2.setPreferredSize(new Dimension(45, 10));
    column3.add(fillLabel2);
    
    column1.add(new JLabel(startedNodesS));
    startedNodesLabel = new JLabel();
    startedNodesLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    startedNodesLabel.setBackground(Color.CYAN);
    startedNodesLabel.setOpaque(true);
    column2.add(startedNodesLabel);
    column3.add(new JLabel());
    
    column1.add(new JLabel(activeNodes2S));
    activeNodes2Label = new JLabel();
    activeNodes2Label.setHorizontalAlignment(SwingConstants.RIGHT);
    activeNodes2Label.setBackground(Color.CYAN);
    activeNodes2Label.setOpaque(true);
    column2.add(activeNodes2Label);
    activeNodesPercentLabel = new JLabel();
    activeNodesPercentLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    activeNodesPercentLabel.setBackground(Color.CYAN);
    activeNodesPercentLabel.setOpaque(true);
    column3.add(activeNodesPercentLabel);
    
    column1.add(new JLabel(usedNodesS));
    usedNodesLabel = new JLabel();
    usedNodesLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    usedNodesLabel.setBackground(Color.CYAN);
    usedNodesLabel.setOpaque(true);
    column2.add(usedNodesLabel);
    usedNodesPercentLabel = new JLabel();
    usedNodesPercentLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    usedNodesPercentLabel.setBackground(Color.CYAN);
    usedNodesPercentLabel.setOpaque(true);
    column3.add(usedNodesPercentLabel);

    column1.add(new JLabel(usedMsgS));
    usedMsgLabel = new JLabel();
    usedMsgLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    usedMsgLabel.setBackground(Color.CYAN);
    usedMsgLabel.setOpaque(true);
    column2.add(usedMsgLabel);
    usedMsgPercentLabel = new JLabel();
    usedMsgPercentLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    usedMsgPercentLabel.setBackground(Color.CYAN);
    usedMsgPercentLabel.setOpaque(true);
    column3.add(usedMsgPercentLabel);
    
    column1.add(new JLabel());
    column2.add(new JLabel());
    column3.add(new JLabel());
    
    column1.add(new JLabel(failedNodesS));
    failedNodesLabel = new JLabel();
    failedNodesLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    failedNodesLabel.setBackground(Color.ORANGE);
    failedNodesLabel.setOpaque(true);
    column2.add(failedNodesLabel);
    column3.add(new JLabel());
    
    column1.add(new JLabel(failedConnsS));
    failedConnsLabel = new JLabel();
    failedConnsLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    failedConnsLabel.setBackground(Color.ORANGE);
    failedConnsLabel.setOpaque(true);
    column2.add(failedConnsLabel);
    column3.add(new JLabel());
 
    column1.add(new JLabel(msgReceivedS));
    msgReceivedLabel = new JLabel();
    msgReceivedLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    msgReceivedLabel.setBackground(Color.ORANGE);
    msgReceivedLabel.setOpaque(true);
    column2.add(msgReceivedLabel);
    msgReceivedPercentLabel = new JLabel();
    msgReceivedPercentLabel.setBackground(Color.ORANGE);
    msgReceivedPercentLabel.setHorizontalAlignment(SwingConstants.RIGHT);
    msgReceivedPercentLabel.setOpaque(true);
    column3.add(msgReceivedPercentLabel);
    
    this.getContentPane().add(statisticsPanel, BorderLayout.CENTER);
    
    JPanel buttonsPanel = new JPanel(new FlowLayout(FlowLayout.CENTER, 50, 20));
    okButton = new JButton(closeS);
    okButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (StatisticsDialog.this.control.canClose(true)) {
          StatisticsDialog.this.dispose();
        }
        else {
          String errors = StatisticsDialog.this.control.getErrors();
          JOptionPane.showMessageDialog(StatisticsDialog.this, errors, "Errors", 
            JOptionPane.ERROR_MESSAGE);
        }
      }
    });
    okButton.setMnemonic(closeM.charAt(0));
    buttonsPanel.add(okButton);
    this.getRootPane().setDefaultButton(okButton);
    refreshButton = new JButton(refreshS);
    refreshButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try {
          StatisticsDialog.this.control.executeCommand(
            StatisticsDialogControl.COMMAND_REFRESH);
        }
        catch (NoSuchCommandException ex) {
          assert false;
        }
      }
    });
    refreshButton.setMnemonic(refreshM.charAt(0));
    buttonsPanel.add(refreshButton);
    this.getContentPane().add(buttonsPanel, BorderLayout.SOUTH);
    
    this.setTitle(titleS);
    this.pack();
    this.setLocation(Toolkit.getDefaultToolkit().getScreenSize().width / 2
      - this.getSize().width / 2, 
      Toolkit.getDefaultToolkit().getScreenSize().height / 2
      - this.getSize().height / 2);    
  }
  
  public void setStatisticsDialogControl(StatisticsDialogControl control) {
    this.control = control;
  }

  /* (non-Javadoc)
   * @see fdda.dialog.Dialog#updateObject(java.lang.Object)
   */
  public void updateObject(Object data) {
    stats = (Statistics) data;
    int msgTotal = stats.getSentMessages();
    int msgReceived = stats.getReceivedMessages();
    int nodesTotal = stats.getPresentNodes();
    int connsTotal = stats.getPresentConnections();
    
    NumberFormat nf = NumberFormat.getNumberInstance(Localization.getLocale());
    NumberFormat nfp = NumberFormat.getPercentInstance(Localization.getLocale());
    nf.setMaximumFractionDigits(2);
    nfp.setMaximumFractionDigits(2);
    
    msgSentLabel.setText(nf.format(msgTotal));
    msgSentNLabel.setText(nodesTotal > 0 ? 
      nf.format(msgTotal / (float) nodesTotal) : "-");
    msgSentELabel.setText(connsTotal > 0 ? 
      nf.format(msgTotal / (float) connsTotal)  : "-");
    
    msgReceivedLabel.setText("" + msgReceived);
    msgReceivedPercentLabel.setText(msgTotal > 0 ? 
      nfp.format(msgReceived / (float)msgTotal) : "-");
    
    activeNodesLabel.setText(nf.format(stats.getCurrentlyActiveNodes()));
    pausedNodesLabel.setText(nf.format(stats.getCurrentlyPausedNodes()));
    activeMsgLabel.setText(nf.format(stats.getCurrentlyActiveMessages()));
    
    startedNodesLabel.setText(nf.format(stats.getStartedNodes()));
    int activeNodes = stats.getActiveNodes();
    activeNodes2Label.setText(nf.format(activeNodes));
    activeNodesPercentLabel.setText(nodesTotal > 0 ? 
      nfp.format(activeNodes / (float)nodesTotal) : "-");
    int usedNodes = stats.getUsedNodes();
    usedNodesLabel.setText(nf.format(usedNodes));
    usedNodesPercentLabel.setText(nodesTotal > 0 ? 
      nfp.format(usedNodes / (float)nodesTotal) : "-");
    int usedConns = stats.getUsedConnections();
    usedMsgLabel.setText(nf.format(usedConns));
    usedMsgPercentLabel.setText(connsTotal > 0 ? 
      nfp.format(usedConns / (float)connsTotal) : "-");
    
    failedNodesLabel.setText(nf.format(stats.getNodeFailures()));
    failedConnsLabel.setText(nf.format(stats.getConnectionFailures()));
  }

  /* (non-Javadoc)
   * @see fdda.dialog.Dialog#getData()
   */
  public Object getData() {
    return stats;
  }

  /* (non-Javadoc)
   * @see fdda.dialog.Dialog#disableCommand(int)
   */
  public void disableCommand(int command) throws NoSuchCommandException {
    if (command == StatisticsDialogControl.COMMAND_REFRESH) {
      refreshButton.setEnabled(false);
    }
    else {
      throw new NoSuchCommandException();
    }
  }

  /* (non-Javadoc)
   * @see fdda.dialog.Dialog#enableCommand(int)
   */
  public void enableCommand(int command) throws NoSuchCommandException {
    if (command == StatisticsDialogControl.COMMAND_REFRESH) {
      refreshButton.setEnabled(true);
    }
    else {
      throw new NoSuchCommandException();
    }
  }
  
  /**
   * Loads the localized strings.
   */
  private static void loadLocalizedStrings() {
    ResourceBundle bundle = 
      Localization.getResourceBundleForPackage(StatisticsDialog.class);
    msgSentS = Localization.getLocalizedString(bundle, 
      "msgSentS", "Messages sent: ");
    msgSentNS = Localization.getLocalizedString(bundle, 
      "msgSentNS", "Messages per node: ");
    msgReceivedS = Localization.getLocalizedString(bundle, 
      "msgReceivedS", "Messages received: ");
    msgSentES = Localization.getLocalizedString(bundle, 
      "msgSentES", "Messages per connection: ");
    activeNodesS = Localization.getLocalizedString(bundle, 
      "activeNodesS", "Currently active nodes: ");
    pausedNodesS = Localization.getLocalizedString(bundle, 
      "pausedNodesS", "Currently paused nodes: ");
    activeMsgS = Localization.getLocalizedString(bundle, 
      "activeMsgS", "Currently active messages: ");
    startedNodesS = Localization.getLocalizedString(bundle, 
      "startedNodesS", "Started nodes: ");
    activeNodes2S = Localization.getLocalizedString(bundle, 
      "activeNodes2S", "Overall active nodes: ");
    usedNodesS = Localization.getLocalizedString(bundle, 
      "usedNodesS", "Used nodes: ");
    usedMsgS = Localization.getLocalizedString(bundle, 
      "usedMsgS", "Used connections: ");
    failedNodesS = Localization.getLocalizedString(bundle, 
      "failedNodesS", "Failed nodes: ");
    failedConnsS = Localization.getLocalizedString(bundle, 
      "failedConnsS", "Failed connections: ");
    closeS = Localization.getLocalizedString(bundle, 
      "closeS", "Close");
    refreshS = Localization.getLocalizedString(bundle, 
      "refreshS", "Refresh");
    closeM = Localization.getLocalizedString(bundle, 
      "closeM", "C");
    refreshM = Localization.getLocalizedString(bundle, 
      "refreshM", "R");
    titleS = Localization.getLocalizedString(bundle, 
      "statsTitleS", "Statistics");
  }

}
