/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing;

import fdda.graphics.GraphicalSystem;

import fdda.core.Core;
import fdda.core.projects.Project;

import javax.swing.JInternalFrame;

import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

/**
 * An internal frame that contains a graphical system.
 * @author Jrg Rdenauer
 */
public class SystemFrame extends JInternalFrame {

  /** constant: no window open */
  public static final int SF_NONE = 0;

  /** constant: project frame */
  public static final int SF_PROJECT = 1;

  /** the graphical system */
  protected GraphicalSystem graphicalSystem;

  /**
   * Constructor
   * @param title title of the frame
   */
  public SystemFrame(String title) {
    super(title, true, true, true, true);

    this.addInternalFrameListener(new InternalFrameAdapter() {
      public void internalFrameClosing(InternalFrameEvent e) {
        frameClosing();
      }

      public void internalFrameActivated(InternalFrameEvent e) {
        if (graphicalSystem != null) {
           Core.getProjectManager().setActiveProject(
            graphicalSystem.getProject());
        }
      }
    });
  }

  /**
   * Returns the graphical system
   * @return the graphical system
   */
  public GraphicalSystem getGraphicalSystem(){
    return this.graphicalSystem;
  }

  /**
   * Returns the project to which the system belongs
   * @return the project
   */
  public Project getProject() {
    return graphicalSystem == null ? null : graphicalSystem.getProject();
  }

  /**
   * Sets the graphical system
   * @param graphSystem the graphical system
   */
  public void setGraphicalSystem(GraphicalSystem graphSystem) {
    this.graphicalSystem = graphSystem;
    this.setFrameIcon(new javax.swing.ImageIcon("images/mfenster.gif"));
  }

  /**
   * Closes the frame.
   */
  protected void frameClosing() {
    dispose();
  }

}
