/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing.properties;

import javax.swing.table.AbstractTableModel;
import java.util.Vector;
import java.util.Iterator;

import fdda.core.Core;
import fdda.core.projects.Project;

import fdda.model.Property;
import fdda.utils.Localization;

/**
 * The table model for the table of the object inspector. Holds names of the 
 * properties in one column and values in the other column.
 * @author Jrg Rdenauer
 */
public class ObjTableModel extends AbstractTableModel {


  /** column headers*/
  protected static final String[] COLUMN_NAME = new String[] {
    Localization.getLocalizedString(ObjTableModel.class, "name_header", "Name"), 
    Localization.getLocalizedString(ObjTableModel.class, "value_header", 
      "Value")
  };

  /** the data */
  private Vector data = new Vector();

  /**
   * default constructor
   */
  public ObjTableModel() {
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.table.TableModel#getColumnCount()
   */
  public int getColumnCount() {
    return 2;
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.table.TableModel#getValueAt(int, int)
   */
  public Object getValueAt(int row, int column) {
    return (Property)data.get(row);
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.table.TableModel#getRowCount()
   */
  public int getRowCount() {
    return data.size();
  }

  /**
   * Fills the model with the values
   * @param properties iterator over the properties
   */
  public void setAllValues(Iterator properties) {
    data.clear();
    if (properties != null) {
      Property myTempProperty = null;
      while (properties.hasNext()) {
        myTempProperty = (Property) properties.next();
        if (myTempProperty.isVisible()) {
          data.add(myTempProperty);
        }
      }
    }
    fireTableDataChanged();
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.table.TableModel#getColumnName(int)
   */
  public String getColumnName(int column) {
    return COLUMN_NAME[column];
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.table.TableModel#isCellEditable(int, int)
   */
  public boolean isCellEditable(int row, int column) {
    if (column == 0) {
      return false;
    } 
    else {
      Property cellProp = (Property) data.get(row);
      // the property may not be editable if the project is demonstrated
      Boolean isFixed = (Boolean) 
        cellProp.getFacet(Property.FIXED_WHILE_RUNNING);
      return cellProp.isModifiable() &&
        ((Core.getProjectManager().getActiveProject().getState()
          != Project.STATE_DEMONSTRATION)
          || isFixed == null || !isFixed.booleanValue());
    }
  }

}
