/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing.properties;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextArea;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.Font;

import java.util.LinkedList;

import fdda.core.Core;
import fdda.graphics.SelectionListener;
import fdda.graphics.Selection;
import fdda.graphics.SelectionEvent;

import fdda.model.PropertyEvent;
import fdda.model.PropertyHolder;
import fdda.model.PropertyListener;
import fdda.utils.Localization;


/**
 * Panel used to edit property values. It contains a table with two columns: one
 * with the names and the others with the values of the properties. 
 * The ObjectInspector is a singleton.
 * @author Jrg Rdenauer
 */
public class ObjectInspector extends JPanel 
  implements SelectionListener, PropertyListener {

  /** the instance (singleton) */
  private static ObjectInspector instance = null;

  /** the currently selected component */
  private PropertyHolder propertyHolder;

  /** the selection */
  private Selection theSelection = null;

  /** the table */
  private JTable myAttributeTable = null;

  /** the table model */
  private ObjTableModel myTableModel = null;

  /** the short help area */
  private JTextArea shortHelp = null;

  /**
   * default Konstructor
   */
  protected ObjectInspector() {
    // register as listener at the selection to be able to react to changes
    theSelection 
      = fdda.graphics.ComponentFactory.getInstance().getComponentSelection();
    theSelection.add(this);
    // initializes graphics
    shortHelp = new JTextArea("");
    shortHelp.setLineWrap(true);
    shortHelp.setWrapStyleWord(true);
    shortHelp.setAlignmentX(JTextArea.TOP_ALIGNMENT);
    shortHelp.setPreferredSize(new Dimension(100,20));
    shortHelp.setFont(Font.decode("Dialog-PLAIN-11"));
  
    myTableModel = new ObjTableModel();
    myAttributeTable = new JTable(myTableModel);
    myAttributeTable.setRowHeight(22);
    myAttributeTable.setDefaultRenderer(Object.class,
                                        new PanelRenderer(shortHelp));
    PanelEditor editor = new PanelEditor();
  
    myAttributeTable.setDefaultEditor(Object.class, editor);
  
    this.setLayout(new BorderLayout());
    add(new JLabel(Localization.getLocalizedString(getClass(), "properties", 
      "Properties")), BorderLayout.NORTH);
    JSplitPane centerSplit = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
    centerSplit.setDividerLocation(0.8);
    centerSplit.setResizeWeight(0.8);
    centerSplit.setTopComponent(new JScrollPane(myAttributeTable));
    centerSplit.setBottomComponent(new JScrollPane(shortHelp));
    add(centerSplit, BorderLayout.CENTER);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.SelectionListener#selectionChanged(fdda.graphics.SelectionEvent)
   */
  public void selectionChanged(SelectionEvent e) {
    if (this.propertyHolder != null) {
      this.propertyHolder.removePropertyListener(this);
    }
    if (myAttributeTable != null) {
      // remove old editor
       myAttributeTable.removeEditor();
    }
    if (shortHelp != null) {
        shortHelp.setText("");
    }
    Object currentComponent = theSelection.getSelectedComponent();
    if (currentComponent instanceof PropertyHolder) {
      this.propertyHolder = (PropertyHolder) currentComponent;
      propertyHolder.addPropertyListener(this);
      myTableModel.setAllValues(propertyHolder.getProperties());
    }
    else {
      this.propertyHolder = null;
      myTableModel.setAllValues((new LinkedList()).iterator());
    }
  }

  /**
   * Returns the object inspector
   * @return the object inspector
   */
  public static ObjectInspector getInstance() {
    if (instance == null) {
      instance = new ObjectInspector();
    }
    return instance;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyListener#propertyChanged(fdda.model.PropertyEvent)
   */
  public void propertyChanged(PropertyEvent pe) {
    myTableModel.setAllValues(propertyHolder.getProperties());
    Core.getProjectManager().getActiveProject().setChanged(true);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyListener#propertyAdded(fdda.model.PropertyEvent)
   */
  public void propertyAdded(PropertyEvent pe) {
    myTableModel.setAllValues(propertyHolder.getProperties());
    Core.getProjectManager().getActiveProject().setChanged(true);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyListener#propertyRemoved(fdda.model.PropertyEvent)
   */
  public void propertyRemoved(PropertyEvent pe) {
    myTableModel.setAllValues(propertyHolder.getProperties());
    Core.getProjectManager().getActiveProject().setChanged(true);
  }

}
