/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing.properties;

import javax.swing.table.TableCellEditor;

import java.awt.Component;

import javax.swing.JTable;
import javax.swing.JLabel;

import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.EventListenerList;

import java.util.EventObject;

import fdda.model.Property;
import fdda.propertyTypes.PropertyType;
import fdda.utils.Localization;

/**
 * The panel editor delegates the editing to a component 
 * @author Jrg Rdenauer
 */
public class PanelEditor implements TableCellEditor {

  /** listener list */
  private EventListenerList listeners = new EventListenerList();

  /** the property */
  private Property cellProp = null;

  /**
   * Constructor
   */
  public PanelEditor() {
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.table.TableCellEditor#getTableCellEditorComponent(javax.swing.JTable, java.lang.Object, boolean, int, int)
   */
  public Component getTableCellEditorComponent(final JTable table,
    Object value, boolean isSelected, int row, int column) {
      if (table != null) {
        // remove old editor
        table.removeEditor();
        if (table.getColumnName(column).equals(
          ((ObjTableModel) table.getModel()).getColumnName(0))) {
          if (column == 1) { 
            column = 0;
          }
        } 
        else {
          if (column == 0) { 
            column = 1;
          }
        }

        cellProp =
          (Property) table.getModel().getValueAt(row, column);
      }
      java.awt.Component retComp = new JLabel(Localization.getLocalizedString(
        getClass(), "no_editor", "Editor not available"));
      if (cellProp != null) {
        PropertyType typClass = 
          fdda.model.PropertyFactory.createPropertyType(cellProp.getType());
        if (typClass != null) {
          if (!cellProp.isModifiable()) {
            retComp = typClass.getCellRenderer(cellProp, column);
          }
          else if (cellProp.getFacet(Property.MULTI_VALUED) != null) {
            MultiValueEditor mve
              = new MultiValueEditor(cellProp, cellProp.getName());
            mve.addCellEditorListener(new CellEditorListener() {

              public void editingStopped(ChangeEvent e) {
                cellProp = ((MultiValueEditor) e.getSource()).getProperty();
              }

              public void editingCanceled(ChangeEvent e) {
              }

            });
            retComp = mve.getTableCellEditorComponent();
            fireEditingStopped();
          }
          else {
            OiCellEditor editor =
              typClass.getCellEditor(cellProp, cellProp.getValue(0));
            editor.addCellEditorListener(new CellEditorListener() {

              public void editingStopped(ChangeEvent e) {
                cellProp.setValueAt(0,
                  ((OiCellEditor) e.getSource()).getCellEditorValue());
                fireEditingStopped();
                table.removeEditor();
              }

              public void editingCanceled(ChangeEvent e) {
                table.removeEditor();
              }

            });
            retComp = editor.getTableCellEditorComponent();
          }
        }
      }
    return retComp;
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.CellEditor#getCellEditorValue()
   */
  public Object getCellEditorValue() {
    return null;
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.CellEditor#isCellEditable(java.util.EventObject)
   */
  public boolean isCellEditable(EventObject anEvent) {
    return (anEvent != null);
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.CellEditor#shouldSelectCell(java.util.EventObject)
   */
  public boolean shouldSelectCell(EventObject anEvent) {
    return true;
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.CellEditor#stopCellEditing()
   */
  public boolean stopCellEditing() {
    return true;
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.CellEditor#cancelCellEditing()
   */
  public void cancelCellEditing() {
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.CellEditor#addCellEditorListener(javax.swing.event.CellEditorListener)
   */
  public void addCellEditorListener(CellEditorListener l) {
    listeners.add(CellEditorListener.class, l);
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.CellEditor#removeCellEditorListener(javax.swing.event.CellEditorListener)
   */
  public void removeCellEditorListener(CellEditorListener l) {
    listeners.remove(CellEditorListener.class, l);
  }

  /**
   * Returns the property
   * @return the property
   */
  public Property getProperty() {
    return cellProp;
  }

  /**
   * Informs the listeners that editing has stopped.
   */
  protected void fireEditingStopped() {
    ChangeEvent changeEvent = null;
    Object[] listeners = this.listeners.getListenerList();
    for (int i = listeners.length - 2; i >= 0; i -= 2) {
      if (listeners[i] == CellEditorListener.class) {
        if (changeEvent == null) {
          changeEvent = new ChangeEvent(this);
        }
        ((CellEditorListener) listeners[i + 1]).editingStopped(changeEvent);
      }
    }
  }

  /**
   * Informs the listeners that editing has been canceled.
   */
  protected void fireEditingCanceled() {
    ChangeEvent changeEvent = null;
    Object[] listeners = this.listeners.getListenerList();
    for (int i = listeners.length - 2; i >= 0; i -= 2 ) {
      if (listeners[i] == CellEditorListener.class) {
        if (changeEvent == null) {
          changeEvent = new ChangeEvent(this);
        }
        ((CellEditorListener) listeners[i + 1]).editingCanceled(changeEvent);
      }
    }
  }

}