/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing.properties;

import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTable;
import javax.swing.JLabel;

import javax.swing.table.TableCellRenderer;


import fdda.model.Property;
import fdda.propertyTypes.PropertyType;
import fdda.utils.Localization;

/**
 * The panel renderer loads the correct renderer for each property type.
 * @author Jrg Rdenauer
 */
class PanelRenderer extends JPanel
  implements TableCellRenderer {

  /** the short help */
  private JTextArea myShortHelp = null;

  /**
   * Constructor
   * @param ShortHelp the area for the short help messages
   */
  public PanelRenderer(JTextArea ShortHelp) {
    myShortHelp = ShortHelp;
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
   */
  public java.awt.Component getTableCellRendererComponent(JTable table,
    Object value,  boolean isSelected,  boolean hasFocus, int row, int column) {
      if (table.getColumnName(column).equals(
        ((ObjTableModel) table.getModel()).getColumnName(0))) {
        if (column == 1) { 
          column = 0;
        }
      } 
      else {
        if (column == 0) { 
          column = 1;
        }
      }

      Property cellProp = (Property) table.getModel().getValueAt(row, column);
      // set the short help text
      if (table.getSelectedRowCount() == 1 && table.getSelectedRow() == row) {
        String  newHelpText = cellProp.getDescription();
        sethelpText(newHelpText);
      }
      // Load type class and load renderer
      java.awt.Component retComp = new JLabel(Localization.getLocalizedString(
        getClass(), "no_renderer", "No renderer available"));
      if (cellProp != null) {
        PropertyType typeClass = 
          fdda.model.PropertyFactory.createPropertyType(cellProp.getType());
        if (typeClass != null) {
          retComp = typeClass.getCellRenderer(cellProp, column);
        }
      }
      return retComp;
  }

  /**
   * Sets the short help text
   * @param inText the text
   */
  public void sethelpText(String inText) {
    if (myShortHelp != null) {
      myShortHelp.setText(inText);
    }
  }

}