/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing.utils;

import java.awt.*;
import java.awt.event.*;
import javax.swing.ImageIcon;
import javax.swing.JButton;

/**
 * A Button that shows its border only while the mouse is above it, giving it
 * the 'flat' design of recent windows versions.
 * @author Sren Brgmann
 * @version 1.0
 */
public class RollOverButton extends JButton {

  /**
   * Constructor with image, meant for toolbar buttons.
   * @param image image that the button shall show
   * @param tooltip tooltip for the button
   */
  public RollOverButton(ImageIcon image, String tooltip) {
    this.setIcon(image);
    this.setRolloverIcon(image);
    this.setPressedIcon(image);
    this.setRolloverEnabled(true);
    this.setBorderPainted(false);
    this.setToolTipText(tooltip);
    this.setMaximumSize(new Dimension(25,25));
    this.setMinimumSize(new Dimension(25,25));
    this.setPreferredSize(new Dimension(25,25));
    this.addMouseListener(new MouseAdapter() {
      public void mouseEntered(MouseEvent mouseEvent) {
        setBorderPainted(true);
      }
      public void mouseExited(MouseEvent mouseEvent) {
        setBorderPainted(false);
      }
    });
  }

  /**
   * Constructor
   * @param title text on the button
   */
  public RollOverButton(String title) {
    this.setRolloverEnabled(true);
    this.setBorderPainted(false);
    this.setText(title);
    this.addMouseListener(new MouseAdapter() {
      public void mouseEntered(MouseEvent mouseEvent) {
        setBorderPainted(true);
      }
      public void mouseExited(MouseEvent mouseEvent) {
        setBorderPainted(false);
      }
    });
  }
}

