/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing.utils;

import java.awt.*;
import java.awt.event.*;
import javax.swing.ImageIcon;
import javax.swing.JToggleButton;
import javax.swing.border.Border;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * A Toggle button with the 'flat' design of recent windows versions.
 * @author Jrg Rdenauer
 */
public class RollOverToggleButton extends JToggleButton {
  
  /** border of the button */
  private Border myBorder;
  /** stores whether the mouse is above the button */
  private boolean mouseOnButton = false;

  /**
   * Constructor for the button. Meant for toolbars.
   * @param image the image shown on the button
   * @param tooltip the tooltip for the button
   */
  public RollOverToggleButton(ImageIcon image, String tooltip) {
    this.setIcon(image);
    this.setRolloverIcon(image);
    this.setPressedIcon(image);
    this.setRolloverEnabled(true);
    this.setBorderPainted(false);
    this.myBorder = this.getBorder();
    this.setBorder(isSelected() ? null : myBorder);
    this.setToolTipText(tooltip);
    this.setMaximumSize(new Dimension(25,25));
    this.setMinimumSize(new Dimension(25,25));
    this.setPreferredSize(new Dimension(25,25));
    // change can occur programmatically, too     
    this.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        RollOverToggleButton.this.setBorder(isSelected() ? null : myBorder);
        if (!isSelected()) setBorderPainted(mouseOnButton);
      }
    });
    
    this.addMouseListener(new MouseAdapter() {
      public void mouseEntered(MouseEvent mouseEvent) {
        if (!isSelected()) {
          setBorderPainted(true);
        }
        mouseOnButton = true;
      }
      public void mouseExited(MouseEvent mouseEvent) {
        if (!isSelected()) {
          setBorderPainted(false);
        }
        mouseOnButton = false;
      }
    });
  }

}
