/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing.utils;

import javax.swing.filechooser.FileFilter;
import java.io.File;

/**
 * A file filter for JFileChooser. Allows files with a specified ending and 
 * directories.
 * @author Jrg Rdenauer
 */
public class SimpleFileFilter extends FileFilter {

  /** the extension for allowed files */
  private String myAcceptExtension = null;

  /**
   * Constructor
   */
  public SimpleFileFilter() {
  }

  /**
   * Constructor with allowed extension.
   * @param acceptExtension the extension (without the '.' - e.g. "xml")
   */
  public SimpleFileFilter(String acceptExtension) {
    myAcceptExtension = acceptExtension.toLowerCase();
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.filechooser.FileFilter#accept(java.io.File)
   */
  public boolean accept(File inFile) {
    if (myAcceptExtension == null) {
      return true;
    }
    if (inFile.isDirectory()) {
      return true;
    }
    String extension =  getExtension(inFile);
    return myAcceptExtension.equals(extension);
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.filechooser.FileFilter#getDescription()
   */
  public String getDescription() {
    return "*." + myAcceptExtension;
  }

  /**
   * Returns the extension of a file. Returns null if the file does not have
   * an extension.
   * @param f the file
   * @return the extension
   */
  private String getExtension(File f) {
    String ext = null;
    String s = f.getName();
    int i = s.lastIndexOf('.');
    if (i > 0 &&  i < s.length() - 1) {
        ext = s.substring(i + 1).toLowerCase();
    }
    return ext;
  }
  
}
