/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing.utils;

import java.awt.*;
import java.awt.event.*;
import javax.swing.JToolBar;

/**
 * A special toolbar. Its size can be changed by the user with the mouse.
 * @author Sren Brgmann
 * @version 1.0
 */
public class SizeableToolbar extends JToolBar {

  /** Side: Bottom */
  public static final int BOTTOM_SIDE = 2;

  /** Side: Left */
  public static final int LEFT_SIDE = 0;

  /** Side: Right */
  public static final int RIGHT_SIDE = 1;

  /** x coordinate before change */
  private int oldX;

  /** y coordinate before change */
  private int oldY;

  /** Side of the toolbar: left, right or bottom */
  private int side;

  /**
   * Constructor
   */
  public SizeableToolbar() {
    addMouseListener(new MyMouseListener());
    addMouseMotionListener(new MyMouseMotionListener());
    side = LEFT_SIDE;
  }

  /**
   * Sets the side on which the toolbar shall be shown (a constant)
   * @param newSide the new side
   */
  public void setSide(int newSide) {
    side = newSide;
  }

  private class MyMouseListener extends MouseAdapter {
    public void mouseEntered(MouseEvent e) {
      if (side == BOTTOM_SIDE) {
        setCursor(Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR));
      }
      else {
        setCursor(Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR));
      }
    }
    public void mouseExited(MouseEvent e) {
      setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }
  };


  /**
   * Die Variable MyMouseMotionListener speichert den Zeiger auf
   * den MouseMotionListener, der das dragging, also das Ziehen der Maus
   * mit gedrckter linker Maustaste erfasst.
   */
  private class MyMouseMotionListener implements MouseMotionListener {

    public void mouseDragged(MouseEvent e) {
      if (side == BOTTOM_SIDE) {
        setCursor(Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR));
      }
      else {
        setCursor(Cursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR));
      }
      if (oldX == 0)  {
        oldX = e.getX();
        return;
      }
      if (oldY == 0)  {
        oldY = e.getY();
        return;
      }
      if (side == LEFT_SIDE) {
        // make wider?
        if ((oldX < e.getX()) && (e.getX() > getWidth() - 10)) {
          setPreferredSize(new Dimension(e.getX() + 2, getHeight()));
          revalidate();
        }
        // make smaller?
        else if ((oldX > e.getX()) && (e.getX() >= 30)) {
          setPreferredSize(new Dimension(e.getX() + 1, getHeight()));
          revalidate();
        }
        oldX = e.getX();
      }
      else if (side == RIGHT_SIDE) {
        // make smaller?
        if ((oldX < e.getX()) && (e.getX() < getWidth() - 20)) {
          setPreferredSize(new Dimension(getWidth() - e.getX(), getHeight()));
          revalidate();
        }
        // make wider?
        if ((oldX > e.getX()) && (e.getX() < getWidth() - 20)) {
          setPreferredSize(new Dimension(getWidth() - e.getX(), getHeight()));
          revalidate();
        }
        oldX = e.getX();
      }
      else if (side == BOTTOM_SIDE) {
        // make higher?
        if ((oldY < e.getY()) && (e.getY() < getHeight() - 20)) {
          setPreferredSize(new Dimension(getWidth(), getHeight() - e.getY()));
          revalidate();
        }
        // make lower?
        if ((oldY > e.getY()) && (e.getY() < getHeight() - 20)) {
          setPreferredSize(new Dimension(getWidth(), getHeight() - e.getY()));
          revalidate();
        }
        oldY = e.getY();
      }
    }

    public void mouseMoved(MouseEvent e) {}
  };
}
