/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.graphics;

/**
 * Interface for the visitor of components according to the visitor pattern.
 * @author Jrg Rdenauer
 */
public interface ComponentVisitor {

  /**
   * General visit method for not otherwise specified components
   * @param component the component to visit
   * @return depends on subclass
   */
  Object visitComponent(Component component);

  /**
   * Visit a node
   * @param node the node
   * @return depends on subclass
   */
  Object visitNode(Node node);

  /**
   * Visit a line
   * @param line the line
   * @return depends on subclass
   */
  Object visitLine(Line line);

  /**
   * Visit a container
   * @param container the container
   * @return depends on subclass
   */
  Object visitContainer(Container container);
  
  /**
   * Visit a message
   * @param message the message
   * @return depends on subclass
   */
  Object visitMessage(Message message);
}