/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.graphics;

import java.awt.Color;

/**
 * Graphical representation of a message. Important is the moveForward-method.
 * @author Jrg Rdenauer
 */
public interface Message extends Component {
  
  /**
   * Returns the underlying message of the algorithm
   * @return the real message
   */
  fdda.algorithm.Message getRealMessage();
  
  /**
   * Sets the underlying message of the algorithm
   * @param message the real message
   */
  void setRealMessage(fdda.algorithm.Message message);
  
  /**
   * Moves the message forward, towards its destination
   */
  void moveForward();
  
  /**
   * Returns the line on which the message moves
   * @return the line
   */
  Line getLine();
  
  /**
   * Returns whether the message moves in the direction of the line
   * @return whether the message moves in the direction of the line
   */
  boolean doesMoveInLineDirection();
  
  /**
   * Returns the color of the message
   * @return the color
   */
  Color getColor();
  
  /**
   * Sets the color of the message
   * @param color the color
   */
  void setColor(Color color);
}
