/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.graphics;

import fdda.model.PropertyHolder;

import java.awt.Color;

/**
 * Represents the graphical symbol of a node in the model.
 * @author Jrg Rdenauer
 */
public interface Node extends Component, WorkingPart, PropertyHolder {

  /**
   * Returns the x coordinate of the point where lines should start.
   * @return the x coordinate of the line start point
   */
  double getLineStartX();

  /**
   * Returns the y coordinate of the point where lines should start.
   * @return the y coordinate of the line start point
   */
  double getLineStartY();

  /**
   * Sets the main color of the node. May be ignored by implementations.
   * @param color the main color
   */
  void setPrimaryColor(Color color);

  /**
   * Returns the main color of the node.
   * @return the main color.
   */
  Color getPrimaryColor();

  /**
   * Sets the secondary color of the node. May be ignored by implementations.
   * @param color the secondary color.
   */
  void setSecondaryColor(Color color);

  /**
   * Returns the secondary color of the node.
   * @return the secondary color
   */
  Color getSecondaryColor();

  /**
   * Returns the model node which is symbolized by this node.
   * @return the model node
   */
  fdda.model.Node getModelNode();

  /**
   * Sets the model node which is symbolized by this node.
   * @param node the model node
   */
  void setModelNode(fdda.model.Node node);
  
  /**
   * Refreshes the graphics.
   */
  void repaint();

}