/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.graphics.java2d;

import fdda.graphics.ComponentFactory;

/**
 * Implementation of the ComponentFactory for the Java2D graphics.
 * @author Jrg Rdenauer
 */
public class Java2dFactory extends ComponentFactory {

  /** the component selection, a singleton */
  private ComponentSelection selection = null;

  /**
   * Constructor
   */
  public Java2dFactory() {
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentFactory#createNode(double, double, boolean)
   */
  public fdda.graphics.Node createNode(double x, double y, boolean selected) {
    return new Node(x, y, selected);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentFactory#createLine(fdda.graphics.Node, fdda.graphics.Node, boolean, boolean)
   */
  public fdda.graphics.Line createLine(fdda.graphics.Node startNode,
                                       fdda.graphics.Node endNode,
                                       boolean selected, boolean directed) {
    Line line = new Line(startNode.getLineStartX(), startNode.getLineStartY(),
                         endNode.getLineStartX(), endNode.getLineStartY());
    if (selected) line.select(); else line.deselect();
    line.setDirected(directed);
    return line;
  }
  
  /**
   * Creates a line without an end node. This is used while drawing the line (it 
   * must exist without a connection while the user draws it from one node to 
   * the other).
   * @param startNode the start node
   * @param directed whether the line shall be directed
   * @return the newly created line
   */
  public fdda.graphics.Line createLooseLine(fdda.graphics.Node startNode,
                                            boolean directed) {
    Line line = new Line(startNode.getLineStartX(), startNode.getLineStartY(),
                         startNode.getLineStartX(), startNode.getLineStartY());
    line.deselect();
    line.setDirected(directed);
    return line;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentFactory#createContainer()
   */
  public fdda.graphics.Container createContainer() {
    return new Container();
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentFactory#createMessage(fdda.graphics.Line, boolean)
   */
  public fdda.graphics.Message createMessage(fdda.graphics.Line line, 
                                             boolean forward) {
    return new Message((Line) line, forward);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentFactory#getComponentSelection()
   */
  public fdda.graphics.Selection getComponentSelection() {
    // singleton pattern
    if (selection == null) {
      selection = new ComponentSelection();
    }
    return selection;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentFactory#createGraphicalSystem()
   */
  public fdda.graphics.GraphicalSystem createGraphicalSystem() {
    return new GraphicalSystem();
  }

}