/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.model;

/**
 * A connection between two nodes. It can be either directed - allowing messages
 * to pass only in one direction - or undirected. It has a graphical 
 * representation. A connection has some properties and may be not working.
 * 
 * @see fdda.graphics.Line
 * @author Jrg Rdenauer
 */
public interface Connection extends PropertyHolder, WorkingPart {

  /** 
   * the name of the property that determines whether the connection is directed
   */
  String DIRECTED_PROPERTY_NAME = "directed";
  
  /**
   * the name of the property that determines the speed of the connection
   */
  String SPEED_PROPERTY_NAME = "speed";
  
  /**
   * Returns the node where the connection starts. (Even undirected connections
   * have a 'start' and an 'end' node).
   * 
   * @return the start node of the connection
   */
  Node getStartNode();

  /**
   * Returns the node where the connection ends. (Even undirected connections
   * have a 'start' and an 'end' node).
   * 
   * @return the end node of the connection
   */
  Node getEndNode();

  /**
   * Returns the graphical representation of the connection
   * 
   * @return the graphical representation
   */
  fdda.graphics.Line getGraphicalRepresentation();

  /**
   * Returns whether the connection is directed
   * 
   * @return whether the connection is directed
   */
  boolean isDirected();

  /**
   * Specifies whether the connection is directed. Changes the corresponding
   * property.
   * 
   * @param directed whether the connection shall be directed
   */
  void setDirected(boolean directed);
  
  /**
   * Returns the internal number of the connection, which is unique.
   * 
   * @return the internal number
   */
  int getNr();
  
  /**
   * Returns the speed of the connection. The usual speed is 1.0, 2.0 would be 
   * two times as fast, 0.0 would be no speed at all, etc.
   * @return the speed of the connection
   */
  double getSpeed();

}