/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.plugins;

/**
 * Generic interface for all plug-ins. All plug-ins of fdda must implement this
 * interface.
 * @author Jrg Rdenauer
 */
public interface Plugin {

  /** Plug-in type: loads projects */
  int SYSTEM_READER = 1;

  /** Plug-in type: saves projects */
  int SYSTEM_WRITER = 2;

  /** Plug-in type: loads algorithms */
  int ALGORITHM_READER = 3;

  /**
   * Returns the name of the plug-in.
   * @return the name
   */
  String getName();

  /**
   * Returns the type of the plug-in (one of the constants of this interface)
   * @return the type
   */
  int getType();

  /**
   * Returns whether this plug-in can be configured by a call to configure()
   * @return true, iff the plug-in can be configured
   */
  boolean hasOptions();

  /**
   * Tells the plug-in to use the specified dialog component for its 
   * interactions with the user.
   * @param component name of the dialog component (e.g. "swing")
   * @throws ComponentNotSupportedException if the plug-in can't use this
   * component
   */
  void useDialog(String component) throws ComponentNotSupportedException;

  /**
   * This method is used to configure the plug-in by user interaction. It may
   * show a dialog.
   */
  void configure();

}
