/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.plugins;

import fdda.core.projects.Project;

/**
 * Interface for plug-ins that load projects.
 * @author Jrg Rdenauer
 */
public interface SystemReader extends Plugin {

  /**
   * Tells the plug-in that the specified graphics component is used and it 
   * should load its information.
   * @param component identifier of the graphics component (e.g. "java2d")
   * @throws ComponentNotSupportedException if the plug-in doesn't support
   * the loading of the data for this component
   */
  void useGraphic(String component) throws ComponentNotSupportedException;

  /**
   * Interacts with the user to define options for the load operation. E.g a 
   * dialog to select a file could be shown. Either this method or 
   * setSystemLocation must be called before calling loadProject. If the user
   * cancels the operation, the method returns false and loadProject mustn't
   * be called.
   * @return false, iff the user canceled the operation
   */
  boolean performUI();

  /**
   * Sets the storage location of the project. If the location can't be used
   * by this plug-in, returns false.
   * @param location the location
   * @return true iff the location can be used
   */
  boolean setSystemLocation(String location);

  /**
   * Loads a project. PerformUI or setSystemLocation must be called earlier to 
   * set the location. If an error occurs, the appropriate method of the error
   * handler is called. 
   * 
   * To integrate into fdda, the method must set the preference <emph>userRoot
   * </emph>.fdda.lastProjectLocation to the used location.
   * @param errorHandler the errorHandler
   * @param project the project that is filled with the loaded data
   * @throws MissingOptionsException if no location was specified
   */
  void loadProject(ErrorHandler errorHandler, Project project)
    throws MissingOptionsException;

  /**
   * Returns information about the last loaded projects. The size of the array
   * may vary with the plug-in.
   * @return information about the last loaded projects
   */
  ProjectInformation[] getLastLoadedProjectsInformation();
  
}
