/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.plugins;

import fdda.core.projects.Project;

/**
 * Interface for plug-ins that store projects.
 * @author Jrg Rdenauer
 */
public interface SystemWriter extends Plugin {
  
  /**
   * Tells the plug-in that the specified graphics component is used and it 
   * should store its information.
   * @param component identifier of the graphics component (e.g. "java2d")
   * @throws ComponentNotSupportedException if the plug-in doesn't support
   * the storage of the data for this component
   */
  void useGraphic(String component) throws ComponentNotSupportedException;

  /**
   * Interacts with the user to define options for the save operation. E.g., a
   * dialog to select a file name could be shown. Either this operation or
   * setSystemLocation must be called before calling writeProject. The operation
   * returns false if the user canceled the operation; writeProject mustn't be
   * called in this case.
   * @return false, iff the user canceled the saving
   */
  boolean performUI();

  /**
   * Defines where the project shall be saved (alternative to performUI). If the
   * given location (e.g. a file path) can't be used, false is returned.
   * @param location the location
   * @return whether the location can be used.
   */
  boolean setSystemLocation(String location);

  /**
   * Returns information about the last stored projects. The size of the array
   * may vary with each plugin.
   * @return information about last stored projects
   */
  ProjectInformation[] getLastSavedProjectsInformation();


  /**
   * Stores a project in a persistent storage. If an error occurs, the 
   * appropriate method of the error handler is called.
   * 
   * To integrate into fdda, the method should perform several other tasks:
   * <enumeration>
   * <item>Maybe change the project name through the ProjectManager</item>
   * <item>Call project.setChanged(false)</item>
   * <item>Set the preference <emph>userRoot</emph>.fdda.lastProjectLocation to
   * the location of the stored project</item>
   * </enumeration>
   * @param project the project 
   * @param errorHandler the error handler
   * @throws MissingOptionsException if no location was specified
   */
  void writeProject(Project project, ErrorHandler errorHandler)
  throws MissingOptionsException;

}
