/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.plugins.xmlreader;

import fdda.dialog.swing.WindowManager;
import fdda.utils.Localization;

import java.util.ResourceBundle;
import java.util.prefs.Preferences;

import javax.swing.JFileChooser;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.BorderFactory;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Toolkit;

import java.awt.event.ActionEvent;


/**
 * A dialog where the user can select the default path.
 * @author Jrg Rdenauer
 */
class SwingAlgReaderOptions extends JDialog {

  private static String titleS = null;
  private static String chooserTitleS = null;
  private static String chooserButtonS = null;
  private static String chooserMnemonicS = null;
  private static String chooserToolTipS = null;
  private static String okS = null;
  private static String okCharS = null;
  private static String cancelS = null;
  private static String cancelCharS = null;
  private static String dirS = null;

  /** currently selected default directory */
  private String defaulAlgDir;
  /** closed by ok button? */
  private boolean closedByOK;

  private JPanel pnlEnclosing = new JPanel();
  private JButton btnOk = new JButton();
  private JButton btnCancel = new JButton();
  private JLabel lblProjDir = new JLabel();
  private JTextField txtProjDir = new JTextField();
  private JButton btnSelectProjectDir = new JButton();

  /**
   * Constructor
   */
  public SwingAlgReaderOptions() {
    super(WindowManager.getInstance().getMainFrame(), true);
    if (titleS == null) {
      loadLocalizedStrings();
    }
    initLayout();
    this.setTitle(titleS);
    this.pack();
    defaulAlgDir = Preferences.userNodeForPackage(getClass()).get(
      AlgorithmReader.ALGORITHM_PATH_OPTION, 
      AlgorithmReader.ALGORITHM_DEFAULT_PATH);  
  }

  /**
   * Shows the dialog.
   * @return false, iff the user canceled
   */
  public boolean showDialog() {
    closedByOK = false;
    txtProjDir.setText(defaulAlgDir);
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    setLocation(screenSize.width / 2 - 260, screenSize.height / 2 - 110);
    super.show();
    return closedByOK;
  }

  /**
   * Returns the default directory selected by the user.
   * @return the default dir
   */
  public String getDefaultProjDir() {
    return defaulAlgDir;
  }

  /**
   * Loads the localized Strings.
   */
  private static void loadLocalizedStrings() {
    ResourceBundle bundle = 
     Localization.getResourceBundleForPackage(SwingAlgReaderOptions.class);
    titleS = Localization.getLocalizedString(bundle, "algReaderTitleS", 
      "Options of simple XML input");
    chooserTitleS = Localization.getLocalizedString(bundle, "chooserTitleS",
      "Choose directory");
    chooserButtonS = Localization.getLocalizedString(bundle, "chooserButtonS",
      "Choose");
    chooserMnemonicS = Localization.getLocalizedString(bundle, 
      "chooserMnemonicS", "C");
    chooserToolTipS = Localization.getLocalizedString(bundle, "chooserToolTipS",
      "Chooses the selected directory");
    okS = Localization.getLocalizedString(bundle, "okS", "Ok");
    cancelS = Localization.getLocalizedString(bundle, "cancelS", "Cancel");
    okCharS = Localization.getLocalizedString(bundle, "okCharS", "O");
    cancelCharS = Localization.getLocalizedString(bundle, "cancelCharS", "C");
    dirS = Localization.getLocalizedString(bundle, "algReaderDirS", 
      "Default directory for stored algorithms: ");
  }

  /** Lays out the dialog */
  private void initLayout() {
    this.getContentPane().setLayout(new BorderLayout(10, 10));
    pnlEnclosing.setLayout(new GridLayout(0, 1, 5, 5));
    btnOk.setSelected(true);
    btnOk.setText(okS);
    btnOk.setMnemonic(okCharS.charAt(0));
    btnOk.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        closeDialog(e);
      }
    });
    btnCancel.setText(cancelS);
    btnCancel.setMnemonic(cancelCharS.charAt(0));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        cancelDialog(e);
      }
    });
    JPanel buttonsTopPanel = 
      new JPanel(new FlowLayout(FlowLayout.CENTER, 15, 10));
    JPanel buttonsPanel = new JPanel(new GridLayout(0, 2, 15, 10));
    buttonsPanel.add(btnOk);
    buttonsPanel.add(btnCancel);
    buttonsTopPanel.add(buttonsPanel);
    this.getContentPane().add(buttonsTopPanel, BorderLayout.SOUTH);
    JLabel paddingLabel1 = new JLabel("");
    paddingLabel1.setSize(5, 5);
    this.getContentPane().add(paddingLabel1, BorderLayout.NORTH);
    JLabel paddingLabel2 = new JLabel("");
    paddingLabel2.setSize(20, 20);
    this.getContentPane().add(paddingLabel2, BorderLayout.WEST);
    JLabel paddingLabel3 = new JLabel("");
    paddingLabel3.setSize(20, 20);
    this.getContentPane().add(paddingLabel3, BorderLayout.EAST);
    lblProjDir.setText(dirS);
    pnlEnclosing.add(lblProjDir);
    JPanel selectionPanel = new JPanel(new BorderLayout(10, 10));
    selectionPanel.add(txtProjDir, BorderLayout.CENTER);
    btnSelectProjectDir.setText("...");
    btnSelectProjectDir.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        selectProjDir(e);
      }
    });
    selectionPanel.add(btnSelectProjectDir, BorderLayout.EAST);
    pnlEnclosing.add(selectionPanel);
    JPanel controlPanel = new JPanel(new BorderLayout(5, 5));
    controlPanel.setBorder(BorderFactory.createEtchedBorder());
    controlPanel.add(pnlEnclosing, BorderLayout.CENTER);
    JLabel paddingLabel4 = new JLabel("");
    paddingLabel4.setSize(2, 2);
    controlPanel.add(paddingLabel4, BorderLayout.NORTH);
    JLabel paddingLabel5 = new JLabel("");
    paddingLabel5.setSize(2, 2);
    controlPanel.add(paddingLabel5, BorderLayout.EAST);
    JLabel paddingLabel6 = new JLabel("");
    paddingLabel6.setSize(2, 2);
    controlPanel.add(paddingLabel6, BorderLayout.WEST);
    JLabel paddingLabel7 = new JLabel("");
    paddingLabel7.setSize(2, 6);
    controlPanel.add(paddingLabel7, BorderLayout.SOUTH);
    this.getContentPane().add(controlPanel, BorderLayout.CENTER);
  }

  /** closes the dialog if ok is pressed */
  private void closeDialog(ActionEvent e) {
    this.closedByOK = true;
    defaulAlgDir = txtProjDir.getText();
    this.dispose();
  }

  /** closes the dialog if cancel is pressed */
  private void cancelDialog(ActionEvent e) {
    this.closedByOK = false;
    this.dispose();
  }

  /** action handler for default dir selection */
  private void selectProjDir(ActionEvent e) {
    String dir = txtProjDir.getText();
    dir = selectDir(dir);
    txtProjDir.setText(dir);
  }

  /**
   * Selects a directory, using JFileChooser
   * @param dir start directory
   * @return the selected directory
   */
  private String selectDir(String dir) {
    JFileChooser fileChooser = new JFileChooser(dir);
    fileChooser.setApproveButtonMnemonic(chooserMnemonicS.charAt(0));
    fileChooser.setApproveButtonToolTipText(chooserToolTipS);
    fileChooser.setDialogTitle(chooserTitleS);
    fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    int retCode = fileChooser.showDialog(this, chooserButtonS);
    if (retCode == JFileChooser.APPROVE_OPTION) {
      dir = fileChooser.getSelectedFile().getPath();
    }
    return dir;
  }


}
