/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.plugins.xmlwriter;

import java.io.PrintWriter;

import fdda.graphics.java2d.*;

import java.util.Iterator;

/**
 * Writes information about the graphical model (java2d) to the xml file
 * @author Jrg Rdenauer
 */
final class GraphicsWriter {
  
  /** Private constructor to prevent instanciation */
  private GraphicsWriter() {
  }
  
  /**
   * Writes node information.
   * @param node the node
   * @param w the writer
   */
  public static void writeNode(fdda.graphics.Node node, PrintWriter w) {
    Node gNode = (Node) node;
    w.println("    <graphNode x=\"" + gNode.getX() + "\" y=\"" + gNode.getY()
            + "\" />");
  }
  
  /**
   * Writes line information, including points
   * @param line the line
   * @param w the writer
   */
  public static void writeLine(fdda.graphics.Line line, PrintWriter w) {
    Line gLine = (Line) line;
    w.println("    <line x=\"" + gLine.getX() + "\" y=\"" + gLine.getY() 
            + " \" endx=\"" + gLine.getEndX() + "\" endy=\"" + gLine.getEndY()
            + "\" >");
    Iterator it = gLine.getIterator();
    while (it.hasNext()) {
      Object part = it.next();
      if (part instanceof Point) {
        Point point = (Point) part;
        w.println("      <point x=\"" + point.getX() + "\" y=\"" + point.getY()
                + "\" />");
      }
    }
    w.println("    </line>");
  }

}
