/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.plugins.xmlwriter;

import java.io.*;

import fdda.model.*;

import java.util.Iterator;

/**
 * Writes the model to the xml file. Delegates graphics writing to the
 * GraphicsWriter.
 * @see fdda.plugins.xmlwriter.GraphicsWriter
 * @author Jrg Rdenauer
 */
final class ModelWriter {
  
  /** Private constructor to prevent instanciation */
  private ModelWriter() {
  }
  
  /**
   * Writes node information
   * @param node the node
   * @param w the writer
   */
  static void writeNode(Node node, PrintWriter w) {
    w.println("  <node id=\"n" + node.getNr() 
      + "\" label=\"" + node.getLabel() 
      + "\" working=\"" + (node.isWorking() ? "true" : "false") + "\">");
    GraphicsWriter.writeNode(node.getGraphicalRepresentation(), w);    
    Iterator incomings = node.getIncomingConnections().iterator();
    while (incomings.hasNext()) {
      w.println("    <incoming id=\"c" 
        + ((Connection) incomings.next()).getNr() + "\" />");
    }
    Iterator outgoings = node.getOutgoingConnections().iterator();
    while (outgoings.hasNext()) {
      w.println("    <outgoing id=\"c"
        + ((Connection) outgoings.next()).getNr() + "\" />");
    }
    writeProperties(node, w);
    w.println("  </node>");
  }
  
  /**
   * Writes connection information
   * @param connection the connection
   * @param w the writer
   */
  static void writeConnection(Connection connection, PrintWriter w) {
    w.println("  <connection id=\"c" + connection.getNr() 
      + "\" working=\"" + (connection.isWorking() ? "true" : "false") + "\">");
    GraphicsWriter.writeLine(connection.getGraphicalRepresentation(), w);
    writeProperties(connection, w);
    w.println("  </connection>");
  }
  
  /**
   * Writes property information for all properties in a component.
   * @param holder the property holder
   * @param w the writer
   */
  private static void writeProperties(PropertyHolder holder, PrintWriter w) {
    Iterator properties = holder.getProperties();
    while (properties.hasNext()) {
      Property property = (Property) properties.next();
      String displayName = property.getDisplayedName();
      if (displayName == null) displayName = property.getName();
      w.println("    <property name=\"" + property.getName()
        + "\" type=\"" + property.getType()
        + "\" displayName=\"" + displayName
        + "\" description=\"" + property.getDescription() + "\">");
      Iterator facets = property.facets();
      while (facets.hasNext()) {
        String facetName = (String) facets.next();
        w.println("        <facet name=\"" + facetName + "\" value=\""
          + property.getFacet(facetName) + "\" />");
      }
      for (int i = 0; i < property.getCountOfValues(); i++) {
        w.print("        <value>");
        w.print(property.getValue(i));
        w.println("</value>");
      }
      w.println("    </property>");
    }
  }

}
