/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.plugins.xmlwriter;

import fdda.dialog.swing.WindowManager;

import fdda.utils.Localization;
import fdda.utils.Notification;

import java.io.File;

import javax.swing.filechooser.FileFilter;

import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.BorderFactory;
import javax.swing.JFileChooser;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Toolkit;

import java.awt.event.ActionEvent;

import java.util.ResourceBundle;
import java.util.prefs.Preferences;

/**
 * A dialog where the user can select the default path.
 * @author Jrg Rdenauer
 */
class SwingWriterOptions extends JDialog {
  
  private static String fileFilterDesc = null;
  private static String overwriteQuestion = null;
  private static String titleS = null;
  private static String chooserTitleS = null;
  private static String chooserButtonS = null;
  private static String chooserMnemonicS = null;
  private static String chooserToolTipS = null;
  private static String okS = null;
  private static String okCharS = null;
  private static String cancelS = null;
  private static String cancelCharS = null;
  private static String dirS = null;
  private static String yesS = null;
  private static String noS = null;

  /** the default directory */
  private String defaultProjDir;

  /** Closed by ok button? */
  private boolean closedByOK;

  private JPanel pnlEnclosing = new JPanel();
  private JButton btnOk = new JButton();
  private JButton btnCancel = new JButton();
  private JLabel lblProjDir = new JLabel();
  private JTextField txtProjDir = new JTextField();
  private JButton btnSelectProjectDir = new JButton();

  /**
   * Lets the user select the file where the project shall be saved
   * @return the file
   */
  public static File chooseFile() {
    File file = null;
    if (fileFilterDesc == null) {
      loadLocalizedStrings();
    }
    FileFilter ff = new FileFilter() {
      public String getDescription() {
        return fileFilterDesc;
      }
      public boolean accept(File f) {
        return f != null && !f.isDirectory() 
          && f.getName().toLowerCase().endsWith(".xml");
      }
    };    
    JFileChooser fileChooser = new JFileChooser();
    fileChooser.setFileFilter(ff);
    fileChooser.setAcceptAllFileFilterUsed(true);
    fileChooser.setCurrentDirectory(new File(Preferences.userNodeForPackage(
      SwingWriterOptions.class).get(ProjectWriter.PROJECT_PATH_OPTION, 
        ProjectWriter.PROJECT_DEFAULT_PATH)));
    fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
    int retVal = 0;
    int retVal2 = 0;
    JFrame parent =fdda.dialog.swing.WindowManager.getInstance().getMainFrame();
    do {
      retVal2 = 0;
      retVal = fileChooser.showSaveDialog(parent);
      if (retVal == JFileChooser.APPROVE_OPTION) {
        file = fileChooser.getSelectedFile();
        if (file != null) {
          if (!file.getName().endsWith(".xml")) {
            file = new File(file.getPath() + ".xml");
          }
          if (file.exists()) {
            retVal2 = Notification.message(Notification.M_QUESTION,
              Notification.S_XML, Localization.formatMessage(overwriteQuestion, 
              file.getName()), new String[] {yesS, noS, cancelS});
          }
        }
      }
    } while (retVal2 == 1);
    if (retVal != JFileChooser.APPROVE_OPTION || retVal2 != 0) {
      file = null;
    }
    return file;    
  }
  
  /**
   * Constructor
   */
  public SwingWriterOptions() {
    super(WindowManager.getInstance().getMainFrame(), true);
    if (fileFilterDesc == null) {
      loadLocalizedStrings();
    }
    this.setTitle(titleS);
    initLayout();
    this.pack();
    defaultProjDir = Preferences.userNodeForPackage(getClass()).get(
      ProjectWriter.PROJECT_PATH_OPTION, ProjectWriter.PROJECT_DEFAULT_PATH);
  }
  
  /**
   * Shows the dialog.
   * @return false, iff the user canceled
   */
  public boolean showDialog() {
    closedByOK = false;
    txtProjDir.setText(defaultProjDir);
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    setLocation(screenSize.width / 2 - 260, screenSize.height / 2 - 110);
    super.show();
    return closedByOK;
  }
  
  /**
   * Returns the default directory selected by the user.
   * @return the default dir
   */
  public String getDefaultProjDir() {
    return defaultProjDir;
  }
  
  /**
   * Loads the localized Strings.
   */
  private static void loadLocalizedStrings() {
    ResourceBundle bundle = 
     Localization.getResourceBundleForPackage(SwingWriterOptions.class);
    fileFilterDesc = Localization.getLocalizedString(bundle, "fileFilterDesc", 
      "XML files");
    overwriteQuestion = Localization.getLocalizedString(bundle, 
      "overwriteQuestion", "File {0} does exist. Overwrite it?");
    titleS = Localization.getLocalizedString(bundle, "titleS", 
      "Options of simple XML output");
    chooserTitleS = Localization.getLocalizedString(bundle, "chooserTitleS",
      "Choose directory");
    chooserButtonS = Localization.getLocalizedString(bundle, "chooserButtonS",
      "Choose");
    chooserMnemonicS = Localization.getLocalizedString(bundle, 
      "chooserMnemonicS", "C");
    chooserToolTipS = Localization.getLocalizedString(bundle, "chooserToolTipS",
      "Chooses the selected directory");
    okS = Localization.getLocalizedString(bundle, "okS", "Ok");
    cancelS = Localization.getLocalizedString(bundle, "cancelS", "Cancel");
    okCharS = Localization.getLocalizedString(bundle, "okCharS", "O");
    cancelCharS = Localization.getLocalizedString(bundle, "cancelCharS", "C");
    dirS = Localization.getLocalizedString(bundle, "dirS", 
      "Default directory for saved projects: ");
    yesS = Localization.getLocalizedString(bundle, "yesS", "Yes");
    noS = Localization.getLocalizedString(bundle, "noS", "No");
  }

  /** Lays out the dialog */
  private void initLayout() {
    this.getContentPane().setLayout(new BorderLayout(10, 10));
    pnlEnclosing.setLayout(new GridLayout(0, 1, 5, 5));
    btnOk.setSelected(true);
    btnOk.setText(okS);
    btnOk.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        closeDialog(e);
      }
    });
    btnOk.setMnemonic(okCharS.charAt(0));
    btnCancel.setText(cancelS);
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        cancelDialog(e);
      }
    });
    btnCancel.setMnemonic(cancelCharS.charAt(0));
    JPanel buttonsTopPanel = 
      new JPanel(new FlowLayout(FlowLayout.CENTER, 15, 10));
    JPanel buttonsPanel = new JPanel(new GridLayout(0, 2, 15, 10));
    buttonsPanel.add(btnOk);
    buttonsPanel.add(btnCancel);
    buttonsTopPanel.add(buttonsPanel);
    this.getContentPane().add(buttonsTopPanel, BorderLayout.SOUTH);
    JLabel paddingLabel1 = new JLabel("");
    paddingLabel1.setSize(5, 5);
    this.getContentPane().add(paddingLabel1, BorderLayout.NORTH);
    JLabel paddingLabel2 = new JLabel("");
    paddingLabel2.setSize(20, 20);
    this.getContentPane().add(paddingLabel2, BorderLayout.WEST);
    JLabel paddingLabel3 = new JLabel("");
    paddingLabel3.setSize(20, 20);
    this.getContentPane().add(paddingLabel3, BorderLayout.EAST);
    lblProjDir.setText(dirS);
    pnlEnclosing.add(lblProjDir);
    JPanel selectionPanel = new JPanel(new BorderLayout(10, 10));
    selectionPanel.add(txtProjDir, BorderLayout.CENTER);
    btnSelectProjectDir.setText("...");
    btnSelectProjectDir.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        selectProjDir(e);
      }
    });
    selectionPanel.add(btnSelectProjectDir, BorderLayout.EAST);
    pnlEnclosing.add(selectionPanel);
    JPanel controlPanel = new JPanel(new BorderLayout(5, 5));
    controlPanel.setBorder(BorderFactory.createEtchedBorder());
    controlPanel.add(pnlEnclosing, BorderLayout.CENTER);
    JLabel paddingLabel4 = new JLabel("");
    paddingLabel4.setSize(2, 2);
    controlPanel.add(paddingLabel4, BorderLayout.NORTH);
    JLabel paddingLabel5 = new JLabel("");
    paddingLabel5.setSize(2, 2);
    controlPanel.add(paddingLabel5, BorderLayout.EAST);
    JLabel paddingLabel6 = new JLabel("");
    paddingLabel6.setSize(2, 2);
    controlPanel.add(paddingLabel6, BorderLayout.WEST);
    JLabel paddingLabel7 = new JLabel("");
    paddingLabel7.setSize(2, 6);
    controlPanel.add(paddingLabel7, BorderLayout.SOUTH);
    this.getContentPane().add(controlPanel, BorderLayout.CENTER);
  }

  /** closes the dialog if ok is pressed */
  private void closeDialog(ActionEvent e) {
    this.closedByOK = true;
    defaultProjDir = txtProjDir.getText();
    this.dispose();
  }

  /** closes the dialog if cancel is pressed */
  private void cancelDialog(ActionEvent e) {
    this.closedByOK = false;
    this.dispose();
  }

  /** action handler for default dir selection */
  private void selectProjDir(ActionEvent e) {
    String dir = txtProjDir.getText();
    dir = selectDir(dir);
    txtProjDir.setText(dir);
  }
  
  /**
   * Selects a directory, using JFileChooser
   * @param dir start directory
   * @return the selected directory
   */
  private String selectDir(String dir) {
    JFileChooser fileChooser = new JFileChooser(dir);
    fileChooser.setApproveButtonMnemonic(chooserMnemonicS.charAt(0));
    fileChooser.setApproveButtonToolTipText(chooserToolTipS);
    fileChooser.setDialogTitle(chooserTitleS);
    fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    int retCode = fileChooser.showDialog(this, chooserButtonS);
    if (retCode == JFileChooser.APPROVE_OPTION) {
      dir = fileChooser.getSelectedFile().getPath();
    }
    return dir;
  }

}
