/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.propertyTypes;

import fdda.model.Property;

import javax.swing.JLabel;

/**
 * Abstract implementation of PropertyType, superclass for the real property 
 * types. Stores the corresponding type generator and provides a default 
 * implementation of getCellRenderer that uses a JLabel.
 * 
 * @author Jrg Rdenauer
 */
public abstract class AbstractPropertyType implements PropertyType {
  
  /** the type generator */
  protected TypeGenerator generator = null;

  /**
   * Default constructor.
   */
  public AbstractPropertyType () {
  }

  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.PropertyType#getCellRenderer
   * (fdda.model.Property, int)
   * Returns a JLabel as default
   */
  public java.awt.Component getCellRenderer(Property inProperty, int column) {
    // first column: the name of the property
    if (column == 0) {
      String displayName = inProperty.getDisplayedName();
      if (displayName == null) displayName = inProperty.getName();
      if (inProperty.getCountOfValues() < 2) {
        return new JLabel(displayName);
      } 
      else { // multi-valued: append the current number of values
        return new JLabel(displayName 
          + " (" + inProperty.getCountOfValues() + ")");
      }
    }
    // second column: either a string representation of the single value
    // or ... to signify that there are many values. 
    else {
      int coV = inProperty.getCountOfValues();
      if (coV > 1) {
        return new JLabel("...");
      } 
      else if (coV == 1 && inProperty.getValue(0) != null) {
        return new JLabel(inProperty.getValue(0).toString());
      } 
      else { // no values
        return new JLabel("");
      }
    }
  }

}
