/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.propertyTypes;

import java.util.Iterator;
import java.util.Set;

import javax.swing.JComboBox;

import fdda.core.Core;
import fdda.dialog.swing.properties.OiCellEditor;
import fdda.model.Property;

import fdda.propertyTypes.implementations.*;

/**
 * Type for the properties that determine the algorithmic type of a node.
 * @author Jrg Rdenauer
 */
public class NodeTypeType extends AbstractPropertyType {

  /* (non-Javadoc)
   * @see fdda.propertyTypes.PropertyType#getTypeGenerator()
   */
  public TypeGenerator getTypeGenerator() {
    if (generator == null) {
      generator = new NodeTypeGenerator();
    }
    return generator;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.PropertyType#getCellRenderer(fdda.model.Property, int)
   */
  public java.awt.Component getCellRenderer(Property property, int column) {
    if (column == 0) return super.getCellRenderer(property, column);
    JComboBox box = new JComboBox();
    box.setEditable(false);
    Set nodeTypes = 
      Core.getProjectManager().getActiveProject().getAlgorithm().
        getKnownNodeTypes();
    for (Iterator it = nodeTypes.iterator(); it.hasNext(); ) {
      box.addItem(it.next());
    }
    box.setSelectedItem(property.getValue(0));
    return box;
  }

  /* (non-Javadoc)
   * @see fdda.propertyTypes.PropertyType#getCellEditor(fdda.model.Property, java.lang.Object)
   */
  public OiCellEditor getCellEditor(Property property, Object value) {
    return new NodeTypeEditor(property, value, 
      (NodeTypeGenerator) getTypeGenerator());
  }

}
