/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.propertyTypes;

import fdda.model.Property;
import fdda.dialog.swing.properties.OiCellEditor;

import java.awt.Component;

/**
 * The generic interface for the possible types of a property value. Each type 
 * must provide a 'generator' that can clone values and check whether a value is
 * correct. It must also provide an editor that can be used in the property edit
 * panel and a cell renderer that visualizes values when they are not edited.
 * 
 * Property types are loaded dynamically as needed. It is therefore possible to 
 * add new types to the application without recompiling. If a property is 
 * assigned the type 'X', the application will look for a class 
 * 'fdda.propertyTypes.XType' which implements this interface.
 * @author Jrg Rdenauer
 */
public interface PropertyType {
  
  /**
   * Returns a generator for the type.
   * 
   * @return generator
   */
  TypeGenerator getTypeGenerator();
  
  /**
   * Returns an editor for values of this type. The editor should fit into a 
   * table cell of the property edit panel.
   * 
   * @param property the property whose value shall be edited
   * @param value the value that will be edited
   * @return the editor for this value
   */
  OiCellEditor getCellEditor(Property property, Object value);
  
  /**
   * Returns a cell renderer that visualizes values of this type. This cell 
   * renderer is used in the table of the property edit panel.
   * 
   * @param property the property whose value shall be visualized
   * @param column the column in the table - 0 is the left column that normally 
   * just contains the name of the property. 1 is the right column that normally
   * contains the value.
   * 
   * @return renderer for a table cell
   */
  Component getCellRenderer(Property property, int column);
  

}
