/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.propertyTypes.implementations;

import fdda.model.Property;

import fdda.dialog.swing.properties.OiCellEditor;

import javax.swing.JCheckBox;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

/**
 * An Editor for boolean values - uses a check box.
 * @author Jrg Rdenauer
 */
public class BooleanEditor extends OiCellEditor {

  /** the checkbox */
  private JCheckBox checkBox = null;

  /**
   * Default constructor.
   * 
   * @param property the property whose value will be edited
   * @param value the value of the property
   * @param generator the generator for objects of the type
   */
  public BooleanEditor(Property property, Object value, 
                       BooleanGenerator generator) {
    super(property, value, generator);
    this.checkBox = new JCheckBox("", ((Boolean) this.value).booleanValue());
    // when value changes, fire 'editing stopped' so property will be informed
    this.checkBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        BooleanEditor.this.value
          = new Boolean(e.getStateChange() == ItemEvent.SELECTED);
        fireEditingStopped();
      }
    });
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.dialog.swing.properties.OiCellEditor#getValueAsString()
   */
  protected String getValueAsString() {
    return value.toString();
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.dialog.swing.properties.OiCellEditor#getTableCellEditorComponent()
   */
  public java.awt.Component getTableCellEditorComponent() {
    return checkBox;
  }

}