/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.propertyTypes.implementations;

import fdda.model.Property;

import fdda.propertyTypes.AbstractGenerator;
import fdda.propertyTypes.TypeGenerator;
import fdda.propertyTypes.WrongFacetException;

import java.text.ParseException;

/**
 * Generator for boolean values.
 * @author Jrg Rdenauer
 */
public class BooleanGenerator extends AbstractGenerator {

  /**
   * Default constructor.
   */
  public BooleanGenerator() {
  }

  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#cloneValue(java.lang.Object)
   */
  public Object cloneValue(Object valueObject) {
    return Boolean.valueOf(valueObject.toString());
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#parseString(java.lang.String)
   */
  public Object parseString(String string) throws ParseException {
    // don't use Boolean.valueOf because that doesn't throw ParseExceptions
    if (string != null) {
      if (string.toLowerCase().equals("true")) {
        return Boolean.TRUE;
      }
      else if (string.toLowerCase().equals("false")) {
        return Boolean.FALSE;
      }
      else {
        throw new ParseException(string, 0);
      }
    }
    else {
      throw new ParseException("null", 0);
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#correctValue(java.lang.Object, fdda.model.Property)
   */
  public Object correctValue(Object inValue, Property property) 
  throws WrongFacetException {
    if (inValue instanceof Boolean) {
      correctValueRetCode = TypeGenerator.WELL_DONE;
    } 
    else {
      correctValueRetCode = TypeGenerator.WRONG_CLASS_OBJECT;
    }
    return inValue;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#correctFacets(fdda.model.Property)
   */
  public int correctFacets(Property property) {
    return TypeGenerator.WELL_DONE;
  }

}
