/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.propertyTypes.implementations;

import fdda.dialog.swing.properties.OiCellEditor;

import fdda.model.Property;

import javax.swing.JTextField;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


/**
 * Uses a textfield to edit double values.
 * @author Jrg Rdenauer
 */
public class DoubleEditor extends OiCellEditor {

  /** the textfield */
  private JTextField textField = null;

  /**
   * Default constructor.
   * 
   * @param property the property whose value will be edited.
   * @param value the value
   * @param generator generator for doubles
   */
  public DoubleEditor(Property property, Object value, 
                      DoubleGenerator generator) {
    super(property, value, generator);
    if (value == null) {
      this.value = (Double) myProperty.getFacet(Property.DEFAULT_VALUE);
      if (this.value == null) {
        this.value = new Double(0.0);
      }
    }
    else {
      this.value = value;
    }
    textField = new JTextField(this.value.toString());
    textField.addActionListener(new ActionListener() {
     public void actionPerformed(ActionEvent e) {
       fireEditingStopped();
     }
    });
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.dialog.swing.properties.OiCellEditor#getValueAsString()
   */
  protected String getValueAsString() {
    return textField.getText().replace(',', '.');
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.swing.properties.OiCellEditor#getTableCellEditorComponent()
   */
  public java.awt.Component getTableCellEditorComponent() {
    return textField;
  }

}