/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.propertyTypes.implementations;

import java.awt.Component;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import java.util.Iterator;
import java.util.Set;

import javax.swing.JComboBox;

import fdda.core.Core; // to get available choices

import fdda.dialog.swing.properties.OiCellEditor;
import fdda.model.Property;

/**
 * Editor for the type of nodes. Uses a combo box to present the possible 
 * choices.
 * @author Jrg Rdenauer
 */
public class NodeTypeEditor extends OiCellEditor {

  /** the combo box used for editing */
  private JComboBox theBox;

  /**
   * Constructor
   * 
   * @param property the property whose value will be edited
   * @param value the value
   * @param generator the generator for NodeTypes
   */
  public NodeTypeEditor(Property property, Object value, 
                        NodeTypeGenerator generator) {
    super(property, value, generator);
    theBox = new JComboBox();
    theBox.setEditable(false);
    Set nodeTypes = Core.getProjectManager().getActiveProject().
      getAlgorithm().getKnownNodeTypes();
    for (Iterator it = nodeTypes.iterator(); it.hasNext(); ) {
      theBox.addItem(it.next());
    }
    theBox.setSelectedItem(property.getValue(0));
    theBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        NodeTypeEditor.this.value = theBox.getSelectedItem();
        fireEditingStopped();
      }      
    });
  }

  /* (non-Javadoc)
   * @see fdda.dialog.swing.properties.OiCellEditor#getTableCellEditorComponent()
   */
  public Component getTableCellEditorComponent() {
    return theBox;
  }

  /* (non-Javadoc)
   * @see fdda.dialog.swing.properties.OiCellEditor#getValueAsString()
   */
  protected String getValueAsString() {
    return theBox.getSelectedItem().toString();
  }
  
}
