/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.propertyTypes.implementations;

import java.text.ParseException;
import java.util.Set;

// to get the available choices
import fdda.algorithm.Algorithm;
import fdda.core.projects.Project;
import fdda.core.Core;

import fdda.model.Property;

import fdda.propertyTypes.AbstractGenerator;
import fdda.propertyTypes.TypeGenerator;
import fdda.propertyTypes.WrongFacetException;

/**
 * Generator for the node types.
 * @author Jrg Rdenauer
 */
public class NodeTypeGenerator extends AbstractGenerator {

  /** default type for nodes */
  private static final String CLASS_DEFAULT_VALUE =
    Algorithm.getDefaultAlgorithm().getDefaultType();

  /* (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#cloneValue(java.lang.Object)
   */
  public Object cloneValue(Object valueObject) {
    return valueObject.toString();
  }

  /* (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#parseString(java.lang.String)
   */
  public Object parseString(String string) throws ParseException {
    return string;
  }

  /* (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#correctValue(java.lang.Object, fdda.model.Property)
   */
  public Object correctValue(Object inValue, Property property)
  throws WrongFacetException {
    if (inValue instanceof String) {
      correctValueRetCode = correctFacets(property);
      Project project = Core.getProjectManager().getActiveProject();
      if (project == null) {
        return inValue;
      }
      Set nodeTypes = project.getAlgorithm().getKnownNodeTypes();
      if (nodeTypes.contains(inValue)) {
        correctValueRetCode = TypeGenerator.WELL_DONE;
      }
      else {
        Object defaultValue = property.getFacet(Property.DEFAULT_VALUE);
        if (defaultValue == null) {
          inValue = CLASS_DEFAULT_VALUE;
          correctValueRetCode = TypeGenerator.VALUE_SET_TO_CLASS_DEFAULT;
        }
        else {
          inValue = defaultValue;
          correctValueRetCode = TypeGenerator.VALUE_SET_TO_USER_DEFAULT;
        }
      }
    }
    else {
      correctValueRetCode = TypeGenerator.WRONG_CLASS_OBJECT;
    }
    return inValue;  
  }

  /* (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#correctFacets(fdda.model.Property)
   */
  public int correctFacets(Property property) {
    Object defaultValue = property.getFacet(Property.DEFAULT_VALUE);
    if (defaultValue != null) {
      Project project = Core.getProjectManager().getActiveProject();
      if (project == null) return TypeGenerator.WELL_DONE;
      Set nodeTypes = project.getAlgorithm().getKnownNodeTypes();
      if (!nodeTypes.contains(defaultValue)) {
        property.setFacet(Property.DEFAULT_VALUE, CLASS_DEFAULT_VALUE);
        return TypeGenerator.FACET_DEFAULT_NOT_ALLOWED;
      }
    }
    return TypeGenerator.WELL_DONE;
  }
  
}
