/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.propertyTypes.implementations;

import fdda.dialog.swing.properties.OiCellEditor;

import fdda.model.Property;

import javax.swing.JTextField;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * Editor for strings - uses a textfield.
 * @author Jrg Rdenauer
 */
public class StringEditor extends OiCellEditor {

  /** the text field */
  private JTextField textField = null;

  /**
   * Default constructor.
   * @param property the property whose value will be edited
   * @param value the value
   * @param generator generator for string objects
   */
  public StringEditor(Property property, Object value, 
                      StringGenerator generator) {
    super(property, value, generator);
    if (value == null) {
      String defValue = (String) myProperty.getFacet(Property.DEFAULT_VALUE);
      if (defValue == null) {
        defValue = "";
      }
      this.value = defValue;
     }
    textField = new JTextField(this.value.toString());
    textField.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        fireEditingStopped();
      }
    });
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.dialog.swing.properties.OiCellEditor#getValueAsString()
   */
  protected String getValueAsString() {
    return textField.getText();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.swing.properties.OiCellEditor#getTableCellEditorComponent()
   */
  public java.awt.Component getTableCellEditorComponent() {
    return textField;
  }
  
}
